/*
===================================================================
Copyright (C) 2009-2010 Editions JOCATOP / Christophe Gossa
 
This file is part of Jocareso.

Jocareso is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Jocareso is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jocareso.  If not, see http://www.gnu.org/licenses/


Editions JOCATOP
Route de Saint Saturnin
84310 MORIERES
FRANCE

Christophe Gossa
christophe.gossa@jocatop.fr
===================================================================
*/


/*
    2005-12-08 12:37    Christophe Gossa
    Ajout de gethostbyaddr pour les adresses IP fixes, cela permet d'accelerer la connection
    pour les clients "lents"

*/

//Declaration de ma bibliotheque
#include "NDLSocket.h"

//Fonction postes Client
long NDLClientOpenConnection ( NETWORKDATALINK * pNDL, char * pszServerName, long Port, long NagleOverRideLength )
{
    struct hostent          *pHost;
    struct sockaddr_in      ClientSocketAdress;
    long                    ClientSocketAdressLength;
    long                    Retour;
    long                    FlagEnable;

    //Verification de la taille de Nagle
    if (NagleOverRideLength>2048) NagleOverRideLength = 2048;

    //Initialise la socket
    memset ( pNDL, 0, sizeof(NETWORKDATALINK) );
    pNDL->Port              = Port;
    pNDL->Socket            = socketNDL ( AF_INET, SOCK_STREAM, 0 );
    pNDL->MinimalFrameSize  = NagleOverRideLength;

    //La socket est-elle invalide ?
    if (pNDL->Socket == INVALID_SOCKET) return -1;
    
    //On considere que l'utilisateur utilise un IPv4 en mode adresse physique
    if (strisalpha(pszServerName[0]))
    {
        //Recuperation du nom du host serveur
        pHost = gethostbynameNDL (( const char * ) pszServerName );

if (!pHost)
{
    //On ferme la socket que l'on a correctement ouverte
    closesocketNDL ( pNDL->Socket );
    pNDL->Socket = INVALID_SOCKET;
    return -1;
}

ClientSocketAdressLength            = sizeof ( struct sockaddr_in );
memset ( &ClientSocketAdress, 0, ClientSocketAdressLength );
memcpy ( &ClientSocketAdress.sin_addr, pHost->h_addr, pHost->h_length );
ClientSocketAdress.sin_family       = AF_INET;
ClientSocketAdress.sin_port         = htonsNDL (( unsigned short ) pNDL->Port );
    }
    else
    {		
ClientSocketAdressLength            = sizeof ( struct sockaddr_in );
memset ( &ClientSocketAdress, 0, ClientSocketAdressLength );
ClientSocketAdress.sin_addr.s_addr = inet_addrNDL ( pszServerName );
ClientSocketAdress.sin_family       = AF_INET;
ClientSocketAdress.sin_port         = htonsNDL (( unsigned short ) pNDL->Port );
        //pHost = gethostbyaddrNDL ( (char *)&lAddress, 4, AF_INET );
    }

    //Nagle ?
    if (NagleOverRideLength) _NDLSocketConfig ( &pNDL->Socket );

    //Tentative de connexion au serveur
#ifdef _WIN32
    Retour = connectNDL ( pNDL->Socket,
                       ( struct sockaddr * ) &ClientSocketAdress,
                       ClientSocketAdressLength );
#else
    Retour = connectNDL ( pNDL->Socket,
                       ( struct sockaddr * ) &ClientSocketAdress,
                       ClientSocketAdressLength );
#endif
    //Y a-t-il une erreur de connexion ?
    if ( Retour == SOCKET_ERROR )
    {
        //On ferme la socket que l'on a correctement ouverte
        closesocketNDL ( pNDL->Socket );
        pNDL->Socket = INVALID_SOCKET;
        return -1;
    }

    //Rend la socket non-bloquante
    FlagEnable = 1;
    ioctlsocketNDL ( pNDL->Socket, FIONBIO, ( unsigned long * ) &FlagEnable );

    return 0;
}
