/*
===================================================================
Copyright (C) 2009-2010 Editions JOCATOP / Christophe Gossa
 
This file is part of Jocareso.

Jocareso is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Jocareso is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jocareso.  If not, see http://www.gnu.org/licenses/


Editions JOCATOP
Route de Saint Saturnin
84310 MORIERES
FRANCE

Christophe Gossa
christophe.gossa@jocatop.fr
===================================================================
*/


//Declaration de ma bibliotheque
#include "NDLSocket.h"
#include <errno.h>        







long NDLServerOpenConnection            ( NETWORKDATALINK * pNDL, long Port, long NagleOverRideLength )
{
    struct sockaddr_in      ServerSocketAdress;
    long                    ServerSocketAdressLength;
    long                    FlagEnable;
    long                    Retour;
#ifndef _WIN32    
    socklen_t               socklenServerSocketAdressLength;
#endif

    //Verification de la taille de Nagle
    if (NagleOverRideLength>2048) NagleOverRideLength = 2048;

    //Initialise la socket
    memset ( pNDL, 0, sizeof(NETWORKDATALINK) );
    pNDL->Port              = Port;
    pNDL->Socket            = socketNDL ( AF_INET, SOCK_STREAM, 0 );
    pNDL->MinimalFrameSize  = NagleOverRideLength;

    //La socket est-elle invalide ?
    if ( pNDL->Socket == INVALID_SOCKET ) return -1;
    //COM_Printf ( "Socket ouverte" );

    //Rend la socket non-bloquante
    FlagEnable = 1;
    ioctlsocketNDL ( pNDL->Socket, FIONBIO, ( unsigned long * ) &FlagEnable );

    //Remplissage de la structure d'association adresse serveur / socket    
    ServerSocketAdressLength            = sizeof ( struct sockaddr_in );
    memset ( &ServerSocketAdress, 0, ServerSocketAdressLength );
    ServerSocketAdress.sin_family       = AF_INET;                                  //Protocole Internet (TCP, UDP, ...)
    ServerSocketAdress.sin_addr.s_addr  = INADDR_ANY;                               //N'importe quelle adresse entrante
    ServerSocketAdress.sin_port         = htonsNDL (( unsigned short ) pNDL->Port );//Port de communication

    //Specifie les options de la socket : permission pour etre connecte a une adresse deja connectee
    FlagEnable = 1;
    setsockoptNDL ( pNDL->Socket,               //socket a parametrer
                 SOL_SOCKET,                    //Parametres de la socket
                 SO_REUSEADDR,                  //Mode Reutilisation d'adresse
                 ( const char * ) &FlagEnable,  //Flag d'autorisation mis sur 1
                 sizeof ( long ));              //Taille du flag

    //Nagle ?
    if (NagleOverRideLength) _NDLSocketConfig ( &pNDL->Socket );
    
    //Association de l'adresse du serveur a la socket
#ifdef _WIN32
    Retour = bindNDL ( pNDL->Socket, 
                    ( struct sockaddr * ) &ServerSocketAdress, 
                    ServerSocketAdressLength );
#else
    Retour = bindNDL ( pNDL->Socket, 
                    ( struct sockaddr * ) &ServerSocketAdress, 
                    ServerSocketAdressLength );
#endif
    //COM_Printf ( "bind : %d", Retour );
    //Y a-t-il une erreur d'association ?
    if ( Retour == SOCKET_ERROR )
    {
        //On ferme la socket que l'on a correctement ouverte
        closesocketNDL ( pNDL->Socket );
        pNDL->Socket = INVALID_SOCKET;
        return -1;
    }
    //COM_Printf ( "OK" );


    //Recuperation des informations de la socket
#ifdef _WIN32
    Retour = getsocknameNDL ( pNDL->Socket, 
                           ( struct sockaddr * ) &ServerSocketAdress, 
                           ( int * ) &ServerSocketAdressLength );
#else
    socklenServerSocketAdressLength = ServerSocketAdressLength;
    Retour = getsocknameNDL ( pNDL->Socket, 
                           ( struct sockaddr * ) &ServerSocketAdress, 
                           &socklenServerSocketAdressLength );
#endif
    //COM_Printf ( "gethostname : %d", Retour );
    //Y a-t-il une erreur de getsockname ?
    if ( Retour == SOCKET_ERROR )
    {
        //On ferme la socket que l'on a correctement ouverte
        closesocketNDL ( pNDL->Socket );
        pNDL->Socket = INVALID_SOCKET;
        return -1;
    }
    //COM_Printf ( "OK" );

    //Preparation de l'ecoute
    Retour = listenNDL ( pNDL->Socket,                 //Socket � �couter
                      SOCKET_NB_MAX );              //Nombre de connexion max
    //COM_Printf ( "listen : %d", Retour );
    //Y a-t-il une erreur de listen ?
    if ( Retour == SOCKET_ERROR )
    {
        //On ferme la socket que l'on a correctement ouverte
        closesocketNDL ( pNDL->Socket );
        pNDL->Socket = INVALID_SOCKET;
        return -1;
    }
    //COM_Printf ( "OK" );

    

    return 0;
}

void _NDLSocketConfig ( SOCKET *pSocket )
{
    long FlagEnable;

    //Specifie les options de la socket : Nagle !
    FlagEnable = 1;
    setsockoptNDL ( *pSocket,                      //socket � param�trer
                 IPPROTO_TCP,                   //Param�tres de la socket
                 TCP_NODELAY,                   //Mode R�utilisation d'adresse
                 ( const char * ) &FlagEnable,  //Flag d'autorisation mis sur 1
                 sizeof ( int ));               //Taille du flag
    
}

