/*
===================================================================
Copyright (C) 2009-2010 Editions JOCATOP / Christophe Gossa
 
This file is part of Jocareso.

Jocareso is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Jocareso is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jocareso.  If not, see http://www.gnu.org/licenses/


Editions JOCATOP
Route de Saint Saturnin
84310 MORIERES
FRANCE

Christophe Gossa
christophe.gossa@jocatop.fr
===================================================================
*/


//Declaration de ma bibliotheque
#include "NDLSocket.h" 


#define max(x, y) (((x) > (y)) ? (x) : (y))
long NDLServerWaitForConnection         ( NETWORKDATALINK * pNDLServer, NETWORKDATALINK * pNDLClient )
{
    struct sockaddr_in      ClientSocketAdress;
    long                    ClientSocketAdressLength;
#ifndef _WIN32    
    socklen_t               socklenClientSocketAdressLength;
#endif
    long                    FlagEnable;

    //Socket Client
    memset ( pNDLClient, 0, sizeof(NETWORKDATALINK) );
    pNDLClient->MinimalFrameSize = pNDLServer->MinimalFrameSize;

    // Get first incoming frame
    
#ifdef _WIN32
    ClientSocketAdressLength    = sizeof ( struct sockaddr_in );
    pNDLClient->Socket          = acceptNDL ( pNDLServer->Socket, 
                                           ( struct sockaddr * ) &ClientSocketAdress,
                                           ( int * ) &ClientSocketAdressLength );
#else
    socklenClientSocketAdressLength    = sizeof ( struct sockaddr_in );
    pNDLClient->Socket          = acceptNDL ( pNDLServer->Socket, 
                                           ( struct sockaddr * ) &ClientSocketAdress,
                                           &socklenClientSocketAdressLength );
#endif


    //La socket recu est invalide ?
    if ( pNDLClient->Socket == INVALID_SOCKET ) return -4;

    //Nagle ?
    if (pNDLClient->MinimalFrameSize) _NDLSocketConfig ( &pNDLClient->Socket );

    //Rend la socket non-bloquante
    FlagEnable = 1;
    ioctlsocketNDL ( pNDLClient->Socket, FIONBIO, ( unsigned long * ) &FlagEnable );

    // Get peer name
    {
        struct sockaddr_in sa;
        long sal;
#ifndef _WIN32
        socklen_t socklenSAL;
#endif
        memset ( &sa, 0, sizeof(sa) );
        
#ifdef _WIN32
        sal = sizeof(sa);
        if (getpeernameNDL ( pNDLClient->Socket, ( struct sockaddr * )&sa, (int *)&sal )==0)
#else
        socklenSAL = sizeof(sa);
        if (getpeernameNDL ( pNDLClient->Socket, ( struct sockaddr * )&sa, &socklenSAL )==0)
#endif
        {
            strcpy ( pNDLClient->szPeerIP, inet_ntoaNDL ( sa.sin_addr ) );
        }
        else strcpy ( pNDLClient->szPeerIP, "?" );

        strcpy ( pNDLClient->szPeerName, "?" );

    }

    return 0;
}


long NDLServerSelectDataCreate ( NDLSELECTDATA * pSelectData )
{
    if (!pSelectData)
        return -1;
        
    pSelectData->pSet = (fd_set *)KERNEL_ALLOC ( sizeof(fd_set) );
    
    if (!pSelectData->pSet)
        return -1;
        
    memset ( pSelectData->pSet, 0, sizeof(fd_set) );

#ifndef _WIN32
    pSelectData->iMaxDesc = FD_SETSIZE;
#endif
    
    return 0;
}

#ifdef _WIN32
long NDLServerSelectDataAdd ( NDLSELECTDATA * pSelectData, NETWORKDATALINK * pNDL )
{
    if ( (!pSelectData) || (!pNDL) )
        return -1;
        
    if (!pSelectData->pSet)
        return -1;

	if (pSelectData->pSet->fd_count >= FD_SETSIZE)
    {
        pSelectData->pSet = (fd_set *)KERNEL_REALLOC ( pSelectData->pSet, sizeof(fd_set) + sizeof(SOCKET)*(pSelectData->pSet->fd_count-FD_SETSIZE+1) );
    }


    pSelectData->pSet->fd_array[pSelectData->pSet->fd_count] = pNDL->Socket;
    pSelectData->pSet->fd_count++;
        
    return 0;
}
#else
// In Unix, fd_set.count does not exist. Therefore, we need as a parameter the number of the new descriptor
long NDLServerSelectDataAdd ( NDLSELECTDATA * pSelectData, NETWORKDATALINK * pNDL )
{
    long iMax;
    long iOldSize;
    long iNewSize;

    if ( (!pSelectData) || (!pNDL) )
        return -1;
        
    if (!pSelectData->pSet)
        return -1;

    iMax = max(pSelectData->iMaxDesc, (pNDL->Socket) + 1);

    if(pSelectData->iMaxDesc != iMax)
    {
        iNewSize = (iMax / NFDBITS) + (iMax % NFDBITS ? 1 : 0);
		iNewSize *= (NFDBITS >> 3);	// In bytes
		iOldSize = (pSelectData->iMaxDesc / NFDBITS) + (pSelectData->iMaxDesc % NFDBITS ? 1 : 0);
		iOldSize *= (NFDBITS >> 3);	// In bytes

		pSelectData->pSet = (fd_set *) KERNEL_REALLOC(pSelectData->pSet, iNewSize);
		memset(((char*) pSelectData->pSet) + iOldSize, 0, iNewSize - iOldSize);

		pSelectData->iMaxDesc = iMax;
    }

    FD_SET((pNDL->Socket), pSelectData->pSet);

    return 0;
}
#endif

long NDLServerSelectDataDelete ( NDLSELECTDATA * pSelectData )
{
    if (!pSelectData)
        return -1;
        
    if (!pSelectData->pSet)
        return -1;
        
    KERNEL_FREE ( pSelectData->pSet );
    pSelectData->pSet = NULL;
    
    return 0;
}

/*

    TimeOutValue =  0 return immediately
                 >= 1 wait in milliseconds

*/
#ifdef _WIN32
long NDLServerSelectDataExecute ( NDLSELECTDATA * pSelectDataRead, NDLSELECTDATA * pSelectDataWrite, NDLSELECTDATA * pSelectDataExcept, long lTimeOutValue )
{
	long				lIndex;
    struct timeval      tvValue;
    int                 iReturn;
    int                 iMax;
    
    tvValue.tv_sec  = 0;
    tvValue.tv_usec = max(0,lTimeOutValue*1000);
    
    iMax = 0;

    if (pSelectDataRead)
    {
        if (pSelectDataRead->pSet)
        {
            for (lIndex=0;lIndex<pSelectDataRead->pSet->fd_count;lIndex++)
            {
                iMax = max ( iMax, pSelectDataRead->pSet->fd_array[lIndex] );
            }
        }
    }

    if (pSelectDataWrite)
    {
        if (pSelectDataWrite->pSet)
        {
            for (lIndex=0;lIndex<pSelectDataWrite->pSet->fd_count;lIndex++)
            {
                iMax = max ( iMax, pSelectDataWrite->pSet->fd_array[lIndex] );
            }
        }
    }

    if (pSelectDataExcept)
    {
        if (pSelectDataExcept->pSet)
        {
            for (lIndex=0;lIndex<pSelectDataExcept->pSet->fd_count;lIndex++)
            {
                iMax = max ( iMax, pSelectDataExcept->pSet->fd_array[lIndex] );
            }
        }
    }
    
    iMax++;

    iReturn = selectNDL ( 
        iMax, 
        (pSelectDataRead)?(pSelectDataRead->pSet):(NULL), 
        (pSelectDataWrite)?(pSelectDataWrite->pSet):(NULL), 
        (pSelectDataExcept)?(pSelectDataExcept->pSet):(NULL),
        &tvValue );
    
    return iReturn;        
}
#else
long NDLServerSelectDataExecute ( NDLSELECTDATA * pSelectDataRead, NDLSELECTDATA * pSelectDataWrite, NDLSELECTDATA * pSelectDataExcept, long lTimeOutValue )
{
    long			lIndex;
    struct timeval	tvValue;
    long			iReturn;
    long			iMax;
	long			iNewSize;
	long			iOldSize;

    tvValue.tv_sec  = 0;
    tvValue.tv_usec = max(0,lTimeOutValue*1000);

	if(pSelectDataRead)
	{
		iMax = pSelectDataRead->iMaxDesc;

		if(pSelectDataWrite)
			iMax = max(iMax, pSelectDataWrite->iMaxDesc);

		if(pSelectDataExcept)
			iMax = max(iMax, pSelectDataExcept->iMaxDesc);
	}
	else if(pSelectDataWrite)
	{
		iMax = pSelectDataWrite->iMaxDesc;

		if(pSelectDataExcept)
			iMax = max(iMax, pSelectDataWrite->iMaxDesc);
	}
	else if(pSelectDataExcept)
		iMax = pSelectDataExcept->iMaxDesc;
	else
		return 0;

	if(pSelectDataWrite)

	iMax = max(pSelectDataRead->iMaxDesc, max(pSelectDataWrite->iMaxDesc, pSelectDataExcept->iMaxDesc));
	iNewSize = ((iMax / NFDBITS) + (iMax % NFDBITS ? 1 : 0)) * (NFDBITS >> 3);

	if(pSelectDataRead && pSelectDataRead->iMaxDesc != iMax)
	{
		iOldSize = pSelectDataRead->iMaxDesc /NFDBITS + (pSelectDataRead->iMaxDesc % NFDBITS ? 1 : 0);
		iOldSize *= (NFDBITS >> 3);
		pSelectDataRead->pSet = (fd_set *) KERNEL_REALLOC(pSelectDataRead->pSet, iNewSize);
		memset(((char *) pSelectDataRead->pSet), 0, iNewSize - iOldSize);
		pSelectDataRead->iMaxDesc = iMax;
	}

	if(pSelectDataWrite && pSelectDataWrite->iMaxDesc != iMax)
	{
		iOldSize = pSelectDataWrite->iMaxDesc / NFDBITS + (pSelectDataWrite->iMaxDesc % NFDBITS ? 1 : 0);
		iOldSize *= (NFDBITS >> 3);
		pSelectDataWrite->pSet = (fd_set *) KERNEL_REALLOC(pSelectDataWrite->pSet, iNewSize);
		memset(((char *) pSelectDataWrite->pSet), 0, iNewSize - iOldSize);
		pSelectDataWrite->iMaxDesc = iMax;
	}

	if(pSelectDataExcept && pSelectDataExcept->iMaxDesc != iMax)
	{
		iOldSize = pSelectDataExcept->iMaxDesc / NFDBITS + (pSelectDataExcept->iMaxDesc % NFDBITS ? 1 : 0);
		iOldSize *= (NFDBITS >> 3);
		pSelectDataExcept->pSet = (fd_set *) KERNEL_REALLOC(pSelectDataExcept->pSet, iNewSize);
		memset(((char *) pSelectDataExcept->pSet), 0, iNewSize - iOldSize);
		pSelectDataExcept->iMaxDesc = iMax;
	}

    iReturn = selectNDL ( 
        iMax, 
        (pSelectDataRead)?(pSelectDataRead->pSet):(NULL), 
        (pSelectDataWrite)?(pSelectDataWrite->pSet):(NULL), 
        (pSelectDataExcept)?(pSelectDataExcept->pSet):(NULL),
        &tvValue );
    
    return iReturn;
}
#endif

/*
int NDLSelect(int iFDMax, fd_set * pReadSet, fd_set * pWriteSet, fd_set * pExceptSet, const struct timeval * pTv)
{
	int iReturn;
	iReturn = selectNDL(iFDMax, pReadSet, NULL, NULL, pTv);
}
*/
