/*
===================================================================
Copyright (C) 2009-2010 Editions JOCATOP / Christophe Gossa
 
This file is part of Jocareso.

Jocareso is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Jocareso is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jocareso.  If not, see http://www.gnu.org/licenses/


Editions JOCATOP
Route de Saint Saturnin
84310 MORIERES
FRANCE

Christophe Gossa
christophe.gossa@jocatop.fr
===================================================================
*/


#ifdef _WIN32
/*
    Pour Windows uniquement (Visual C++ 2008)
    On supprime tous les warnings en utilisant les fonctions CRT non securisees.
*/
#define _CRT_SECURE_NO_WARNINGS
#endif


#include <stdio.h>
#include <string.h>
#include <memory.h>
#include <sys/types.h>
#include <time.h>

#ifdef _WIN32
    #include <winsock2.h>
    #include <winsock.h>
    #include <windows.h>
#else
/*#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>*/
    #include <arpa/inet.h>
    #include <sys/ioctl.h>
    #include <sys/socket.h>
    #include <netinet/in.h>
    #include <netinet/tcp.h>
    #include <unistd.h>
    #include <netdb.h>
    #include <errno.h>
    #ifndef TD_SOCKET
        typedef int SOCKET;
        #define TD_SOCKET
    #endif
    #define INVALID_SOCKET     -1
    #define SOCKET_ERROR       -1
    //(SOCKET)(~0)
    #define closesocket        close
    #define ioctlsocket        ioctl
    #define socketError        errno
#endif

#include "types.h"

#define MAKEIPADRESS(a,b,c,d)    ((a<<24)|(b<<16)|(c<<8)|(d))
#define MAKEIPADDRESS(a,b,c,d)   ((a<<24)|(b<<16)|(c<<8)|(d))

#ifndef _NDLSocketDefines_h
    #define _NDLSocketDefines_h
    #include "NDLSocketDefines.h"
#endif

void * Z_Alloc ( unsigned long ulByteSize );
void * Z_Realloc ( void * p, unsigned long ulByteSize );
void Z_Free ( void * p );

#define KERNEL_ALLOC(n)         Z_Alloc(n)
#define KERNEL_REALLOC(p,n)     Z_Realloc(p,n)
#define KERNEL_FREE(p)          Z_Free(p)

#include "strex.h"


#ifdef _WIN32

    /*
    ################################################################################################################################################

        Socket Functions

    ################################################################################################################################################
    */
    typedef int                 (WINAPI *ioctlsocketDLLProc)            (SOCKET s, long cmd, u_long FAR *argp);
    typedef struct hostent *    (WINAPI *gethostbynameDLLProc)          (const char FAR * name);
    typedef struct hostent *    (WINAPI *gethostbyaddrDLLProc)          (const char FAR * addr, int len, int type);

    typedef u_short             (WINAPI *htonsDLLProc)                  (u_short hostshort);
	typedef u_long				(WINAPI *htonlDLLProc)					(u_long hostlong);
    typedef char *              (WINAPI *inet_ntoaDLLProc)              (struct in_addr in);
    typedef unsigned long       (WINAPI *inet_addrDLLProc)              (const char FAR * cp);
    typedef int                 (WINAPI *getpeernameDLLProc)            (SOCKET s, struct sockaddr FAR *name,
                                                                                        int FAR * namelen);

    typedef int                 (WINAPI *setsockoptDLLProc)             (SOCKET s, int level, int optname,
                                                                                        const char FAR * optval, int optlen);

    typedef SOCKET              (WINAPI *acceptDLLProc)                 (SOCKET s, struct sockaddr FAR *addr, int FAR *addrlen);

    typedef int                 (WINAPI *bindDLLProc)                   (SOCKET s, const struct sockaddr FAR *addr, int namelen);

    typedef int                 (WINAPI *closesocketDLLProc)            (SOCKET s);

    typedef int                 (WINAPI *connectDLLProc)                (SOCKET s, const struct sockaddr FAR *name, int namelen);
    typedef int                 (WINAPI *getsocknameDLLProc)            (SOCKET s, struct sockaddr FAR *name, int FAR * namelen);
    typedef int                 (WINAPI *listenDLLProc)                 (SOCKET s, int backlog);
    typedef SOCKET              (WINAPI *socketDLLProc)                 (int af, int type, int protocol);

    typedef int                 (WINAPI *recvDLLProc)                   (SOCKET s, char FAR * buf, int len, int flags);
    typedef int                 (WINAPI *sendDLLProc)                   (SOCKET s, const char FAR * buf, int len, int flags);
	typedef int					(WINAPI *selectDLLProc)					(SOCKET s, fd_set * pReadSet, fd_set * pWriteSet, fd_set * pExceptSet, const struct timeval * tv);
	typedef int					(WINAPI *getnameinfoDLLProc)			(const struct sockaddr *sa, /*socklen_t*/ int salen, char *host, DWORD hostlen, char *serv, DWORD servlen, int flags);

    /*
    ################################################################################################################################################
    
        WSA Functions
    
    ################################################################################################################################################
    */

    typedef int                 (WINAPI *WSAStartupDLLProc)                 ( WORD wVersionRequired, LPWSADATA lpWSAData );

    typedef int                 (WINAPI *WSACleanupDLLProc)                 ( void );

    typedef WSAEVENT            (WINAPI *WSACreateEventDLLProc)             ( void );

    typedef int                 (WINAPI *WSAEventSelectDLLProc)             ( SOCKET s, WSAEVENT hEventObject, long lNetworkEvents );

    typedef DWORD               (WINAPI *WSAWaitForMultipleEventsDLLProc)   ( DWORD cEvents, const WSAEVENT FAR * lphEvents,
                                                                                              BOOL fWaitAll, DWORD dwTimeout, BOOL fAlertable );
    typedef BOOL                (WINAPI *WSACloseEventDLLProc)              ( WSAEVENT hEvent );

    typedef int                 (WINAPI *WSAGetLastErrorDLLProc)            ( void );


    extern ioctlsocketDLLProc                  ioctlsocketNDL;
    extern gethostbynameDLLProc                gethostbynameNDL;
    extern gethostbyaddrDLLProc                gethostbyaddrNDL;
    extern htonsDLLProc                        htonsNDL;
	extern htonlDLLProc						   htonlNDL;
    extern inet_ntoaDLLProc                    inet_ntoaNDL;
    extern inet_addrDLLProc                    inet_addrNDL;
    extern getpeernameDLLProc                  getpeernameNDL;
    extern setsockoptDLLProc                   setsockoptNDL;
    extern acceptDLLProc                       acceptNDL;
    extern bindDLLProc                         bindNDL;
    extern closesocketDLLProc                  closesocketNDL;
    extern connectDLLProc                      connectNDL;
    extern getsocknameDLLProc                  getsocknameNDL;
    extern listenDLLProc                       listenNDL;
    extern socketDLLProc                       socketNDL;
    extern recvDLLProc                         recvNDL;
    extern sendDLLProc                         sendNDL;
	extern selectDLLProc					   selectNDL;
	extern getnameinfoDLLProc				   getnameinfoNDL;

    extern WSAStartupDLLProc                   WSAStartupNDL;
    extern WSACleanupDLLProc                   WSACleanupNDL;
    extern WSACreateEventDLLProc               WSACreateEventNDL;
    extern WSAEventSelectDLLProc               WSAEventSelectNDL;
    extern WSAWaitForMultipleEventsDLLProc     WSAWaitForMultipleEventsNDL;
    extern WSACloseEventDLLProc                WSACloseEventNDL;
    extern WSAGetLastErrorDLLProc              WSAGetLastErrorNDL;
#else
    #define ioctlsocketNDL                      ioctlsocket
    #define gethostbynameNDL                    gethostbyname
    #define gethostbyaddrNDL                    gethostbyaddr
    #define htonsNDL                            htons
    #define inet_ntoaNDL                        inet_ntoa
    #define inet_addrNDL                        inet_addr
    #define getpeernameNDL                      getpeername
    #define setsockoptNDL                       setsockopt
    #define acceptNDL                           accept
    #define bindNDL                             bind
    #define closesocketNDL                      closesocket
    #define connectNDL                          connect
    #define getsocknameNDL                      getsockname
    #define listenNDL                           listen
    #define socketNDL                           socket
    #define recvNDL                             recv
    #define sendNDL                             send
	#define	selectNDL							select
/*
    extern WSAStartupDLLProc                   WSAStartupNDL;
    extern WSACleanupDLLProc                   WSACleanupNDL;
    extern WSACreateEventDLLProc               WSACreateEventNDL;
    extern WSAEventSelectDLLProc               WSAEventSelectNDL;
    extern WSAWaitForMultipleEventsDLLProc     WSAWaitForMultipleEventsNDL;
    extern WSACloseEventDLLProc                WSACloseEventNDL;
    extern WSAGetLastErrorDLLProc              WSAGetLastErrorNDL;
*/    

#endif








