/*
===================================================================
Copyright (C) 2009-2010 Editions JOCATOP / Christophe Gossa
 
This file is part of Jocareso.

Jocareso is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Jocareso is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jocareso.  If not, see http://www.gnu.org/licenses/


Editions JOCATOP
Route de Saint Saturnin
84310 MORIERES
FRANCE

Christophe Gossa
christophe.gossa@jocatop.fr
===================================================================
*/


/*
    Fonctionnement du serveur et du client

    Serveur
        1. Creation d'une socket SERVER selon le port selectionne "SOCKET_HOST" (RecvSocket)
        2. Attente d'une connection client
        3. Decodage du nom du client
        4. Creation d'une socket SERVER selon le port et vers le client selectionne "SOCKET_TOCLIENT" (SendSocket)
        5. Attente de validation de connection au client

        

    

    Client
        1. Creation d'une socket CLIENT selon le port et vers le serveur selectionne "SOCKET_TOHOST" (SendSocket
        2. Attente de validation de connection au server
        3. Creation d'une socket CLIENT selon le port selectionne "SOCKET_HOST" (RecvSocket)
        4. 





*/

#define     SOCKET_NB_MAX               16
#define     NAGLE_PADDING_LENGTH        1460

#ifdef _WIN32
#pragma     pack(1)
typedef struct {
    BYTE        IdTrame;                //Identifiant de la trame
    DWORD       DataLength;             //Taille de la trame
    WORD        ZeroPaddingLength;      //Ajout des zero pour remplir une taille de trame minimale (Nagle Algorithm)
} NETWORKDATALINK_HEADER;
#pragma     pack()
#else
#pragma pack (push,1)
typedef struct {
    BYTE        IdTrame;                //Identifiant de la trame
    DWORD       DataLength;             //Taille de la trame
    WORD        ZeroPaddingLength;      //Ajout des zero pour remplir une taille de trame minimale (Nagle Algorithm)
} NETWORKDATALINK_HEADER;
#pragma pack (pop)
#endif

typedef struct {
    SOCKET          Socket;             //Socket
    long            Port;               //Port de communication en emission
    unsigned char   ClientConnected;    //1 si le NDL est connecte (uniquement pour les NDL clients)
    long            MinimalFrameSize;   //Taille minimale d'une trame (pour supplenter l'algo de Nagle)
    INT64           TXBytes;
    INT64           RXBytes;
    INT64           TotalTXBytes;
    INT64           TotalRXBytes;
    unsigned long   Identifiant;        //Identifiant
    char            szPeerName[128];     //Nom de la cible
    char            szPeerIP[64];
    long            lUserData[64];       //64 long pour l'utilisateur (RDBServer utilise le 0 et 1)
} NETWORKDATALINK;

typedef struct {
    fd_set * pSet;
#ifndef _WIN32
	long	 iMaxDesc;
#endif
} NDLSELECTDATA;


//Fonction non exportable
//long _NDLServerOpenConnectionSocket     ( SOCKET * pSocket, long PortNumber, long NagleOverRideLength );
//long _NDLClientOpenConnectionSocket     ( SOCKET * pSocket, char * pszServerName, long PortNumber, long NagleOverRideLength );
//long _NDLServerWaitForConnectionSocket  ( SOCKET * pSocketHost, SOCKET * pSocketConnect );
void _NDLSocketConfig                   ( SOCKET *pSocket );


long NDLServerOpenConnection            ( NETWORKDATALINK * pNDL, long Port, long NagleOverRideLength );
long NDLServerWaitForConnection         ( NETWORKDATALINK * pNDLServer, NETWORKDATALINK * pNDLClient );
long NDLServerCloseConnection           ( NETWORKDATALINK * pNDL );
long NDLServerCloseClientConnection     ( NETWORKDATALINK * pNDL );

long NDLServerSelectDataCreate          ( NDLSELECTDATA * pSelectData );
long NDLServerSelectDataAdd             ( NDLSELECTDATA * pSelectData, NETWORKDATALINK * pNDL );
long NDLServerSelectDataDelete          ( NDLSELECTDATA * pSelectData );
long NDLServerSelectDataExecute         ( NDLSELECTDATA * pSelectDataRead, NDLSELECTDATA * pSelectDataWrite, NDLSELECTDATA * pSelectDataExcept, long lTimeOutValue );

long NDLClientOpenConnection            ( NETWORKDATALINK * pNDL, char * pszServerName, long Port, long NagleOverRideLength );
long NDLClientCloseConnection           ( NETWORKDATALINK * pNDL );

long NDLReceiveData                     ( NETWORKDATALINK * pNDLClient, unsigned char *pIdTrame, unsigned char ** pBuffer, unsigned long * pBufferLength );
long NDLWaitForReceiveData              ( NETWORKDATALINK * pNDLClient );
long NDLSendData                        ( NETWORKDATALINK * pNDLClient, unsigned char IdTrame, unsigned char * pBuffer, unsigned long BufferLength );

unsigned long _NDLCheckSum              ( unsigned char * pBuffer, unsigned long BufferLength );
long _NDLReceiveData                    ( SOCKET Socket, unsigned char * pBufferData, long MaxDataToReceive );
long _NDLSendData                       ( SOCKET Socket, unsigned char * pBufferData, long SendDataLength );

long NDLOpen                            ( void );
long NDLClose                           ( void );
