/*
===================================================================
Copyright (C) 2009-2010 Editions JOCATOP / Christophe Gossa
 
This file is part of Jocareso.

Jocareso is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Jocareso is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jocareso.  If not, see http://www.gnu.org/licenses/


Editions JOCATOP
Route de Saint Saturnin
84310 MORIERES
FRANCE

Christophe Gossa
christophe.gossa@jocatop.fr
===================================================================
*/


    #include "RDB.h"
    #include "jocareso.h"


    #define MB(x) MessageBox(GetFocus(), x,"Debug",MB_OK)

    #define MX(x) { char szTmpAMoi[50];                                     \
                        wsprintf (szTmpAMoi,"Valeur : 0x%08x", x);          \
                        MessageBox(GetFocus(), szTmpAMoi,"Debug",MB_OK);    \
                  } 
    
    #define MI(x) { char szTmpAMoi[50];                                     \
                        wsprintf (szTmpAMoi,"Valeur : %d", x);              \
                        MessageBox(GetFocus(), szTmpAMoi,"Debug",MB_OK);    \
                  }
    #define ML(x) { char szTmpAMoi[50];                                     \
                        wsprintf (szTmpAMoi,"Valeur : %ld", x);             \
                        MessageBox(GetFocus(), szTmpAMoi,"Debug",MB_OK);    \
                  }

    /*
        Les entrees : ENTRY


        ENTRY (ROOT)
            -> ENTRY
                -> ENTRY


        Les entrees pointent sur d'autres entrees...

        Une entree contient


        Une entree
        ----------

            ULONG       Taille de l'entree en octets
            CHAR...     Nom de l'entree
            ULONG       Nombre de sous-entrees presentes

            Les sous-entrees
            ----------------

                CHAR...     Nom de la sous-entree
                UCHAR       Type de la sous-entree
                ULONG       Position du FLAG : "En cours de lecture"
                ULONG       Position du FLAG : "Supervisor All Rights" (0xFFFFFFFF pour vide)
                ULONG       Offset : Si pointe sur une donnee (DATA/FILE) - C'est l'offset relatif aux donnees
                                     Si pointe sur une entree (DIRECTORY) - C'est l'offset relatif aux entrees





    */


    /*
        Une donnee : FILE/DATA/JUMPER

            ULONG       Taille totale de ce champ : Taille de la donnee en octet + entete (10 octets)
            UCHAR       Type de donnee : FILE ou DATA ou HEADER            
            ULONG       Nombre d'octets par element (permet de determiner egalement le nombre d'element)
            RAW         Les donnees elles-memes
    */


    //Insertion d'une entree dans la base d'entrees
    long RDBBase_InsertData ( SRDB ** pRDB, unsigned long OffsetSet, TRDB_DATA * pRDBData );

    //Recupere une entree et/ou sa taille selon l'offset choisi
    long RDBBase_GetData ( SRDB ** pRDB, unsigned long OffsetGet, TRDB_DATA * pRDBData, unsigned long * pSize );

    //Creation d'un flux de donnees e partir d'une entree. On peut recuperer sa taille et/ou son adresse
    long RDBBase_MakeData ( TRDB_DATA * pRDBData, unsigned long * pSize, unsigned char ** pStream );






    long RDBBase_MakeData ( TRDB_DATA * pRDBData, unsigned long * pSize, unsigned char ** pStream )
    {
        _TRDBMemoryStream MS;

        if (pRDBData==NULL) return -1;

        //Creation du flux
        _RDBMemoryStreamOpen ( &MS, NULL );
        _RDBMemoryStreamAdd_ULONG  ( &MS, 0 );
        _RDBMemoryStreamAdd_UCHAR  ( &MS, pRDBData->DataType );
        _RDBMemoryStreamAdd_ULONG  ( &MS, pRDBData->SizeOfOneElement );
        _RDBMemoryStreamAdd_RAW    ( &MS, pRDBData->pData, (pRDBData->SizeOfOneElement) * (pRDBData->CountOfElements)  );

        //On fait quelque chose qui n'est normalement pas prevu : modifier une donnee du flux
        *((unsigned long *)MS.pBuffer) = (unsigned long)MS.BufferLength;

        //Remplissage des donnees de sortie
        if (pSize)      *pSize = MS.BufferLength;
        if (pStream)    *pStream = MS.pBuffer;
        else            _RDBMemoryStreamClose ( &MS );
        return 0;
    }


    long RDBBase_GetData ( SRDB ** pRDB, unsigned long OffsetGet, TRDB_DATA * pRDBData, unsigned long * pSize )
    {
        _TRDBMemoryStream  MS;
        unsigned long   Size;
        unsigned long   SizeOfOneElement;
        unsigned char   DataType;
        unsigned char   *pBuffer;
        unsigned long   BufferLength;


        //Teste si le pointeur est bon ?
        if (pRDB==NULL) return -1;

        //Lecture du flux entrant
        _RDBMemoryStreamOpen           ( &MS, (unsigned char *)&((*pRDB)->pDataBase[OffsetGet]) );
        _RDBMemoryStreamGet_ULONG      ( &MS, &Size );
        _RDBMemoryStreamGet_UCHAR      ( &MS, &DataType );
        _RDBMemoryStreamGet_ULONG      ( &MS, &SizeOfOneElement );

        //Remplissage des donnees de sortie
        if (pRDBData)
        {
            if ((!Size)&&(!DataType)&&(!SizeOfOneElement))
            {
                return -1;
                /*
                pRDBData->DataType              =   DataType;
                pRDBData->SizeOfOneElement      =   1;
                pRDBData->CountOfElements       =   0;
                pRDBData->pData                 =   NULL;
                */
            }
            else
            {           
                //Recuperation du tampon
                _RDBMemoryStreamGet_RAW ( &MS, &pBuffer, &BufferLength );

                pRDBData->DataType              =   DataType;
                pRDBData->SizeOfOneElement      =   SizeOfOneElement;
                pRDBData->CountOfElements       =   BufferLength / SizeOfOneElement;
                pRDBData->pData                 =   pBuffer;
            }
        }

        if (pSize) *pSize = (unsigned long)Size;
        return 0;
    }

    long RDBBase_GetDataOffset ( SRDB ** pRDB, unsigned long OffsetGet, unsigned long * pDataOffset )
    {
        _TRDBMemoryStream MS;
        unsigned long   Size;
        unsigned long   SizeOfOneElement;
        unsigned char   DataType;
        unsigned long   BufferLength;


        //Teste si le pointeur est bon ?
        if (pRDB==NULL) return -1;

        //Lecture du flux entrant
        _RDBMemoryStreamOpen           ( &MS, (unsigned char *)&((*pRDB)->pDataBase[OffsetGet]) );
        _RDBMemoryStreamGet_ULONG      ( &MS, &Size );
        _RDBMemoryStreamGet_UCHAR      ( &MS, &DataType );
        _RDBMemoryStreamGet_ULONG      ( &MS, &SizeOfOneElement );
        
        if ((!Size)&&(!DataType)&&(!SizeOfOneElement))
            return -1;
        

        //Recuperation du tampon
        _RDBMemoryStreamGet_RAW ( &MS, NULL, &BufferLength );

        //Remplissage des donnees de sortie
        if (pDataOffset) *pDataOffset = OffsetGet + MS.OffsetRead - BufferLength;
        return 0;
    }

    long RDBBase_InsertData ( SRDB ** pRDB, unsigned long OffsetSet, TRDB_DATA * pRDBData )
    {
        unsigned char * pBuffer;
        unsigned char * pDB;
        unsigned long SizeBuffer;
        
        if ((pRDB==NULL)||(pRDBData==NULL)) return -1;

        //Creation du bloc de donnees e inserer
        RDBBase_MakeData ( pRDBData, &SizeBuffer, &pBuffer );

        //Reallocation de la zone memoire
        pDB = (unsigned char *)KERNEL_REALLOC ( (*pRDB)->pDataBase, (*pRDB)->DataBaseLength + SizeBuffer );
        if (!pDB)
        {
            pDB = (unsigned char *)KERNEL_ALLOC ( (*pRDB)->DataBaseLength + SizeBuffer );
            if (!pDB)
            {

                //MB ( "Erreur grave d'allocation memoire : Barriere 128Mo. Merci de contactez les Editions JOCATOP" );
                return -2;
                KERNEL_FREE ( pBuffer );
            }

            memcpy ( pDB, (*pRDB)->pDataBase, (*pRDB)->DataBaseLength );
            KERNEL_FREE ( (*pRDB)->pDataBase );
            (*pRDB)->pDataBase = pDB;
        }
        else 
            (*pRDB)->pDataBase = pDB;

        //Deplacement de la zone memoire pour l'insertion (si necessaire)
        if ( ((*pRDB)->DataBaseLength - OffsetSet) > 0 )
        {
            long i;

            for (i=(((*pRDB)->DataBaseLength) - OffsetSet)-1;i>=0;i--)
            {
                (*pRDB)->pDataBase[OffsetSet+SizeBuffer+i] = (*pRDB)->pDataBase[OffsetSet+i];
            }

            /*
            memcpy ( &(*pRDB)->pDataBase[OffsetSet+SizeBuffer],
                     &(*pRDB)->pDataBase[OffsetSet],
                     (*pRDB)->DataBaseLength - OffsetSet );
            */
        }

        //Copie des donnees
        memcpy ( &(*pRDB)->pDataBase[OffsetSet], pBuffer, SizeBuffer );

        //Liberation du tampon des donnees
        KERNEL_FREE ( pBuffer );

        (*pRDB)->DataBaseLength += SizeBuffer;

        return 0;
    }


    long RDBBase_DefragData ( SRDB ** pRDB, unsigned long OffsetKilledData, unsigned long LengthKilledData )
    {
        TRDB_ENTRY      RDBEntry;
        unsigned long   Offset;
        unsigned long   Size;
        unsigned char   Fin;

        //Deplacement de bloc
        if (((*pRDB)->DataBaseLength - (OffsetKilledData+LengthKilledData))>0)
            memcpy ( &(*pRDB)->pDataBase[OffsetKilledData],
                     &(*pRDB)->pDataBase[OffsetKilledData+LengthKilledData],
                     (*pRDB)->DataBaseLength - (OffsetKilledData+LengthKilledData) );

        //Reallocation
        (*pRDB)->DataBaseLength -= LengthKilledData;
        (*pRDB)->pDataBase = (unsigned char *)KERNEL_REALLOC ( (*pRDB)->pDataBase, (*pRDB)->DataBaseLength );

        //Modification des entrees
        Offset  = 0;
        Fin     = FALSE;
        do {
            RDBEntry_GetData ( pRDB, Offset, &RDBEntry, &Size );
            if ((RDBEntry.OffsetContent!=0xFFFFFFFF)&&(RDBEntry.OffsetContent>OffsetKilledData))
                RDBEntry_SetEntryOffsetContent ( pRDB, Offset, RDBEntry.OffsetContent - LengthKilledData );

            Offset += Size;
            if (Offset>=(*pRDB)->EntryDataBaseLength) Fin = TRUE;
        } while (Fin==FALSE);

        return 0;
    }






    























    

    /*  *****************************************************************
        Retour :    0   Pas d'erreur
                    -1  Erreur, pointeur d'entree invalide
                    -2  Pas assez de memoire
    */
    long                RDBCreate               ( SRDB ** pRDB, char * pszBaseName );

    
    /*  *****************************************************************
        Retour :    0   Pas d'erreur
                    -1  Erreur, pointeur d'entree invalide
    */
    long                RDBDelete               ( SRDB ** pRDB );


    /*  *****************************************************************
        Retour :    0   Pas d'erreur
                    -1  Erreur, pointeur d'entree invalide
                    -3  Impossible de creer le fichier
                    -4  Impossible d'enregistrer les donnees
                    -5  Impossible de fermer le fichier
    */
    long                RDBSave                 ( SRDB ** pRDB );

    /*  *****************************************************************
        Retour :    0   Pas d'erreur
                    -1  Erreur, pointeur d'entree invalide
                    -5  Impossible de fermer le fichier
                    -6  Impossible de lire les donnees
                    -7  Impossible d'ouvrir le fichier
    */
    long                RDBLoad                 ( SRDB ** pRDB, char * pszBaseName );


    long                RDBCreate               ( SRDB ** pRDB, char * pszBaseName )
    {
        TRDB_ENTRY          RDBEntry;        

        if (_RDBAccessServerMode==_TRDBACCESS_CLIENT)
            return RDBAccess_ClientCreate ( pRDB, pszBaseName );

        //Teste si le pointeur est bon ?
        if ( (pRDB==NULL) || (pszBaseName==NULL) ) return -1;

        //Allocation de la base de depart
        (*pRDB) = (SRDB *) KERNEL_ALLOC ( sizeof ( SRDB ) );
        if ((*pRDB)==NULL) return -2;

        //Initialise l'entete
        (*pRDB)->Header.size            = sizeof(SRDB_HEADER);
        (*pRDB)->Header.ID              = SRDB_HEADER_ID;
        (*pRDB)->Header.Version         = 1;
        (*pRDB)->Header.Compression     = 0;
        (*pRDB)->Header.Defragged       = 0;
        strcpy ( (*pRDB)->Header.szRDBName, pszBaseName );

        //Initialise la valeur ID du superviseur
        (*pRDB)->SupervisorOwnerID = 0xFFFFFFFF;

        //Initialise les donnees avec un index d'entree
        (*pRDB)->EntryDataBaseLength    = 0;
        (*pRDB)->pEntryDataBase         = NULL;
        (*pRDB)->DataBaseLength         = 0;
        (*pRDB)->pDataBase              = NULL;

        //Ajoute le point d'entree
        RDBEntry.OffsetContent              = 0xFFFFFFFF;
        RDBEntry.FLAGS_ReadsCount           = 0;
        RDBEntry.FLAGS_Supervisor           = 0xFFFFFFFF;
        RDBEntry.UseLastFullPathName        = 0;
        RDBEntry.Identifiant                = RDB_ID_DIRECTORY;
        RDBEntry.FullPathName[0]            = '\\';
        RDBEntry.FullPathName[1]            = 0;

        //Ajout de l'entree
        RDBEntry_InsertData ( pRDB, 0, &RDBEntry );

        //Pas d'erreur
        return 0;
    }


    long    RDBDelete ( SRDB ** pRDB )
    {
        if (_RDBAccessServerMode==_TRDBACCESS_CLIENT)
            return RDBAccess_ClientDelete ( pRDB );

        //Teste si le pointeur est bon ?
        if ( (pRDB==NULL) || ((*pRDB)==NULL) ) return -1;

        //Fermeture des zones memoires
        KERNEL_FREE ( (*pRDB)->pDataBase );            //La zone de donnees
        KERNEL_FREE ( (*pRDB)->pEntryDataBase );       //La base d'entrees
        KERNEL_FREE ( (*pRDB) );                       //L'entete
        (*pRDB) = NULL;

        //Pas d'erreur
        return 0;
    }

    long    RDBSave   ( SRDB ** pRDB )
    {
        FILE  *pFile;
        DWORD WriteLenght;
        char pszFileName[1024];

        if (_RDBAccessServerMode==_TRDBACCESS_CLIENT)
            return RDBAccess_ClientSave ( pRDB );

        //Teste si le pointeur est bon ?
        if ( (pRDB==NULL) || ((*pRDB)==NULL) ) return -1;

        //Creation du fichier
        sprintf ( pszFileName, "%s/%s", SYS_GetInstallPath ( ), (*pRDB)->Header.szRDBName );
        pFile = fopen ( pszFileName, "wb" );

        //Impossible d'ouvrir le fichier
        if (!pFile) return -3;

        //Ecriture de l'entete
        WriteLenght = (DWORD) fwrite ( &((*pRDB)->Header), 1, (*pRDB)->Header.size, pFile );
        if (WriteLenght!=(*pRDB)->Header.size)
        {
            fclose ( pFile );
            return -4;
        }

        //Ecriture des entrees
        fwrite ( &((*pRDB)->EntryDataBaseLength), 1, sizeof(unsigned long), pFile );
        WriteLenght = (DWORD) fwrite ( (*pRDB)->pEntryDataBase, 1, (*pRDB)->EntryDataBaseLength, pFile );
        if (WriteLenght!=(*pRDB)->EntryDataBaseLength)
        {
            fclose ( pFile );
            return -4;
        }

        //Ecriture des donnees
        fwrite ( &((*pRDB)->DataBaseLength), 1, sizeof(unsigned long), pFile );
        WriteLenght = (DWORD) fwrite ( (*pRDB)->pDataBase, 1, (*pRDB)->DataBaseLength, pFile );
        if (WriteLenght!=(*pRDB)->DataBaseLength)
        {
            fclose ( pFile );
            return -4;
        }

        //Fermeture
        fclose ( pFile );
        
        //Pas d'erreur
        return 0;
    }


    long                RDBLoadAsRaw ( char * pszBaseName, unsigned char ** pData, long * plDataLen )
    {
        FILE            *pFile;
        char            pszFileName[1024];
        long            lSize;

        //Ouverture du fichier
        if (pszBaseName[0]=='@') strcpy ( pszFileName, pszBaseName+1 );
        else                     sprintf ( pszFileName, "%s/%s", SYS_GetInstallPath ( ), pszBaseName );
        pFile = fopen ( pszFileName, "rb" );

        //Impossible d'ouvrir le fichier
        if (!pFile)
            return -1;

        //Lecture de la taille du fichier
        fseek ( pFile, 0, SEEK_END );
        lSize = ftell ( pFile );
        fseek ( pFile, 0, SEEK_SET );

        //Allocation
        (*pData) = (unsigned char *)KERNEL_ALLOC ( lSize );

        //Lecture
        fread ( *pData, lSize, 1, pFile );

        //Fermeture
        fclose ( pFile );

        *plDataLen = lSize;

        return 0;
    }

    long                RDBLoadAsRawPacket ( char * pszBaseName, long * plFileLen, long lBlockByteOffset, long lBlockSizeMax, unsigned char * pBlockData, long * plBlockDataLen )
    {
        FILE            *pFile;
        char            pszFileName[1024];
        long            lSize;

        //Ouverture du fichier
        if (pszBaseName[0]=='@') strcpy ( pszFileName, pszBaseName+1 );
        else                     sprintf ( pszFileName, "%s/%s", SYS_GetInstallPath ( ), pszBaseName );
        pFile = fopen ( pszFileName, "rb" );

        //Impossible d'ouvrir le fichier
        if (!pFile)
            return -1;

        //Lecture de la taille du fichier
        fseek ( pFile, 0, SEEK_END );
        lSize = ftell ( pFile );
        fseek ( pFile, lBlockByteOffset, SEEK_SET );
        
        //Lecture
        *plBlockDataLen = fread ( pBlockData, 1, lBlockSizeMax, pFile );

        //Fermeture
        fclose ( pFile );

        *plFileLen = lSize;

        return 0;
    }

    long                RDBSaveFromRaw ( char * pszBaseName, unsigned char * pData, long lDataLen )
    {
        FILE            *pFile;
        char            pszFileName[1024];

        //Ouverture du fichier
        if (pszBaseName[0]=='@') strcpy ( pszFileName, pszBaseName+1 );
        else                     sprintf ( pszFileName, "%s/%s", SYS_GetInstallPath ( ), pszBaseName );
        pFile = fopen ( pszFileName, "wb" );

        //Impossible d'ouvrir le fichier
        if (!pFile)
            return -1;

        //Ecriture
        fwrite ( pData, lDataLen, 1, pFile );

        //Fermeture
        fclose ( pFile );

        return 0;
    }
    
    long                RDBSaveFromRawPacket ( char * pszBaseName, long lBlockByteOffset, unsigned char * pBlockData, long lBlockDataLen )
    {
        FILE            *pFile;
        char            pszFileName[1024];

        //Ouverture du fichier
        if (pszBaseName[0]=='@') strcpy ( pszFileName, pszBaseName+1 );
        else                     sprintf ( pszFileName, "%s/%s", SYS_GetInstallPath ( ), pszBaseName );
        
        if (lBlockByteOffset) pFile = fopen ( pszFileName, "r+b" );
        else pFile = fopen ( pszFileName, "wb" );

        //Impossible d'ouvrir le fichier
        if (!pFile)
            return -1;
        
        //Ecriture
        fseek ( pFile, lBlockByteOffset, SEEK_SET );
        fwrite ( pBlockData, lBlockDataLen, 1, pFile );

        //Fermeture
        fclose ( pFile );
        
        return 0;
    }

    long                RDBLoad                 ( SRDB ** pRDB, char * pszBaseName )
    {
        FILE            *pFile;
        DWORD           ReadLenght;
        char            pszFileName[1024];

        if (_RDBAccessServerMode==_TRDBACCESS_CLIENT)
            return RDBAccess_ClientLoad ( pRDB, pszBaseName );

        //Teste si le pointeur est bon ?
        if ( (pRDB==NULL) || (pszBaseName==NULL) ) return -1;

        //Allocation de la base de depart
        (*pRDB) = (SRDB *) KERNEL_ALLOC ( sizeof ( SRDB ) );
        if ((*pRDB)==NULL) return -2;

        //Ouverture du fichier
        sprintf ( pszFileName, "%s/%s", SYS_GetInstallPath ( ), pszBaseName );
        pFile = fopen ( pszFileName, "rb" );

        //Impossible d'ouvrir le fichier
        if (!pFile)
        {
            KERNEL_FREE ( (*pRDB) );
            *pRDB = NULL;
            return -7;
        }

        //Lecture de l'entete
        ReadLenght = (DWORD) fread ( &((*pRDB)->Header), 1, sizeof(SRDB_HEADER), pFile );        

        //Verification de la taille de l'entete
        if ( ((*pRDB)->Header.size!=sizeof(SRDB_HEADER)) || (ReadLenght!=sizeof(SRDB_HEADER)) )
        {
            KERNEL_FREE ( (*pRDB) );
            *pRDB = NULL;
            fclose ( pFile );
            return -6;
        }

        //*********************************************************************************************************************

        //Lecture de la taille des entrees
        ReadLenght = (DWORD) fread ( (void *)&((*pRDB)->EntryDataBaseLength), 1, sizeof(unsigned long), pFile );
        if (ReadLenght!=sizeof(unsigned long))
        {
            KERNEL_FREE ( (*pRDB) );
            *pRDB = NULL;
            fclose ( pFile );
            return -6;
        }

        //Allocation de l'espace memoire pour les donnees
        (*pRDB)->pEntryDataBase = (unsigned char *) KERNEL_ALLOC ( (*pRDB)->EntryDataBaseLength );

        //Lecture des donnees
        ReadLenght = (DWORD) fread ( (void *)(*pRDB)->pEntryDataBase, 1, (*pRDB)->EntryDataBaseLength, pFile );
        if ((*pRDB)->EntryDataBaseLength!=ReadLenght)
        {
            KERNEL_FREE ( (*pRDB)->pEntryDataBase );
            KERNEL_FREE ( (*pRDB) );
            *pRDB = NULL;
            fclose ( pFile );
            return -6;
        }

        //*********************************************************************************************************************

        //Lecture de la taille des donnees
        ReadLenght = (DWORD) fread ( (void *)&((*pRDB)->DataBaseLength), 1, sizeof(unsigned long), pFile );
        if (ReadLenght!=sizeof(unsigned long))
        {
            KERNEL_FREE ( (*pRDB)->pEntryDataBase );
            KERNEL_FREE ( (*pRDB) );
            *pRDB = NULL;
            fclose ( pFile );
            return -6;
        }

        //Allocation de l'espace memoire pour les donnees
        (*pRDB)->pDataBase = (unsigned char *) KERNEL_ALLOC ( (*pRDB)->DataBaseLength );

        //Lecture des donnees
        ReadLenght = (DWORD) fread ( (void *)(*pRDB)->pDataBase, 1, (*pRDB)->DataBaseLength, pFile );
        if ((*pRDB)->DataBaseLength!=ReadLenght)
        {
            KERNEL_FREE ( (*pRDB)->pDataBase );
            KERNEL_FREE ( (*pRDB)->pEntryDataBase );
            KERNEL_FREE ( (*pRDB) );
            *pRDB = NULL;
            fclose ( pFile );
            return -6;
        }

            
        //Fermeture
        fclose ( pFile );

        //Initialise la valeur ID du superviseur
        (*pRDB)->SupervisorOwnerID = 0xFFFFFFFF;                    //Cette valeur sera toujours incremente par le serveur

        //Liberation des Flags
        RDBEntry_KillAllFlags ( pRDB );

        //Pas d'erreur
        return 0;
    }


    long    _RDBGetDataSizeID ( unsigned char Identifiant )
    {
        long Size;
        switch (Identifiant)
        {   
            //Pour un ROOT ou un DIRECTORY, il n'y a pas d'autres donnees, donc rien e faire de plus
            case RDB_ID_ROOT:
            case RDB_ID_DIRECTORY:
            case RDB_ID_FILE:
            case RDB_ID_DATA:
            case RDB_ID_JUMPER:
            default:
                Size = 0;
                break;

            case RDB_ID_UCHAR:      Size = 1;   break;
            case RDB_ID_USHORT:     Size = 2;   break;
            case RDB_ID_ULONG:      Size = 4;   break;
            case RDB_ID_UWIDE:      Size = 8;   break;
            case RDB_ID_CHAR:       Size = 1;   break;
            case RDB_ID_SHORT:      Size = 2;   break;
            case RDB_ID_LONG:       Size = 4;   break;
            case RDB_ID_WIDE:       Size = 8;   break;
            case RDB_ID_FLOAT:      Size = 4;   break;
            case RDB_ID_DOUBLE:     Size = 8;   break;
        }

        return Size;
    }


    long                RDBFindFileList         ( SRDB_HEADER ** pEntryData, long * plEntryCount )
    {
        long            lEntryCount;
        long            lForgetEntry;
        char            *pszFileName;
        long            lFileSize;
        char            szSearchPath[256];
#ifdef _WIN32
        long            lSearchHandle;
        struct _finddata_t fdEntry;
#else
        DIR			    *pDirHandle;
        struct dirent   *pDirEntry;
#endif                

        lEntryCount = 0;
        *pEntryData = NULL;
        *plEntryCount = 0;

        sprintf ( szSearchPath, "%s/", SYS_GetInstallPath ( ) );

        
#ifndef _WIN32
        //Linux only
        strcat ( szSearchPath, "." );    
        pDirHandle = opendir ( szSearchPath );
        if (!pDirHandle) return -1;
#endif

        do {
            lForgetEntry = 0;

#ifdef _WIN32
            if (lEntryCount==0)
            {
                strcat ( szSearchPath, "*" );
                lSearchHandle = (long)_findfirst ( szSearchPath, &fdEntry );
                if (lSearchHandle==-1) return -1;
            }
            else
            {
                if (_findnext(lSearchHandle,&fdEntry))
                {
                    _findclose ( lSearchHandle );
                    return 0;
                }
            }
            pszFileName = fdEntry.name;
            lFileSize = fdEntry.size;
#else
            pDirEntry = readdir ( pDirHandle );
            if (!pDirEntry)
            {
                closedir ( pDirHandle );
                return (lEntryCount)?(0):(-1);
            }
            pszFileName = pDirEntry->d_name;
            lFileSize = 0;
#endif

            if (!stristr(pszFileName,".RDB"))
                lForgetEntry = 1;

            if (!lForgetEntry)
            {
                (*plEntryCount)++;
                (*pEntryData) = (SRDB_HEADER *)KERNEL_REALLOC ( (*pEntryData), sizeof(SRDB_HEADER) * (*plEntryCount) );

                strcpy ( (*pEntryData)[(*plEntryCount)-1].szRDBName, pszFileName );
                (*pEntryData)[(*plEntryCount)-1].size = lFileSize;
            }
            lEntryCount++;
        } while (lEntryCount<256);

#ifdef _WIN32
        _findclose ( lSearchHandle );
#else
        closedir ( pDirHandle );
#endif

        return 0;
    }
