/*
===================================================================
Copyright (C) 2009-2010 Editions JOCATOP / Christophe Gossa
 
This file is part of Jocareso.

Jocareso is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Jocareso is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jocareso.  If not, see http://www.gnu.org/licenses/


Editions JOCATOP
Route de Saint Saturnin
84310 MORIERES
FRANCE

Christophe Gossa
christophe.gossa@jocatop.fr
===================================================================
*/


    
    #include "RDB.h"
    #include "sleep.h"
    #include "screensaver.h"
    #include "tick.h"
    #include "thread.h"
    #include "criticalsection.h"
    

    /*

        Protocole de communication

            Les messages Client=>Serveur

            ID  0x00
            P   -




            ID      Sens    Parametres      Description

            0x00    C=>S    -               Connexion a un serveur
                                            On attend en retour un acquittement

            0x80    S=>C    -               Acquittement de connexion (message provenant du serveur)



            0x01    C=>S    -               Deconnexion d'un serveur
                                            Pas d'acquitement necessaire



            0x02    C=>S    szBaseName      Initialise l'acces a une base
                                            On attend en retour confirmation de l'acces a une base

            0x82    S=>C    szBaseName      Acquittement de l'acces a une base
                                            Ce message signale que la base a bien ete ouverte


            0x03    C=>S    szBaseName      Informe le serveur que nous n'avons plus besoin de cette base
                                            Acquitement inutile


            0x04    C=>S    szBaseName      STARTEXCLUSIVE
                                            Attente de l'acquitement STARTEXCLUSIVE

            0x05    C=>S    szBaseName      STOPEXCLUSIVE
                                            Attente de l'acquitement STOPEXCLUSIVE

            0x06    C=>S    szBaseName      STARTREAD
                                            Attente de l'acquitement STARTREAD

            0x07    C=>S    szBaseName      STOPREAD
                                            Attente de l'acquitement STOPREAD


            0x04    C=>S    EDBLength       Demande au serveur la longueur de la EntryDataBaseLength
                                            On attend en retour la longueur demandee

            0x83    S=>C    EDBLength       Renvoi la valeur EntryDataBaseLength au client



        Les acquittements (Serveur->Client)

    */


    /*

        en mode reseau, la connection/deconnection a une base est automatiquement geree par le programme du serveur
        Seule les fonctions Connect et Deconnect permette d'etablir et de fermer la connexion


    */


    NETWORKDATALINK     _NDLClient;
    SCISTATE            _sciStateClient;
    extern long         _RDBAccessServerCompatibilityMode;



    
    long    RDBAccess_Client_GetCurrentCompatibilityMode ( void )
    {
        return _RDBAccessServerCompatibilityMode;
    }


    long    RDBAccess_Client_ConnectToServer ( char * pszServerName, long Port, long NagleOverRideLength, char * pszPassword, long lTryAdminMode, long lEnableCompatibilityMode )
    {
        long Retour;
        unsigned char bIdTrameConnect;
        unsigned char IdTrame;
        unsigned char *pIO;
        unsigned long Length;
        unsigned long InitTime;//,ElapsedTime;
        //char szPassword[256];
        long lUserIndex;
        long lRetour;
        long lConnectAccepted;
        long lDoLoop;
        long lOpendConnection = 0;

        //On veut se connecter en mode        
        //if (lTryAdminMode) //&&(!pszPassword))
        //    return -1;
            
        lUserIndex = 0;
        lDoLoop = 1;
        
        
        //bIdTrameConnect = (pszPassword)?(RDB_ACCESS_CLIENT_CONNECTEX):(RDB_ACCESS_CLIENT_CONNECT);
        
        do {
            lRetour             = 0;
            lConnectAccepted    = 0;
            
            if (lTryAdminMode)
            {
                bIdTrameConnect = RDB_ACCESS_CLIENT_CONNECTEX;
                lDoLoop = 0;
            }
            else
            {
                if ((lUserIndex>=0)&&(lUserIndex<64))
                    bIdTrameConnect = RDB_ACCESS_CLIENT_CONNECTEX;
                else
                    bIdTrameConnect = RDB_ACCESS_CLIENT_CONNECT;
            }
        
            //Ouverture de la connection
            if (lOpendConnection)
            {
                NDLSendData ( &_NDLClient, RDB_ACCESS_CLIENT_DECONNECT, NULL, 0 );
                NDLClientCloseConnection ( &_NDLClient );
                NDLClose ( );
                lOpendConnection = 0;
            }
            
            NDLOpen ( );
            Retour = NDLClientOpenConnection ( &_NDLClient, pszServerName, Port, NagleOverRideLength );
            if (Retour!=0)
            {
                NDLClose ( );
                return -1;
            }
            lOpendConnection = 1;

            //Envoi de la trame de connexion
            Retour = NDLSendData ( &_NDLClient, bIdTrameConnect, NULL, 0 );
            if (Retour!=0)
            {
                NDLClientCloseConnection ( &_NDLClient );
                NDLClose ( );
                lRetour = -1;
            }
            else
            {
                //Attend le retour (max 5 secondes)
                InitTime = TickGetCount2 ( ) + 5000;
                do {
                    //Initialise les variables
                    IdTrame = 0;
                    pIO = NULL;

                    //Reception de la trame
                    Retour = NDLReceiveData ( &_NDLClient, &IdTrame, &pIO, &Length );

                    //On libere eventuellement le tampon de reception
                    if (pIO)
                    {
                        if (Length==4) _NDLClient.Identifiant = *((unsigned long *)pIO);
                        KERNEL_FREE ( pIO );
                    }

                    //Est-ce le bon entete            
                    if (Retour==0)
                    {
                        if (IdTrame!=(bIdTrameConnect|RDB_ACCESS_SERVER_ACQ_BACK_MASK))
                        {
                            Retour = 0;
                            lRetour = -1;
                        }
                    }

                    //Mesure le temps ecoule
                    if ((Retour!=0)&&(TickGetCount2 ( )>=InitTime))
                    {
                        NDLClientCloseConnection ( &_NDLClient );
                        NDLClose ( );
                        //return -1;
                        //Retour = 0;
                        //lRetour = -1;
                        //lDoLoop = 0;
                        return -1;
                    }

                } while (Retour!=0);
                
                if ((Retour==0)&&(lRetour==0))
                {
                    if (bIdTrameConnect==RDB_ACCESS_CLIENT_CONNECTEX)
                    {
                        if ((!pszPassword)&&(lTryAdminMode))
                            lConnectAccepted = 1;
                        else
                        {
                            if (RDBAccess_ClientEnterAuthLink ( pszPassword, (lTryAdminMode)?(-1):(lUserIndex) )==0)
                                lConnectAccepted = 1;
                            else lOpendConnection = 0;
                        }                        
                        //
                        //Start full connection protocol
                    } 
                    else
                    {
                        lConnectAccepted = 1;
                        _RDBAccessServerCompatibilityMode = 1;
                    }
                    
                    if (lConnectAccepted)
                        lDoLoop = 0;
                }
            }
            
            if (!lConnectAccepted)
            {
                lUserIndex+=1;
                if (lEnableCompatibilityMode)
                {
                    if (lUserIndex>=65)
                        lDoLoop = 0;
                }
                else
                {
                    if (lUserIndex>=64)
                        lDoLoop = 0;
                }
            }
            
        } while (lDoLoop);                
        
        if ((!lConnectAccepted)&&(lOpendConnection))
        {
            NDLSendData ( &_NDLClient, RDB_ACCESS_CLIENT_DECONNECT, NULL, 0 );
            NDLClientCloseConnection ( &_NDLClient );
            NDLClose ( );
        }
        
        
        if (!lConnectAccepted)
            return -1;
            
        
                
                
        
        //We need to find the jocareso protocol...
        //We send the frame "RDB_ACCESS_CLIENT_TransmitName"
        //  If we receive ID_RDBAccessNetwork_Accept
        //      it's jocareso 0.2.x
        //  If we receive RDB_ACCESS_CLIENT_TransmitName
        //      it's jocareso 0.3.x
        //  otherwise it's not jocareso !
        
        //On va dans un premier temps envoyé une demande 
        
        //On va tenter de se connecter en mode sécurisé (jocareso 0.3.0 et plus)
        
        
        //ServerClientInterface_ConvertPasswordToTextSHA512 ( szPassword, pszPassword );
        //ServerClientInterface_MakePKey ( &_sciStateClient, szPassword );
        

        RDBAccess_ClientSendComputerName ( );

        return 0;
    }

    long    RDBAccess_Client_DeConnectToServer ( void )
    {
        long Retour;
        unsigned char IdTrame;
        unsigned char *pIO;
        unsigned long Length;
        unsigned long InitTime,ElapsedTime;

        //Message de deconnection
        Retour = NDLSendData ( &_NDLClient, RDB_ACCESS_CLIENT_DECONNECT, NULL, 0 );
        if (Retour!=0) return -1;

        //Att
        InitTime = TickGetCount2 ( );
        do {
            //Initialise les variables
            IdTrame = 0;
            pIO = NULL;

            //Reception de la trame
            Retour = NDLReceiveData ( &_NDLClient, &IdTrame, &pIO, &Length );

            //On libere eventuellement le tampon de reception
            if (pIO) KERNEL_FREE ( pIO );

            //Est-ce le bon entete
            if (IdTrame!=(RDB_ACCESS_CLIENT_DECONNECT|RDB_ACCESS_SERVER_ACQ_BACK_MASK))
                Retour = -1;

            //Mesure le temps ecoule
            ElapsedTime = TickGetCount2 ( ) - InitTime;
            if ((Retour!=0)&&(ElapsedTime>=5000))
                return -1;

        } while (Retour!=0);

        //Ferme la connection
        NDLClientCloseConnection ( &_NDLClient );

        //Fermeture de la NDL
        NDLClose ( );

        return 0;
    }


    long                RDBAccess_ClientCreate               ( SRDB ** pRDB, char * pszBaseName )
    {

        //Teste si le pointeur est bon ?
        if ( (pRDB==NULL) || (pszBaseName==NULL) ) return -1;

        //Allocation de la base de depart
        (*pRDB) = (SRDB *) KERNEL_ALLOC ( sizeof ( SRDB ) );
        if ((*pRDB)==NULL) return -2;

        //Initialise l'entete
        (*pRDB)->Header.size            = sizeof(SRDB_HEADER);
        (*pRDB)->Header.ID              = SRDB_HEADER_ID;
        (*pRDB)->Header.Version         = 1;
        (*pRDB)->Header.Compression     = 0;
        (*pRDB)->Header.Defragged       = 0;
        strcpy ( (*pRDB)->Header.szRDBName, pszBaseName );

        //Initialise la valeur ID du superviseur
        (*pRDB)->SupervisorOwnerID      = _NDLClient.Identifiant;                    //Cette valeur sera toujours incremente par le serveur

        //Initialise les donnees avec un index d'entree
        (*pRDB)->EntryDataBaseLength    = 0;
        (*pRDB)->pEntryDataBase         = NULL;
        (*pRDB)->DataBaseLength         = 0;
        (*pRDB)->pDataBase              = NULL;

        //Pas d'erreur
        return 0;
    }


    long    RDBAccess_ClientDelete ( SRDB ** pRDB )
    {
        //Teste si le pointeur est bon ?
        if ( (pRDB==NULL) || ((*pRDB)==NULL) ) return -1;

        //Fermeture des zones memoires
        KERNEL_FREE ( (*pRDB) );                       //L'ent�te
        (*pRDB) = NULL;

        //Pas d'erreur
        return 0;
    }

    long    RDBAccess_ClientSave   ( SRDB ** pRDB )
    {
        return 0;
    }



    long                RDBAccess_ClientLoad                 ( SRDB ** pRDB, char * pszBaseName )
    {
        return RDBAccess_ClientCreate ( pRDB, pszBaseName );
    }


