/*
===================================================================
Copyright (C) 2009-2010 Editions JOCATOP / Christophe Gossa
 
This file is part of Jocareso.

Jocareso is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Jocareso is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jocareso.  If not, see http://www.gnu.org/licenses/


Editions JOCATOP
Route de Saint Saturnin
84310 MORIERES
FRANCE

Christophe Gossa
christophe.gossa@jocatop.fr
===================================================================
*/


    #include "RDB.h"

    long                RDBWriteData_long            ( SRDB ** pRDB, char * pszRootDirectoryName, char * pszDataName, long Data )
    {
        return RDBWriteData ( pRDB, pszRootDirectoryName, pszDataName, (unsigned char *)&Data, sizeof(long) );
    }


    long                RDBMakeData_long        ( SRDB ** pRDB, char * pszRootDirectoryName, char * pszDataName, long Data )
    {
        return RDBMakeData ( pRDB, pszRootDirectoryName, pszDataName, (unsigned char *)&Data, sizeof(Data), 1 );
    }

    long                RDBReadData_long        ( SRDB ** pRDB, char * pszRootDirectoryName, char * pszDataName )
    {
        long            *pData, Data;
        unsigned long   SizeOfOneElement;
        unsigned long   CountOfElements;
        long            Retour;

        Retour = RDBReadData ( pRDB, pszRootDirectoryName, pszDataName, (unsigned char **)&pData, &SizeOfOneElement, &CountOfElements );

        if (!Retour)
        {
            if (SizeOfOneElement==sizeof(long))
            {
                memcpy ( &Data, pData, sizeof(long) );
                KERNEL_FREE ( pData );
                return Data;
            }
        }
        return 0xFFFFFFFF;
    }


    long                RDBWriteData_String ( SRDB ** pRDB, char * pszRootDirectoryName, char * pszDataName, char * pszData )
    {
        return RDBWriteData ( pRDB, pszRootDirectoryName, pszDataName, (unsigned char *)pszData, (unsigned long)strlen(pszData)+1 );
    }

    long                RDBMakeData_String ( SRDB ** pRDB, char * pszRootDirectoryName, char * pszDataName, char * pszData )
    {
        return RDBMakeDataEx ( pRDB, pszRootDirectoryName, pszDataName, (unsigned char *)&pszData, (unsigned long)strlen(pszData)+1, 1, RDB_ID_STRING );
    }

    long                RDBReadData_String        ( SRDB ** pRDB, char * pszRootDirectoryName, char * pszDataName, char ** pszData )
    {
        char            *psz;
        unsigned long   SizeOfOneElement;
        unsigned long   CountOfElements;
        long            lRetour;

        psz = NULL;

        lRetour = RDBReadData ( pRDB, pszRootDirectoryName, pszDataName, (unsigned char **)&psz, &SizeOfOneElement, &CountOfElements );

        if (!lRetour)
        {
            if (!psz) return -1;
            if ((SizeOfOneElement!=1)||(!CountOfElements))
            {
                KERNEL_FREE ( psz );
                return -1;
            }
            if (psz[CountOfElements-1]!=0)
            {
                KERNEL_FREE ( psz );
                return -1;
            }

            *pszData = psz;
            return 0;
        }
        return -1;
    }




    long                RDBMakeDataEx           ( SRDB ** pRDB, char * pszRootDirectoryName, char * pszDataName, unsigned char * pData, unsigned long SizeOfOneElement, unsigned long CountOfElements, long lIdentifiant )
    {
        unsigned long   OffsetEntry; 
        unsigned long   Size;
        TRDB_ENTRY      RDBEntry;
        long            Retour;
        TRDB_DATA       RDBData;

        //Teste si le pointeur est bon ?
        if (pRDB==NULL) return -1;

        //Est-on en mode client ******************************************************************************************
        if (_RDBAccessServerMode == _TRDBACCESS_CLIENT)
            return RDBAccess_ClientMakeData ( pRDB, pszRootDirectoryName, pszDataName, pData, SizeOfOneElement, CountOfElements );

        //Recherche si le fichier n'existe pas deja
        Retour = RDBEntry_SearchName ( pRDB, pszRootDirectoryName, pszDataName, SEARCHMODE_DATA|SEARCHMODE_STARTENTRY, &OffsetEntry );
        if (Retour== 0) return -2;                       
        if (Retour==-3) return -3;

        //Lecture des donnees du repertoire racine
        Retour = RDBEntry_SearchName ( pRDB, pszRootDirectoryName, NULL, SEARCHMODE_DIRECTORY|SEARCHMODE_STARTENTRY, &OffsetEntry );
        if (Retour!=0) return -4;
        Retour = RDBEntry_GetData ( pRDB, OffsetEntry, &RDBEntry, &Size );
        if (Retour!=0) return -4;

        //Verification des droits en mode superviseur
        if ((RDBEntry.FLAGS_Supervisor!=0xFFFFFFFF)&&(RDBEntry.FLAGS_Supervisor!=((*pRDB)->SupervisorOwnerID))) return -4;

        //Creation de l'entree du fichier
        RDBEntry.FLAGS_ReadsCount       = 0;
        RDBEntry.FLAGS_Supervisor       = 0xFFFFFFFF;
        RDBEntry.Identifiant            = (unsigned char)lIdentifiant;
        RDBEntry.OffsetContent          = (*pRDB)->DataBaseLength;
        RDBEntry.UseLastFullPathName    = 1;
        sprintf ( RDBEntry.FullPathName, "%s", pszDataName );

        //Insertion de l'entree
        Retour = RDBEntry_InsertData ( pRDB, OffsetEntry+Size, &RDBEntry );
        if (Retour!=0) return -5;

        //Creation des donnees du fichier
        RDBData.DataType                = (unsigned char)lIdentifiant;
        RDBData.CountOfElements         = CountOfElements;
        RDBData.SizeOfOneElement        = SizeOfOneElement;
        RDBData.pData                   = pData;

        //Insertion des donnees
        return RDBBase_InsertData ( pRDB, (*pRDB)->DataBaseLength, &RDBData );
    }

    long                RDBMakeData             ( SRDB ** pRDB, char * pszRootDirectoryName, char * pszDataName, unsigned char * pData, unsigned long SizeOfOneElement, unsigned long CountOfElements )
    {
        return RDBMakeDataEx ( pRDB, pszRootDirectoryName, pszDataName, pData, SizeOfOneElement, CountOfElements, RDB_ID_DATA );
    }


    long                RDBKillData             ( SRDB ** pRDB, char * pszRootDirectoryName, char * pszDataName )
    {
        unsigned long   OffsetEntry; 
        unsigned long   Size;
        TRDB_ENTRY      RDBEntry;
        long            Retour;

        //Teste si le pointeur est bon ?
        if (pRDB==NULL) return -1;

        //Est-on en mode client ******************************************************************************************
        if (_RDBAccessServerMode == _TRDBACCESS_CLIENT)
            return RDBAccess_ClientKillData ( pRDB, pszRootDirectoryName, pszDataName );

        //Lecture des donnees du repertoire racine
        Retour = RDBEntry_SearchName ( pRDB, pszRootDirectoryName, NULL, SEARCHMODE_DIRECTORY|SEARCHMODE_STARTENTRY, &OffsetEntry );
        if (Retour!=0) return -4;
        Retour = RDBEntry_GetData ( pRDB, OffsetEntry, &RDBEntry, &Size );
        if (Retour!=0) return -4;

        //Verification des droits en mode superviseur et Read Off
        if ((RDBEntry.FLAGS_Supervisor!=0xFFFFFFFF)&&(RDBEntry.FLAGS_Supervisor!=((*pRDB)->SupervisorOwnerID))) return -4;
        if (RDBEntry.FLAGS_ReadsCount>0) return -4;

        //Recherche si le fichier existe bien
        Retour = RDBEntry_SearchName ( pRDB, pszRootDirectoryName, pszDataName, SEARCHMODE_DATA|SEARCHMODE_STARTENTRY, &OffsetEntry );
        if (Retour!=0) return -2;

        //Lecture des donnees du fichier
        Retour = RDBEntry_GetData ( pRDB, OffsetEntry, &RDBEntry, &Size );
        if (Retour!=0) return -4;

        //Verification des droits en mode superviseur et Read Off
        if ((RDBEntry.FLAGS_Supervisor!=0xFFFFFFFF)&&(RDBEntry.FLAGS_Supervisor!=((*pRDB)->SupervisorOwnerID))) return -4;
        if (RDBEntry.FLAGS_ReadsCount>0) return -4;

        //Suppression de l'entree
        Retour = RDBEntry_KillData ( pRDB, OffsetEntry );
        if (Retour!=0) return -2;

        //Changement du type de donnee stockee par un jumper (operation plus ou moins licite) @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
        (*pRDB)->pDataBase[RDBEntry.OffsetContent+4] = RDB_ID_JUMPER;

        //Je prefere cette operation
        RDBBase_GetData ( pRDB, RDBEntry.OffsetContent, NULL, &Size );
        RDBBase_DefragData ( pRDB, RDBEntry.OffsetContent, Size );
        return 0;
    }

    long                RDBReadData             ( SRDB ** pRDB, char * pszRootDirectoryName, char * pszDataName,
                                                  unsigned char ** pData, unsigned long *pSizeOfOneElement, unsigned long *pCountOfElements )
    {
        unsigned long   OffsetEntry; 
        unsigned long   Size;
        TRDB_DATA       RDBData;
        TRDB_ENTRY      RDBEntry;
        long            Retour;

        //Teste si le pointeur est bon ?
        if ((pRDB==NULL)||(pData==NULL)||(pSizeOfOneElement==NULL)||(pCountOfElements==NULL))
            return -1;

        //Est-on en mode client ******************************************************************************************
        if (_RDBAccessServerMode == _TRDBACCESS_CLIENT)
            return RDBAccess_ClientReadData ( pRDB, pszRootDirectoryName, pszDataName, pData, pSizeOfOneElement, pCountOfElements );

        //Lecture des donnees du repertoire racine
        Retour = RDBEntry_SearchName ( pRDB, pszRootDirectoryName, NULL, SEARCHMODE_DIRECTORY|SEARCHMODE_STARTENTRY, &OffsetEntry );
        if (Retour!=0) return -4;
        Retour = RDBEntry_GetData ( pRDB, OffsetEntry, &RDBEntry, &Size );
        if (Retour!=0) return -4;

        //Verification des droits en mode superviseur
        if ((RDBEntry.FLAGS_Supervisor!=0xFFFFFFFF)&&(RDBEntry.FLAGS_Supervisor!=((*pRDB)->SupervisorOwnerID))) return -4;

        //Recherche si le fichier existe bien
        Retour = RDBEntry_SearchName ( pRDB, pszRootDirectoryName, pszDataName, SEARCHMODE_DATA|SEARCHMODE_STARTENTRY, &OffsetEntry );
        if (Retour!=0) return -2;

        //Lecture des donnees du fichier
        Retour = RDBEntry_GetData ( pRDB, OffsetEntry, &RDBEntry, &Size );
        if (Retour!=0) return -4;

        //Verification des droits en mode superviseur
        if ((RDBEntry.FLAGS_Supervisor!=0xFFFFFFFF)&&(RDBEntry.FLAGS_Supervisor!=((*pRDB)->SupervisorOwnerID))) return -4;

        //Lecture des donnees
        Retour = RDBBase_GetData ( pRDB, RDBEntry.OffsetContent, &RDBData, NULL );
        if (Retour!=0) return -2;

        //Recuperation des donnees
        *pData              = RDBData.pData;
        *pSizeOfOneElement  = RDBData.SizeOfOneElement;

        //Bug decouvert le 2006-07-20
        //Avant : *pSizeOfOneElement = RDBData.CountOfElements;
        //Le probleme : *pCountOfElements n'etait pas rempli et *pSizeOfOneElement contenait le nombre d'elements...
        *pCountOfElements   = RDBData.CountOfElements;  

        return 0;
    }


    long                RDBWriteData            ( SRDB ** pRDB, char * pszRootDirectoryName, char * pszDataName, unsigned char * pData, unsigned long SizeInBytes )
    {
        unsigned long   OffsetEntry; 
        unsigned long   Size;
        unsigned long   OffsetData;
        TRDB_ENTRY      RDBEntry;
        long            Retour;

        //Teste si le pointeur est bon ?
        if ((pRDB==NULL)||(pData==NULL))
            return -1;

        //Est-on en mode client ******************************************************************************************
        if (_RDBAccessServerMode == _TRDBACCESS_CLIENT)
            return RDBAccess_ClientWriteData ( pRDB, pszRootDirectoryName, pszDataName, pData, SizeInBytes );

        //Lecture des donnees du repertoire racine
        Retour = RDBEntry_SearchName ( pRDB, pszRootDirectoryName, NULL, SEARCHMODE_DIRECTORY|SEARCHMODE_STARTENTRY, &OffsetEntry );
        if (Retour!=0) return -4;
        Retour = RDBEntry_GetData ( pRDB, OffsetEntry, &RDBEntry, &Size );
        if (Retour!=0) return -4;

        //Verification des droits en mode superviseur
        if ((RDBEntry.FLAGS_Supervisor!=0xFFFFFFFF)&&(RDBEntry.FLAGS_Supervisor!=((*pRDB)->SupervisorOwnerID))) return -4;

        //Recherche si le fichier existe bien
        Retour = RDBEntry_SearchName ( pRDB, pszRootDirectoryName, pszDataName, SEARCHMODE_DATA|SEARCHMODE_STARTENTRY, &OffsetEntry );
        if (Retour!=0) return -2;

        //Lecture des donnees du fichier
        Retour = RDBEntry_GetData ( pRDB, OffsetEntry, &RDBEntry, &Size );
        if (Retour!=0) return -4;

        //Verification des droits en mode superviseur
        if ((RDBEntry.FLAGS_Supervisor!=0xFFFFFFFF)&&(RDBEntry.FLAGS_Supervisor!=((*pRDB)->SupervisorOwnerID))) return -4;

        //Lecture des donnees
        Retour = RDBBase_GetDataOffset ( pRDB, RDBEntry.OffsetContent, &OffsetData );
        if (Retour!=0) return -2;

        //Ecriture des donnees
        memcpy ( &(*pRDB)->pDataBase[OffsetData], pData, SizeInBytes );

        return 0;
    }
