/*
===================================================================
Copyright (C) 2009-2010 Editions JOCATOP / Christophe Gossa
 
This file is part of Jocareso.

Jocareso is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Jocareso is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jocareso.  If not, see http://www.gnu.org/licenses/


Editions JOCATOP
Route de Saint Saturnin
84310 MORIERES
FRANCE

Christophe Gossa
christophe.gossa@jocatop.fr
===================================================================
*/


    #include "RDB.h"
    /*
        Les erreurs de MakeDirectory

            0   Pas d'erreur
            -1  Base de donnees inaccessible
            -2  Le dossier existe deja
            -3  La racine est introuvable
            -4  Impossible de lire la racine/Racine en mode superviseur



    */

    //Creation d'un repertoire
    long    RDBMakeDirectory    ( SRDB ** pRDB, char * pszRootDirectoryName, char * pszDirectoryName )
    {
        unsigned long   OffsetEntry;
        TRDB_ENTRY      RDBEntry;
        long            Retour;
        char            *psz;

        //Teste si le pointeur est bon ?
        if (pRDB==NULL) return -1;

        //Est-on en mode client ******************************************************************************************
        if (_RDBAccessServerMode == _TRDBACCESS_CLIENT)
            return RDBAccess_ClientMakeDirectory ( pRDB, pszRootDirectoryName, pszDirectoryName );

        //Recherche si le dossier n'existe pas deja
        Retour = RDBEntry_SearchName ( pRDB, pszRootDirectoryName, pszDirectoryName, SEARCHMODE_DIRECTORY|SEARCHMODE_STARTENTRY, &OffsetEntry );
        if (Retour== 0) return -2;                       
        //if (Retour==-3) return -3;

        //Recherche le nombre de "\\" dans le nom du dossier a creer
        psz = strchr ( pszDirectoryName, '\\' );
        if (psz) return -3;

        //Lecture des donnees du repertoire racine
        Retour = RDBEntry_SearchName ( pRDB, pszRootDirectoryName, NULL, SEARCHMODE_DIRECTORY|SEARCHMODE_STARTENTRY, &OffsetEntry );
        if (Retour!=0) return -4;
        Retour = RDBEntry_GetData ( pRDB, OffsetEntry, &RDBEntry, NULL );
        if (Retour!=0) return -4;

        //Verification des droits en mode superviseur
        if ((RDBEntry.FLAGS_Supervisor!=0xFFFFFFFF)&&(RDBEntry.FLAGS_Supervisor!=((*pRDB)->SupervisorOwnerID))) return -4;

        //Recherche la position ou on va inserer le dossier
        Retour = RDBEntry_SearchName ( pRDB, pszRootDirectoryName, NULL, SEARCHMODE_DIRECTORY|SEARCHMODE_TONEXTENTRY, &OffsetEntry );
        if (!Retour) OffsetEntry = (*pRDB)->EntryDataBaseLength;

        //Creation de l'entree du dossier
        RDBEntry.FLAGS_ReadsCount       = 0;
        RDBEntry.FLAGS_Supervisor       = 0xFFFFFFFF;
        RDBEntry.Identifiant            = RDB_ID_DIRECTORY;
        RDBEntry.OffsetContent          = 0xFFFFFFFF;
        RDBEntry.UseLastFullPathName    = 0;
        sprintf ( RDBEntry.FullPathName, "%s%s\\", pszRootDirectoryName, pszDirectoryName );

        //MI ( OffsetEntry );

        //Insertion de l'entree
        return RDBEntry_InsertData ( pRDB, OffsetEntry, &RDBEntry );
    }

    //Suppression d'un repertoire (et de tous ses sous-repertoires...)
    long    RDBKillDirectory    ( SRDB ** pRDB, char * pszRootDirectoryName, char * pszDirectoryName )
    {
        unsigned long   OffsetEntry;
        unsigned long   Size;
        TRDB_ENTRY      RDBEntry;
        TRDB_ENTRY      RDBEntryNext;
        long            Retour;

        //Teste si le pointeur est bon ?
        if (pRDB==NULL) return -1;

        //Est-on en mode client ******************************************************************************************
        if (_RDBAccessServerMode == _TRDBACCESS_CLIENT)
            return RDBAccess_ClientKillDirectory ( pRDB, pszRootDirectoryName, pszDirectoryName );

        //Recherche si le dossier existe bien
        Retour = RDBEntry_SearchName ( pRDB, pszRootDirectoryName, pszDirectoryName, SEARCHMODE_DIRECTORY|SEARCHMODE_STARTENTRY, &OffsetEntry );
        if (Retour!=0) return -2;

        //Lecture des donnees du repertoire
        Retour = RDBEntry_GetData ( pRDB, OffsetEntry, &RDBEntry, &Size );
        if (Retour!=0) return -3;

        //Lecture de l'entree suivante
        if ( (OffsetEntry+Size) < (*pRDB)->EntryDataBaseLength )
        {
            //Verification que l'entree suivante n'est pas une donnee
            Retour = RDBEntry_GetData ( pRDB, OffsetEntry+Size, &RDBEntryNext, NULL );
            if ((Retour==0)&&(RDBEntryNext.UseLastFullPathName==1)) return -5;
        }

        //Verification des droits en mode superviseur
        if ((RDBEntry.FLAGS_Supervisor!=0xFFFFFFFF)&&(RDBEntry.FLAGS_Supervisor!=((*pRDB)->SupervisorOwnerID))) return -4;
        if (RDBEntry.FLAGS_ReadsCount>0) return -4;

        //Suppression de l'entree
        return RDBEntry_KillData ( pRDB, OffsetEntry );
    }
    
    #define _RDBDCL_DIRECTORY   0
    #define _RDBDCL_DATA        1

    long RDBDirectoryContentList ( SRDB ** pRDB, char * pszRootDirectoryName, SRDB_LIST ** pList, unsigned long * pNbList )
    {
        long i;
        long lRetour;
        SRDB_LISTEX * pListEx;
        unsigned long lNbListEx;
        

        //Teste si le pointeur est bon ?
        if ((pRDB==NULL)||(pList==NULL)||(pNbList==NULL)) return -1;

        //Est-on en mode client ******************************************************************************************
        if (_RDBAccessServerMode == _TRDBACCESS_CLIENT)
            return RDBAccess_ClientDirectoryContentList ( pRDB, pszRootDirectoryName, pList, pNbList );


        pListEx = (SRDB_LISTEX *)((long)0xA5A5A5A5);
        lNbListEx = 0x5A5A5A5A;
        lRetour = RDBDirectoryContentListEx ( pRDB, pszRootDirectoryName, &pListEx, &lNbListEx  );

        *pList = NULL;
        *pNbList = 0;
        if (!lRetour)
        {
            if (pListEx)
            {
                *pNbList = lNbListEx;
                *pList   = (SRDB_LIST *)KERNEL_ALLOC ( (*pNbList) * sizeof ( SRDB_LIST ) );
                for (i=0;i<(signed)(*pNbList);i++)
                {
                    (*pList)[i].Identifiant = pListEx[i].Identifiant;
                    strcpy ( (*pList)[i].szName, pListEx[i].szName );
                }
                KERNEL_FREE ( pListEx );
            }
            else *pList = NULL;
        }


        return lRetour;
    }

    long RDBDirectoryContentListEx ( SRDB ** pRDB, char * pszRootDirectoryName, SRDB_LISTEX ** pListEx, unsigned long * pNbList )
    {
        unsigned long   OffsetEntry, Size, Fin;
        TRDB_ENTRY      RDBEntryFound;
        unsigned long   RootDirectoryNameLength, CountSlash;
        unsigned char   GetMode;
        char            *psz;
        long            Retour;

        //Teste si le pointeur est bon ?
        if ((pRDB==NULL)||(pListEx==NULL)||(pNbList==NULL)) return -1;

        //Est-on en mode client ******************************************************************************************
        if (_RDBAccessServerMode == _TRDBACCESS_CLIENT)
            return RDBAccess_ClientDirectoryContentListEx ( pRDB, pszRootDirectoryName, pListEx, pNbList );

        //Recherche si le dossier existe bien
        if ( (*pListEx==(SRDB_LISTEX *)((long)0xA5A5A5A5)) && 
             (*pNbList==0x5A5A5A5A) ) 
             GetMode = SEARCHMODE_DIRECTORY|SEARCHMODE_STARTENTRY;
        else GetMode = SEARCHMODE_DIRECTORY|SEARCHMODE_STARTENTRY|SEARCHMODE_EXTENDED;
        Retour = RDBEntry_SearchName ( pRDB, pszRootDirectoryName, NULL, GetMode, &OffsetEntry );
        if (Retour!=0) return -2;

        //Lecture des donnees du repertoire
        Retour = RDBEntry_GetData ( pRDB, OffsetEntry, &RDBEntryFound, &Size );
        if (Retour!=0) return -3;

        //Verification des droits en mode superviseur
        if ((RDBEntryFound.FLAGS_Supervisor!=0xFFFFFFFF)&&(RDBEntryFound.FLAGS_Supervisor!=((*pRDB)->SupervisorOwnerID))) return -4;

        //On se deplace maintenant sur la premiere entree
        OffsetEntry += Size;

        //Il n'y rien a lister ?
        if (OffsetEntry>=(*pRDB)->EntryDataBaseLength)
        {
            *pListEx      = NULL;
            *pNbList      = 0;
            return 0;
        }

        //On parcours la base jusqu'a ce que le chemin de base ne soit plus valide
        *pNbList                = 0;
        *pListEx                = NULL;
        Fin                     = FALSE;
        RootDirectoryNameLength = (unsigned long)strlen ( pszRootDirectoryName );
        GetMode                 = _RDBDCL_DATA; //On commence par lister les donnees                

        /*
            Il faut parcourir l'entree de dossier courante pour en recuperer la liste des fichiers/donnees, puis continuer a parcourir
            la liste des entrees tant que le debut du nom de l'entree correspond au dossier courant et que le nombre de slash n'est pas superieur
            a 1.
        */

        //On commence a parcourir les entrees de fichiers/donnees
        do {

            //Recupere les info sur l'entree en cours ainsi que sa taille
            Retour = RDBEntry_GetData ( pRDB, OffsetEntry, &RDBEntryFound, &Size );

            if (GetMode==_RDBDCL_DATA)
            {
                if (RDBEntryFound.UseLastFullPathName==0)
                {
                    //On vient de trouver une entree de repertoire
                    GetMode = _RDBDCL_DIRECTORY;
                }
                else
                {
                    //Remplissage de la liste
                    *pListEx = (SRDB_LISTEX *)KERNEL_REALLOC ( *pListEx, ((*pNbList)+1) * sizeof ( SRDB_LISTEX ) );
                    (*pListEx)[*pNbList].lTotalSize     = 0;
                    (*pListEx)[*pNbList].Identifiant    = RDBEntryFound.Identifiant;
                    strcpy ( (*pListEx)[*pNbList].szName, RDBEntryFound.FullPathName );
                    RDBBase_GetData ( pRDB, RDBEntryFound.OffsetContent, NULL, &((*pListEx)[*pNbList].lTotalSize) );
                    (*pNbList)++;
                }
            }

            if (GetMode==_RDBDCL_DIRECTORY)
            {
                if (RDBEntryFound.UseLastFullPathName==0)
                {
                    //La longueur du nom de l'entree trouvee differe ? OUI
                    if (strlen(RDBEntryFound.FullPathName)<RootDirectoryNameLength)
                    {
                        //Fin = TRUE; //Alors on quitte
                    }
                    else
                    {
                        //Teste si le debut du nom du dossier correspond
                        if (memcmp ( pszRootDirectoryName, RDBEntryFound.FullPathName, RootDirectoryNameLength)==0)
                        {
                            //Oui : On recherche le nombre de "\\"
                            CountSlash = 0;
                            psz = &RDBEntryFound.FullPathName[RootDirectoryNameLength];
                            do {
                                psz = strchr ( psz, '\\' );
                                if (psz)
                                {
                                    CountSlash++;
                                    psz++;
                                }
                            } while (psz);

                            //Si un seul slash, alors on est simplement dans le repertoire enfant de l'entree de base
                            if ((CountSlash==1)||((CountSlash==2)&&(RootDirectoryNameLength==0)))
                            {
                                *pListEx = (SRDB_LISTEX *)KERNEL_REALLOC ( *pListEx, ((*pNbList)+1) * sizeof ( SRDB_LISTEX ) );
                                (*pListEx)[*pNbList].Identifiant  =   RDBEntryFound.Identifiant;
                                strcpy ( (*pListEx)[*pNbList].szName, &RDBEntryFound.FullPathName[RootDirectoryNameLength] );
                                (*pNbList)++;
                            }
                        }
                        else
                        {
                            //Non : Alors on a completement change de dossier
                            //Fin = TRUE;
                        }
                    }
                }


            }

            //Gestion du deplacement
            OffsetEntry += Size;
            if (OffsetEntry>=(*pRDB)->EntryDataBaseLength) 
                Fin = TRUE;

        } while (Fin==FALSE);
        return 0;
    }

