/*
===================================================================
Copyright (C) 2009-2010 Editions JOCATOP / Christophe Gossa
 
This file is part of Jocareso.

Jocareso is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Jocareso is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jocareso.  If not, see http://www.gnu.org/licenses/


Editions JOCATOP
Route de Saint Saturnin
84310 MORIERES
FRANCE

Christophe Gossa
christophe.gossa@jocatop.fr
===================================================================
*/


    #include "RDB.h"

    #include "strex.h"


    /*
        Une entree

            USHORT          Taille totale de l'entree (en comptant cette donnee)
            UCHAR           Type de l'entree pointee : DATA/FILE/DIRECTORY
            UCHAR           UseLastFullName (indique egalement un repertoire si 0)
            ULONG           Drapeau "Read Count" : Valeur par defaut = 0
            ULONG           Drapeau "Supervisor" : Valeur par defaut = 0xFFFFFFFF
            ULONG           Offset dans les donnees
            STRING          Nom de l'entree

    */

    //Gestion de la base d'entree

    /*  ##################################################################################################################################

        Recuperation des informations d'une entree

            Fonctionnement :
            
                Il est possible de ne pas fournir les parametres pRDBEntry et/ou pSize.
                
                La fonction rempli si possible ces 2 pointeurs.
                
                    pSize       Taille de l'entree
                    pRDBEntry   Donnees de l'entree a remplir
                            

    */


    /*
        Exemple d'organisation des entrees

        \
        TESTE.TXT
        \Systeme
        standard.dll
        enhanced.dll
        extended.dll
        \Systeme\Blue
        \Systeme\Blue\EX
        EX.dll
        EX2.dll
        \Systeme\Green
        Light.txt
        Light.dll

    */

    long        RDBEntry_KillSupervisor ( SRDB ** pRDB, unsigned long OwnerID )
    {
        TRDB_ENTRY      RDBEntry;
        unsigned long   Offset;
        unsigned long   Size;
        unsigned char   Fin;

        //On ne peut pas desactiver une entree deja desactivee !
        if (OwnerID==0xFFFFFFFF) return 0;

        Offset  = 0;
        Fin     = FALSE;
        do {
            RDBEntry_GetData ( pRDB, Offset, &RDBEntry, &Size );
            if (RDBEntry.FLAGS_Supervisor==OwnerID) RDBEntry_SetFlag ( pRDB, Offset, 0xFFFFFFFF, 0 );
            Offset += Size;
            if (Offset>=(*pRDB)->EntryDataBaseLength) Fin = TRUE;
        } while (Fin==FALSE);
        return 0;
    }

    long        RDBEntry_KillAllFlags ( SRDB ** pRDB )
    {
        TRDB_ENTRY      RDBEntry;
        unsigned long   Offset;
        unsigned long   Size;
        unsigned char   Fin;

        Offset  = 0;
        Fin     = FALSE;
        do {
            RDBEntry_GetData ( pRDB, Offset, &RDBEntry, &Size );
            RDBEntry_SetFlag ( pRDB, Offset, 0xFFFFFFFF, 0 );
            Offset += Size;
            if (Offset>=(*pRDB)->EntryDataBaseLength) Fin = TRUE;
        } while (Fin==FALSE);
        return 0;
    }


    long        RDBEntry_SearchName     ( SRDB ** pRDB, char * pszDirectoryName, char * pszSearchName, unsigned char SearchMode, unsigned long * pOffsetFound )
    {
        TRDB_ENTRY      RDBEntry;
        unsigned long   Offset;
        unsigned long   Size;
        unsigned char   EntryBaseFound;
        unsigned char   Fin;
        char            szPathName[2048];
        long            lTailleDirectoryName;

        //Teste si le pointeur est bon ?
        if (pRDB==NULL) return -1;

        /*
            Les modes de recherche

            SEARCHMODE_STARTENTRY
            SEARCHMODE_AFTERENTRY
            SEARCHMODE_BEFORENEXTENTRY            
            SEARCHMODE_TONEXTENTRY

            SEARCHMODE_DIRECTORY
            SEARCHMODE_DATA

            0 = Recherche le point d'entree du dossier specifie
            1 = Recherche le point d'entree situe juste apres le dossier specifie
            2 = Recherche le point d'entree situe apres le dossier specifie et juste avant le dossier suivant

        */
        if (SearchMode&SEARCHMODE_EXTENDED)
        {
            if (pszDirectoryName)
            {
                lTailleDirectoryName = (long)strlen(pszDirectoryName);
                if ((lTailleDirectoryName==1)&&(pszDirectoryName[0]=='\\')) lTailleDirectoryName = 0;
            }
            else lTailleDirectoryName = 0;
        } else lTailleDirectoryName = -1;
        SearchMode = SearchMode&(~SEARCHMODE_EXTENDED);
        


        //Definition du chemin de recherche complet
        if ((SearchMode&SEARCHMODE_DIRECTORY)&&(pszSearchName)) sprintf ( szPathName, "%s%s\\", pszDirectoryName, pszSearchName );
        else                                                    sprintf ( szPathName, "%s", pszDirectoryName );
        

        



        Offset          = 0;
        EntryBaseFound  = FALSE;
        Fin             = FALSE;
        //MI ( (*pRDB)->EntryDataBaseLength );
        do {
            //Recuperation de l'entree
            RDBEntry_GetData ( pRDB, Offset, &RDBEntry, &Size );

            //sprintf ( szTmp, "%s\nOffset:%d  -  Size:%d", RDBEntry.FullPathName, Offset, Size );
            //MB ( szTmp );

            //Le repertoire de base n'a pas ete trouve
            if (EntryBaseFound==FALSE)
            {
                //Est-ce bien un repertoire ? Le nom correspond t-il a celui recherche ?
                if ( ( RDBEntry.UseLastFullPathName == 0 ) && ( ( strucasecmp ( szPathName, RDBEntry.FullPathName ) == 0 ) || (lTailleDirectoryName==0) ) )
                {
                    //On vient de trouver l'entree de base
                    EntryBaseFound = TRUE;

                    //Recherchions-nous le point d'entree d'un dossier ? Oui - Alors on quitte
                    if (SearchMode == (SEARCHMODE_DIRECTORY|SEARCHMODE_STARTENTRY)) Fin = TRUE;

                    //Recherchions-nous le point d'entree situe juste apres le dossier specifie ? Oui - Alors on modifie l'offset et on quitte
                    if (SearchMode == (SEARCHMODE_DIRECTORY|SEARCHMODE_AFTERENTRY))
                    {
                        Offset += Size;
                        Fin = TRUE;
                    }
                }
            }
            else //On a trouve le repertoire de base, on poursuit la recherche
            {
                //Recherche le point d'entree situe apres le dossier specifie et juste avant le dossier suivant
                if ( (SearchMode == (SEARCHMODE_DIRECTORY|SEARCHMODE_BEFORENEXTENTRY)) && ( RDBEntry.UseLastFullPathName == 0 ) )
                    Fin = TRUE;

                if ( (SearchMode == (SEARCHMODE_DIRECTORY|SEARCHMODE_TONEXTENTRY)) && ( RDBEntry.UseLastFullPathName == 0 ) )
                {
                    Offset += Size;
                    Fin = TRUE;
                }

                if ( SearchMode&SEARCHMODE_DATA )
                {
                    //Est-ce bien une donnee
                    if ( RDBEntry.UseLastFullPathName == 1 )
                    {
                        //Le nom de la donnee correspond ?
                        if (strucasecmp(pszSearchName,RDBEntry.FullPathName)==0)
                        {
                            if (SearchMode != (SEARCHMODE_DATA|SEARCHMODE_STARTENTRY)) Offset += Size;
                            Fin = TRUE;
                        }
                    }
                    else //Ce n'est pas une donnee, c'est une nouvelle entree... ARGH! Donnee non trouvee !
                    {
                        Offset = 0xFFFFFFFF;
                        Fin = TRUE;
                    }
                }
            }

            //Gestion du deplacement dans les entrees
            if (Fin==FALSE)
            {
                Offset += Size;
                if (Offset>=(*pRDB)->EntryDataBaseLength)
                {
                    //On n'a pas trouve d'entree... ARGH !
                    if ( (SearchMode&SEARCHMODE_STARTENTRY) || (SearchMode&SEARCHMODE_AFTERENTRY) ) Offset = 0xFFFFFFFF;
                    Fin = TRUE;
                }
            }

        } while (Fin==FALSE);

        if (EntryBaseFound==FALSE)
            return -3;  //Impossible de trouver l'entree de base

        if ( ( pOffsetFound ) && ( Offset != 0xFFFFFFFF ) )
        {
            *pOffsetFound = Offset;
            return 0;
        }

        if (Offset!=0xFFFFFFFF)
            return 0;


        return -2;  //Impossible de trouver l'entree demande
    }

    long RDBEntry_GetData ( SRDB ** pRDB, unsigned long OffsetGet, TRDB_ENTRY * pRDBEntry, unsigned long * pSize )
    {
        _TRDBMemoryStream MS;
        unsigned short  Size;
        unsigned char   Identifiant;
        unsigned char   UseLastFullPathName;
        unsigned long   ReadsCount;
        unsigned long   Supervisor;
        unsigned long   OffsetContent;
        char            sz[2048];

        //Teste si le pointeur est bon ?
        if (pRDB==NULL) return -1;

        //Lecture du flux entrant
        _RDBMemoryStreamOpen           ( &MS, (unsigned char *)&((*pRDB)->pEntryDataBase[OffsetGet]) );
        _RDBMemoryStreamGet_USHORT     ( &MS, &Size );
        _RDBMemoryStreamGet_UCHAR      ( &MS, &Identifiant );
        _RDBMemoryStreamGet_UCHAR      ( &MS, &UseLastFullPathName );
        _RDBMemoryStreamGet_ULONG      ( &MS, &ReadsCount );
        _RDBMemoryStreamGet_ULONG      ( &MS, &Supervisor );
        _RDBMemoryStreamGet_ULONG      ( &MS, &OffsetContent );
        _RDBMemoryStreamGet_STRING     ( &MS, sz );

        //Remplissage des donnees de sortie
        if (pRDBEntry)
        {
            pRDBEntry->Identifiant              = Identifiant;
            pRDBEntry->UseLastFullPathName      = UseLastFullPathName;
            pRDBEntry->FLAGS_ReadsCount         = ReadsCount;
            pRDBEntry->FLAGS_Supervisor         = Supervisor;
            pRDBEntry->OffsetContent            = OffsetContent;
            strcpy ( pRDBEntry->FullPathName, sz );
        }

        if (pSize) *pSize = (unsigned long)Size;
        return 0;
    }

    long RDBEntry_SetEntryOffsetContent ( SRDB ** pRDB, unsigned long Offset, unsigned long NewOffsetContent )
    {
        _TRDBMemoryStream MS;

        //Teste si le pointeur est bon ?
        if (pRDB==NULL) return -1;

        _RDBMemoryStreamOpen ( &MS, NULL );
        _RDBMemoryStreamAdd_ULONG  ( &MS, NewOffsetContent );
        memcpy ( (unsigned char *)&((*pRDB)->pEntryDataBase[Offset+12]), MS.pBuffer, 4 );
        _RDBMemoryStreamClose ( &MS );

        return 0;
    }

    long RDBEntry_SetFlag ( SRDB ** pRDB, unsigned long Offset, unsigned long FlagSupervisor, unsigned long FlagReadsCount )
    {
        _TRDBMemoryStream MS;

        //Teste si le pointeur est bon ?
        if (pRDB==NULL) return -1;

        _RDBMemoryStreamOpen ( &MS, NULL );
        _RDBMemoryStreamAdd_ULONG  ( &MS, FlagReadsCount );
        _RDBMemoryStreamAdd_ULONG  ( &MS, FlagSupervisor );
        memcpy ( (unsigned char *)&((*pRDB)->pEntryDataBase[Offset+4]), MS.pBuffer, 8 );
        _RDBMemoryStreamClose ( &MS );

        return 0;
    }

    long RDBEntry_MakeData ( TRDB_ENTRY * pRDBEntry, unsigned long * pSize, unsigned char ** pStream )
    {
        _TRDBMemoryStream MS;

        if (pRDBEntry==NULL) return -1;

        //Creation du flux
        _RDBMemoryStreamOpen ( &MS, NULL );
        _RDBMemoryStreamAdd_USHORT ( &MS, 0 );
        _RDBMemoryStreamAdd_UCHAR  ( &MS, pRDBEntry->Identifiant );
        _RDBMemoryStreamAdd_UCHAR  ( &MS, pRDBEntry->UseLastFullPathName );

        _RDBMemoryStreamAdd_ULONG  ( &MS, pRDBEntry->FLAGS_ReadsCount );
        _RDBMemoryStreamAdd_ULONG  ( &MS, pRDBEntry->FLAGS_Supervisor );

        _RDBMemoryStreamAdd_ULONG  ( &MS, pRDBEntry->OffsetContent );

        _RDBMemoryStreamAdd_STRING ( &MS, pRDBEntry->FullPathName );

        //On fait quelque chose qui n'est normalement pas prevu : modifier une donnee du flux
        *((unsigned short *)MS.pBuffer) = (unsigned short)MS.BufferLength;

        //Remplissage des donnees de sortie
        if (pSize) *pSize = MS.BufferLength;
        if (pStream) *pStream = MS.pBuffer;
        else         _RDBMemoryStreamClose ( &MS );
        return 0;
    }



    long RDBEntry_InsertData ( SRDB ** pRDB, unsigned long OffsetSet, TRDB_ENTRY * pRDBEntry )
    {
        unsigned char *pBuffer;
        unsigned long SizeBuffer;
        long i;
        
        if ((pRDB==NULL)||(pRDBEntry==NULL)) return -1;

        //Creation du bloc de donnees a inserer
        RDBEntry_MakeData ( pRDBEntry, &SizeBuffer, &pBuffer );

        //Reallocation de la zone memoire
        (*pRDB)->pEntryDataBase = (unsigned char *)KERNEL_REALLOC ( (*pRDB)->pEntryDataBase, (*pRDB)->EntryDataBaseLength + SizeBuffer );

        //Deplacement de la zone memoire pour l'insertion (si necessaire)
        if ( (((*pRDB)->EntryDataBaseLength) - OffsetSet) > 0 )
        {
            for (i=(((*pRDB)->EntryDataBaseLength) - OffsetSet)-1;i>=0;i--)
            {
                (*pRDB)->pEntryDataBase[OffsetSet+SizeBuffer+i] = (*pRDB)->pEntryDataBase[OffsetSet+i];
            }
            

            /*
            memcpy ( &(*pRDB)->pEntryDataBase[OffsetSet+SizeBuffer],
                     &(*pRDB)->pEntryDataBase[OffsetSet],
                     ((*pRDB)->EntryDataBaseLength) - OffsetSet );
            */
        }

        //Copie des donnees
        memcpy ( &(*pRDB)->pEntryDataBase[OffsetSet], pBuffer, SizeBuffer );

        //Met a jour la longueur des donnees des entrees
        (*pRDB)->EntryDataBaseLength += SizeBuffer;

        //Liberation du tampon des donnees
        KERNEL_FREE ( pBuffer );

        return 0;
    }

    long RDBEntry_KillData ( SRDB ** pRDB, unsigned long OffsetKill )
    {
        unsigned long Size;

        if (pRDB==NULL) return -1;

        //Recuperation des donnees de l'entree a supprimer
        RDBEntry_GetData ( pRDB, OffsetKill, NULL, &Size );

        //Deplacement des donnees des entrees
        memcpy ( &(*pRDB)->pEntryDataBase[OffsetKill],
                 &(*pRDB)->pEntryDataBase[OffsetKill+Size],
                 (*pRDB)->EntryDataBaseLength-OffsetKill-Size );

        //Reallocation
        (*pRDB)->EntryDataBaseLength -= Size;
        (*pRDB)->pEntryDataBase = (unsigned char *)KERNEL_REALLOC ( (*pRDB)->pEntryDataBase, (*pRDB)->EntryDataBaseLength );
        

        return 0;
    }

    long RDBEntry_QuerySupervisorFlags ( SRDB ** pRDB, long * plNbSupervisorSets )
    {
        TRDB_ENTRY      RDBEntry;
        unsigned long   Offset;
        unsigned long   Size;

        //Teste si le pointeur est bon ?
        if (pRDB==NULL) return -1;

        if (!plNbSupervisorSets) return 0;

        Offset          = 0;
        do {
            //Recuperation de l'entree
            RDBEntry_GetData ( pRDB, Offset, &RDBEntry, &Size );
            if (RDBEntry.FLAGS_Supervisor!=0xFFFFFFFF) (*plNbSupervisorSets) += 1;

            Offset += Size;

        } while (Offset<(*pRDB)->EntryDataBaseLength);

        return 0;
    }

    long RDBEntry_QueryReadCountFlags ( SRDB ** pRDB, long * plNbReadCount )
    {
        TRDB_ENTRY      RDBEntry;
        unsigned long   Offset;
        unsigned long   Size;

        //Teste si le pointeur est bon ?
        if (pRDB==NULL) return -1;

        if (!plNbReadCount) return 0;

        Offset          = 0;
        do {
            //Recuperation de l'entree
            RDBEntry_GetData ( pRDB, Offset, &RDBEntry, &Size );
            if ( (RDBEntry.FLAGS_ReadsCount>0) &&
                 (RDBEntry.FLAGS_ReadsCount!=0xFFFFFFFF) ) (*plNbReadCount) += RDBEntry.FLAGS_ReadsCount;

            Offset += Size;

        } while (Offset<(*pRDB)->EntryDataBaseLength);

        return 0;
    }


    












































