/*
===================================================================
Copyright (C) 2009-2010 Editions JOCATOP / Christophe Gossa
 
This file is part of Jocareso.

Jocareso is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Jocareso is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jocareso.  If not, see http://www.gnu.org/licenses/


Editions JOCATOP
Route de Saint Saturnin
84310 MORIERES
FRANCE

Christophe Gossa
christophe.gossa@jocatop.fr
Gavroche Richier
gavroche.richier@jocatop.fr
===================================================================
*/


    #include "RDBServer.h"
    #include "key.h"
    #include "rijndael.h"
    #include "sha2.h"
    #include "RDBServerClientInterface.h"

    /*
    DWORD                   interface_dwState = 0; //0 = not connected
    DWORD                   interface_dwIdentifiant;
    BYTE                    interface_bFKey[256];       //Clef totalement aleatoire
    BYTE                    interface_bIKey[256];       //Clef generee avec l'ID
    BYTE                    interface_bPKey[256];       //Clef generee avec le password
    BYTE                    interface_bAKey[256];       //Clef d'authentification (FKey+IKey+PKey)
    QWORD                   interface_qwTxRxKey[256];    //Clef de transmition/reception
    QWORD                   interface_qwRandomSeed;
    */
    
    void ServerClientInterface_Init ( SCILINK * psciLink )
    {
        memset ( psciLink, 0, sizeof(SCILINK) );
    }

    DWORD ServerClientInterface_GetState ( SCILINK * psciLink )
    {
        return psciLink->interface_dwState;
    }

    DWORD ServerClientInterface_GetIdentifiant ( SCILINK * psciLink )
    {
        return psciLink->interface_dwIdentifiant;
    }

    void ServerClientInterface_SetStateIdentifiant ( SCILINK * psciLink, DWORD dwState, DWORD dwIdentifiant )
    {
        psciLink->interface_dwState = dwState;
        psciLink->interface_dwIdentifiant = dwIdentifiant;    
    }



    void ServerClientInterface_UpdateSeed ( SCILINK * psciLink, unsigned long lValue )
    {
        psciLink->interface_qwRandomSeed += (QWORD)lValue;
        psciLink->interface_qwRandomSeed = ( ( 1664525 * psciLink->interface_qwRandomSeed ) + 1013904223 ) & 0xFFFFFFFF;
    }

    void ServerClientInterface_GetRand ( SCILINK * psciLink, unsigned long * pulValue )
    {
        if (pulValue)
        {
            psciLink->interface_qwRandomSeed = ( ( 1664525 * psciLink->interface_qwRandomSeed ) + 1013904223 ) & 0xFFFFFFFF;
            *pulValue = (unsigned long)psciLink->interface_qwRandomSeed;
        }
    }

    #define M_ROR16(x,c)            (((x)>>(c))|((x)<<(16-(c))))
    long ServerClientInterface_MakeFKey_TableRotation[6] = { 2, 3, 5, 7, 11, 13 };

    long ServerClientInterface_MakeFKey ( SCILINK * psciLink )
    {
        WORD wBaseFKey[2048];
        WORD wValue;
        long i,j,lRotation;

        //Generation d'une base de 32kbits
        ServerClientInterface_UpdateSeed ( psciLink, TickGetCount2() );
        ServerClientInterface_UpdateSeed ( psciLink, (unsigned long)TickGetRDTSC2() );
        for (j=0;j<16;j++)
        {
            for (i=0;i<2048;i++)
            {
                ServerClientInterface_UpdateSeed ( psciLink, i );
                wValue = (WORD)((psciLink->interface_qwRandomSeed>>16)&0xFFFF);
                
                if (j==0) wBaseFKey[i] = wValue;
                else wBaseFKey[i] ^= wValue;
                
                lRotation = ServerClientInterface_MakeFKey_TableRotation[((i^j)^((long)wValue))%6];
                wBaseFKey[i] = M_ROR16(wBaseFKey[i],lRotation);
            }
            ServerClientInterface_UpdateSeed ( psciLink, TickGetCount2() );
            ServerClientInterface_UpdateSeed ( psciLink, (unsigned long)TickGetRDTSC2() );
            wValue = wBaseFKey[0];
            for (i=0;i<2047;i++)
            {            
                wBaseFKey[i] = wBaseFKey[i+1];
                
            }
            wBaseFKey[2047] = wValue;
            ServerClientInterface_UpdateSeed ( psciLink, wValue );
        }
        ServerClientInterface_UpdateSeed ( psciLink, TickGetCount2() );
        ServerClientInterface_UpdateSeed ( psciLink, (unsigned long)TickGetRDTSC2() );

        //Hashage pour en faire un bloc de 2048bits
        KeyCreateHashValue ( 
            (unsigned char *)psciLink->interface_bFKey, 256,
            (unsigned char *)wBaseFKey, 2048*2 );

        for (i=0;i<2048;i++)
            wBaseFKey[i] = 0;

        return 0;
    }

    long ServerClientInterface_MakeIKey ( SCILINK * psciLink, DWORD dwInterfaceID )
    {
        DWORD dwInterfaceIDTable[16];
        long i;

        dwInterfaceIDTable[0] = dwInterfaceID;
        for (i=1;i<16;i++)
        {
            dwInterfaceIDTable[i] = dwInterfaceID^(DWORD)i;
        }

        //Hashage pour en faire un bloc de 2048bits
        KeyCreateHashValue ( 
            (unsigned char *)psciLink->interface_bIKey, 256,
            (unsigned char *)dwInterfaceIDTable, 16*4 );

        for (i=0;i<16;i++)
            dwInterfaceIDTable[i] = 0;

        return 0;
    }

    
    long ServerClientInterface_ConvertHashToText ( char * pszPassword, unsigned char * pHash, long lHashByteCount )
    {
        long lPasswordDisabled;
        long lIndex;
        char szTmp[16];
        
        pszPassword[0] = 0;
        lPasswordDisabled = -1;
        
        for (lIndex=0;lIndex<lHashByteCount;lIndex++)
        {
            sprintf ( szTmp, "%02x", (unsigned char)(pHash[lIndex]) );
            strcat ( pszPassword, szTmp );
            if (pHash[lIndex]) lPasswordDisabled = 0;
        }
        
        return lPasswordDisabled;
    }
    
    long ServerClientInterface_ConvertPasswordToTextSHA512 ( char * pszPasswordSHA512, char * pszPassword )
    {
        unsigned char bSHA512[64];
        
        if (strlen(pszPassword))
        {
            sha512 ( pszPassword, (long)strlen(pszPassword), bSHA512 );        
            ServerClientInterface_ConvertHashToText ( pszPasswordSHA512, bSHA512, 64 );
            return 0;
        }
        else
        {
            pszPasswordSHA512[0] = 0;
            return -1;        
        }
    }


    long ServerClientInterface_MakePKey ( SCILINK * psciLink, unsigned char * pbPasswordSHA512 )
    {
        //Hashage pour en faire un bloc de 2048bits
        KeyCreateHashValue ( 
            (unsigned char *)psciLink->interface_bPKey, 256,
            pbPasswordSHA512, 64 );

        return 0;
    }
    
    long ServerClientInterface_MakeTxRxKey ( SCILINK * psciLink )
    {
        BYTE bKey[512];
        long i;
        
        for (i=0;i<256;i++)
        {
            bKey[i] = psciLink->interface_bPKey[i];
            bKey[i+256] = psciLink->interface_bFKey[i] ^ psciLink->interface_bGKey[i];
        }

        sha512 ( bKey, 512, psciLink->interface_bTxRxKeySHA512 );
        
        return 0;
    }

    long ServerClientInterface_MakeAKey ( SCILINK * psciLink )
    {
        long i;
        
        for (i=0;i<256;i++)
            psciLink->interface_bAKey[i] = psciLink->interface_bFKey[i] ^ psciLink->interface_bPKey[i];
            
            
		return 0;
    }

    long ServerClientInterface_MakeGKey ( SCILINK * psciLink, unsigned char * pRemoteKey )
    {
        long i;
        
        for (i=0;i<256;i++)
            psciLink->interface_bGKey[i] = pRemoteKey[i] ^ psciLink->interface_bPKey[i];

        return 0;
    }

    BYTE * ServerClientInterface_GetAKey ( SCILINK * psciLink )
    {
        return psciLink->interface_bAKey;
    }

    BYTE * ServerClientInterface_GetFKey ( SCILINK * psciLink )
    {
        return psciLink->interface_bFKey;
    }

    void ServerClientInterface_SetFKey ( SCILINK * psciLink, BYTE * pbFKey )
    {
        memcpy ( psciLink->interface_bFKey, pbFKey, 256 );
    }

    /*
    **  0....31     = sha256
    **  32...47     = Random byte
    **  48...51     = Len
    **  52...??     = data
    */

    void ServerClientInterface_Encrypt ( SCILINK * psciLink, unsigned char * pInBuffer, long lInLength, unsigned char ** pOutBuffer, long * plOutLength )
    {
        long lTxSize;
        long i;
		unsigned char * szCipher;
		unsigned char szKey[32];
		unsigned long szRoundKey[60];

        lTxSize = lInLength + 20 + 32;

		// Align on 16 bytes
        if (lTxSize&15)
            lTxSize = (lTxSize&0x7FFFFFF0)+16;

        *pOutBuffer = (unsigned char *)KERNEL_ALLOC ( lTxSize );
		szCipher = (unsigned char *)KERNEL_ALLOC ( lTxSize );

        //Ajout de 16 octets inutiles !
        for (i=0;i<16;i++)
        {
            ServerClientInterface_UpdateSeed ( psciLink, pInBuffer[i%lInLength] );
            (*pOutBuffer)[i+32] = (unsigned char)((psciLink->interface_qwRandomSeed>>16)&0xFF);
        }
        
        //Ajoute la longueur
        *((long *)&((*pOutBuffer)[48])) = lInLength;

        //Ajoute les donnes
        memcpy ( (*pOutBuffer)+20+32, pInBuffer, lInLength );
        
        //Ajoute le SHA256
        sha256 ( (*pOutBuffer)+32, lTxSize-32, (*pOutBuffer) );
        
		// Key initialization
        memcpy(szKey, psciLink->interface_bTxRxKeySHA512, 32);
		for(i = 0; i < 32; ++i)
			szKey[i] ^= psciLink->interface_bTxRxKeySHA512[32 + i];

		SetupEncrypt(szRoundKey, szKey);
		Encrypt(*pOutBuffer, szCipher, szRoundKey, lTxSize);
		memcpy(*pOutBuffer, szCipher, lTxSize);

		KERNEL_FREE(szCipher);
		*plOutLength = lTxSize;
    }

    long ServerClientInterface_Decrypt ( SCILINK * psciLink, unsigned char * pInBuffer, long lInLength, unsigned char ** pOutBuffer, long * plOutLength )
    {
        long lRxSize;
        unsigned long szRoundKey[60];
		unsigned char szKey[32];
        BYTE bSHA256[32];
        long i;

        lRxSize = lInLength;
        *pOutBuffer = (unsigned char *)KERNEL_ALLOC ( lRxSize );

		// Key initialization
        memcpy(szKey, psciLink->interface_bTxRxKeySHA512, 32);
		for(i = 0; i < 32; ++i)
			szKey[i] ^= psciLink->interface_bTxRxKeySHA512[32 + i];

		SetupDecrypt(szRoundKey, szKey);
		Decrypt(pInBuffer, *pOutBuffer, szRoundKey, lRxSize);

        //On calcule le SHA256
        sha256 ( (*pOutBuffer)+32, lRxSize-32, bSHA256 );
        
        //On teste le SHA256
        if (memcmp(bSHA256,(*pOutBuffer),32)!=0)
        {
            KERNEL_FREE ( (*pOutBuffer) );
            return -1;
        }
        
        //La longueur
        *plOutLength = *((long *)&((*pOutBuffer)[48]));
        /*
        if (*plOutLength>(lRxSize+52))
        {
            KERNEL_FREE ( (*pOutBuffer) );
            return -1;
        }*/
            
        memmove ( *pOutBuffer, (*pOutBuffer)+52, *plOutLength );
        
        return 0;
    }

