/*
===================================================================
Copyright (C) 2009-2010 Editions JOCATOP / Christophe Gossa
 
This file is part of Jocareso.

Jocareso is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Jocareso is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jocareso.  If not, see http://www.gnu.org/licenses/


Editions JOCATOP
Route de Saint Saturnin
84310 MORIERES
FRANCE

Christophe Gossa
christophe.gossa@jocatop.fr
===================================================================
*/


    #include "RDBServer.h"

    /*
        Rights Security Enforcement v2
    */
    
    
    #include "criticalsection.h"
    #include "sha2.h"
    
    
    

    
    typedef struct {
        unsigned char   bNameHash[32];
        unsigned long   ulClientIDSupervisor;       //0xFFFFFFFF si faux
        unsigned long   *pulClientIDReadsCount;     //Liste de clients...
        long            lReadsCount;                //Nombre de clients en readonly mode
    } RDBFLAGENTRY;
    
    /*
    
    Le calcul du hash...
    pRDB->Header.szRDBName + pszRootDirectoryName + pszEntryName
    
    */
    
    static RDBFLAGENTRY     *pFlagEntryData;
    static long             lFlagEntryCount;
    static long             lFlagEntryCountAlloc;   //Idem que Count mais le nombre réels d'allocations...
    static CRITICALSECTION  csFlagEntry;
    
    
    long RDBServerRightsOpen ( void )
    {
        lFlagEntryCount         = 0;
        lFlagEntryCountAlloc    = 0;
        pFlagEntryData          = NULL;
        
        CriticalSectionCreate ( &csFlagEntry );
        
        return 0;
    }
    
    long RDBServerRightsClose ( void )
    {
        long lIndex;
        
        for (lIndex=0;lIndex<lFlagEntryCount;lIndex++)
        {
            KERNEL_FREE ( pFlagEntryData[lIndex].pulClientIDReadsCount );
        }
        KERNEL_FREE ( pFlagEntryData );
        lFlagEntryCount         = 0;
        lFlagEntryCountAlloc    = 0;
        pFlagEntryData          = NULL;
        CriticalSectionDelete ( &csFlagEntry );
        return 0;    
    }
    
    /*
    **
    **  lMode
    **      -2 = Disable supervisor
    **      -1 = Disable ReadOnly
    **       1 = Enable ReadOnly
    **       2 = Enable supervisor
    **
    */
    long  RDBServerRightsSet ( SRDB ** pRDB, char * pszRootDirectoryName, char * pszEntryName, long lMode, unsigned long ulClientID )
    {
        char szTmp[4096];
        unsigned char bNameHash[32];
        long lIndex, lFound, lReadCountIndex;
        
        sprintf ( szTmp, "%s%s%s", (*pRDB)->Header.szRDBName, pszRootDirectoryName, pszEntryName );
        sha256 ( (unsigned char *)szTmp, (unsigned long)strlen(szTmp)+1, bNameHash );
        
        CriticalSectionEnter ( &csFlagEntry );
        
        //Searching for the entry...
        for (lIndex=0,lFound=-1;lIndex<lFlagEntryCount;lIndex++)
        {
            if (memcmp(bNameHash,pFlagEntryData[lIndex].bNameHash,32)==0)
            {
                lFound = lIndex;
                break;
            }
        }
        
        //Could not found the entry...
        if (lFound==-1)
        {        
            //Entry was not set... We need to disable, Mmmm. There's an error !
            if (lMode<0)
            {
                CriticalSectionLeave ( &csFlagEntry );
                return -2;
            }
            
            //Entry creation
            if ((lFlagEntryCount+1)>lFlagEntryCountAlloc)
            {
                lFlagEntryCountAlloc += 16;       
                pFlagEntryData = (RDBFLAGENTRY *)KERNEL_REALLOC ( pFlagEntryData, lFlagEntryCountAlloc * sizeof(RDBFLAGENTRY) );
            }
            lIndex = lFlagEntryCount;
            lFlagEntryCount++;

            //Create the new entry            
            memcpy ( pFlagEntryData[lIndex].bNameHash, bNameHash, 32 );
            pFlagEntryData[lIndex].ulClientIDSupervisor     = 0xFFFFFFFF;
            pFlagEntryData[lIndex].lReadsCount              = 0;
            pFlagEntryData[lIndex].pulClientIDReadsCount    = NULL;
        }
        else lIndex = lFound;
        
        //Supervisor mode already set to another client id ?
        if ( (pFlagEntryData[lIndex].ulClientIDSupervisor!=0xFFFFFFFF) && (pFlagEntryData[lIndex].ulClientIDSupervisor!=ulClientID) )
        {
            CriticalSectionLeave ( &csFlagEntry );
            return -4;
        }
        
        switch (lMode)
        {
            case RDBSERVER_RIGHTSET_SUPERVISOR_DISABLE: //Disable Supervisor mode !
                
                //Can't disable any supervisor mode if readonly is enabled !            
                if (pFlagEntryData[lIndex].lReadsCount>0)
                {
                    CriticalSectionLeave ( &csFlagEntry );
                    return -4;
                }
                    
                pFlagEntryData[lIndex].ulClientIDSupervisor = 0xFFFFFFFF;
                break;
                
            case RDBSERVER_RIGHTSET_READONLY_DISABLE: //Disable readonly mode !
                
                for (lReadCountIndex=0,lFound=-1;lReadCountIndex<pFlagEntryData[lIndex].lReadsCount;lReadCountIndex++)
                {
                    if (pFlagEntryData[lIndex].pulClientIDReadsCount[lReadCountIndex] == ulClientID)
                    {
                        lFound = lReadCountIndex;
                        break;
                    }
                }
                
                if (lFound!=-1)
                {
                    //Remove the client id
                    if (pFlagEntryData[lIndex].lReadsCount>1)
                        pFlagEntryData[lIndex].pulClientIDReadsCount[lFound] = pFlagEntryData[lIndex].pulClientIDReadsCount[pFlagEntryData[lIndex].lReadsCount-1];
                    
                    pFlagEntryData[lIndex].lReadsCount--;
                    
                    pFlagEntryData[lIndex].pulClientIDReadsCount = KERNEL_REALLOC ( 
                        pFlagEntryData[lIndex].pulClientIDReadsCount, 
                        pFlagEntryData[lIndex].lReadsCount*sizeof(unsigned long) );
                }
                else
                {
                    CriticalSectionLeave ( &csFlagEntry );
                    return -4; //Can't disable the readonly if not already exists !
                }
                
                break;    
                        
            case RDBSERVER_RIGHTSET_READONLY_ENABLE: //Enable readonly mode !                
        
                //Add the client id to the list !
                pFlagEntryData[lIndex].pulClientIDReadsCount = KERNEL_REALLOC ( 
                    pFlagEntryData[lIndex].pulClientIDReadsCount, 
                    (pFlagEntryData[lIndex].lReadsCount+1)*sizeof(unsigned long) );
                pFlagEntryData[lIndex].pulClientIDReadsCount[pFlagEntryData[lIndex].lReadsCount] = ulClientID;
                pFlagEntryData[lIndex].lReadsCount++;
                
                break;
                
            case RDBSERVER_RIGHTSET_SUPERVISOR_ENABLE: //Enable Supervisor mode !
            
                //Can't enable any supervisor mode if readonly is enabled !            
                if (pFlagEntryData[lIndex].lReadsCount>0)
                {
                    CriticalSectionLeave ( &csFlagEntry );
                    return -4;
                }
                    
                pFlagEntryData[lIndex].ulClientIDSupervisor = ulClientID;
                
                break;
        }
        
        //Optimize the base..
        for (lIndex=0;lIndex<lFlagEntryCount;lIndex++)
        {
            if ( (pFlagEntryData[lIndex].lReadsCount==0) &&
                 (pFlagEntryData[lIndex].ulClientIDSupervisor==0xFFFFFFFF) )
            {
                if (lFlagEntryCount>1)
                {
                    memcpy ( 
                        &(pFlagEntryData[lIndex]),
                        &(pFlagEntryData[lFlagEntryCount-1]),
                        sizeof(RDBFLAGENTRY) );
                }
                        
                lFlagEntryCount--;
                lIndex--;    
            }
        }
        if ((lFlagEntryCount+32)<lFlagEntryCountAlloc)
        {                
            lFlagEntryCountAlloc = lFlagEntryCount + 16;
            pFlagEntryData = (RDBFLAGENTRY *)KERNEL_REALLOC ( pFlagEntryData, lFlagEntryCountAlloc * sizeof(RDBFLAGENTRY) );
        }
        
        
        CriticalSectionLeave ( &csFlagEntry );
    
        return 0;
    }

    
    long  RDBServerRightsGet ( SRDB ** pRDB, char * pszRootDirectoryName, char * pszEntryName, long lMode, unsigned long ulClientID  )
    {
        char szTmp[4096];
        unsigned char bNameHash[32];
        long lIndex, lFound;
        long lReturnValue;
        
        sprintf ( szTmp, "%s%s%s", (*pRDB)->Header.szRDBName, pszRootDirectoryName, pszEntryName );
        sha256 ( (unsigned char *)szTmp, (unsigned long)strlen(szTmp)+1, bNameHash );
        
        CriticalSectionEnter ( &csFlagEntry );
        
        //Searching for the entry...
        for (lIndex=0,lFound=-1;lIndex<lFlagEntryCount;lIndex++)
        {
            if (memcmp(bNameHash,pFlagEntryData[lIndex].bNameHash,32)==0)
            {
                lFound = lIndex;
                break;
            }
        }
        
        //Could not found the entry...
        if (lFound==-1)
        {
            CriticalSectionLeave ( &csFlagEntry );
            return 0;
        }
        else lIndex = lFound;
        
        //Supervisor mode already set to another client id ?
        if ( (pFlagEntryData[lIndex].ulClientIDSupervisor!=0xFFFFFFFF) && (pFlagEntryData[lIndex].ulClientIDSupervisor!=ulClientID) )
        {
            CriticalSectionLeave ( &csFlagEntry );
            return -4;
        }
            
        switch (lMode)
        {
            case RDBSERVER_RIGHTGET_SUPERVISOR:
                if (pFlagEntryData[lIndex].ulClientIDSupervisor==0xFFFFFFFF) lReturnValue = 0;
                else lReturnValue = 1;
                break;
                
            case RDBSERVER_RIGHTGET_READONLY:
            
                /*
                for (lReadCountIndex=0,lFound=-1;lReadCountIndex<pFlagEntryData[lIndex].lReadsCount;lReadCountIndex++)
                {
                    if (pFlagEntryData[lIndex].pulClientIDReadsCount[lReadCountIndex] == ulClientID)
                    {
                        lFound = lReadCountIndex;
                        break;
                    }
                }
                */
                lReturnValue = pFlagEntryData[lIndex].lReadsCount;
                if (lReturnValue>127) lReturnValue = 127;
                
                
                //if (lFound==-1) lReturnValue = 0;
                //else lReturnValue = 1;
                break;
        }
        
        CriticalSectionLeave ( &csFlagEntry );
            
        return lReturnValue;
    }    


    long  RDBServerRightsFree ( unsigned long ulClientID )
    {
        long lIndex, lReadCountIndex;
        
        CriticalSectionEnter ( &csFlagEntry );
        
        //Delete the client id
        for (lIndex=0;lIndex<lFlagEntryCount;lIndex++)
        {
            if (pFlagEntryData[lIndex].ulClientIDSupervisor==ulClientID)
                pFlagEntryData[lIndex].ulClientIDSupervisor = 0xFFFFFFFF;
                
            for (lReadCountIndex=0;lReadCountIndex<pFlagEntryData[lIndex].lReadsCount;lReadCountIndex++)
            {
                if (pFlagEntryData[lIndex].pulClientIDReadsCount[lReadCountIndex]==ulClientID)
                {
                    if (pFlagEntryData[lIndex].lReadsCount>1)
                        pFlagEntryData[lIndex].pulClientIDReadsCount[lReadCountIndex] = pFlagEntryData[lIndex].pulClientIDReadsCount[pFlagEntryData[lIndex].lReadsCount-1];
                    
                    pFlagEntryData[lIndex].lReadsCount--;
                    
                    if (pFlagEntryData[lIndex].lReadsCount)
                        pFlagEntryData[lIndex].pulClientIDReadsCount = KERNEL_REALLOC ( 
                            pFlagEntryData[lIndex].pulClientIDReadsCount, 
                            pFlagEntryData[lIndex].lReadsCount*sizeof(unsigned long) );
                    else
                    {
                        KERNEL_FREE ( pFlagEntryData[lIndex].pulClientIDReadsCount );
                        pFlagEntryData[lIndex].pulClientIDReadsCount = NULL;
                    }
                    
                    
                    lReadCountIndex--;
                }
            }
        }
        
        //Optimize the base..
        for (lIndex=0;lIndex<lFlagEntryCount;lIndex++)
        {
            if ( (pFlagEntryData[lIndex].lReadsCount==0) &&
                 (pFlagEntryData[lIndex].ulClientIDSupervisor==0xFFFFFFFF) )
            {
                if (lFlagEntryCount>1)
                {
                    memcpy ( 
                        &(pFlagEntryData[lIndex]),
                        &(pFlagEntryData[lFlagEntryCount-1]),
                        sizeof(RDBFLAGENTRY) );
                }
                        
                lFlagEntryCount--;
                lIndex--;    
            }
        }
        if ((lFlagEntryCount+32)<lFlagEntryCountAlloc)
        {                
            lFlagEntryCountAlloc = lFlagEntryCount + 16;
            pFlagEntryData = (RDBFLAGENTRY *)KERNEL_REALLOC ( pFlagEntryData, lFlagEntryCountAlloc * sizeof(RDBFLAGENTRY) );
        }

        CriticalSectionLeave ( &csFlagEntry );        
        
        return 0;
    }        
        

#ifdef ce_code_est_bugue__pas_touche

    /*
        Rights Security Enforcement
    */

    typedef struct {
        char            szOwnerIP[32];
        unsigned long   ulIdentifiant;
        char            *pszRDBName;
        char            *pszRootDirectoryName;
        char            *pszEntryName;
        char            *pStringMemory;         //Buffer pour stocker les 3 cha�nes

        long            lFlagReadOnlyCount;
        long            bFlagSupervisor;
    } RIGHTSENTRY;

    typedef struct {
        //Une liste pour gerer les droits (en cas de perte)
        CRITICALSECTION         CriticalSectionRights;
        RIGHTSENTRY             *pRightsEntry;
        long                    lNbRightsEntry;
    } SERVERRIGHTS;

    SERVERRIGHTS                ServerRights;





    long    RDBServerRightsOpen                  ( void );
    long    RDBServerRightsClose                 ( SRDB ** pRDBBase, long NbMaxBases );

    long    RDBServerRightsSetSupervisor         ( unsigned long ulIdentifiant, char * pszRDBName, char * pszRootDirectoryName, char * pszEntryName, long bFlag );
    long    RDBServerRightsSetReadOnly           ( unsigned long ulIdentifiant, char * pszRDBName, char * pszRootDirectoryName, char * pszEntryName, long bFlag );
    long    RDBServerRightsGetReadOnly           ( unsigned long ulIdentifiant, char * pszRDBName, char * pszRootDirectoryName, char * pszEntryName, long * pbFlag );

    long    RDBServerRightsFreeFlags             ( unsigned long ulIdentifiant, SRDB ** pRDBBase, long NbMaxBases );
    long    RDBServerRightsFreeAllFlags          ( SRDB ** pRDBBase, long NbMaxBases );

    long    _RDBServerRightsFindOrAdd            ( RIGHTSENTRY ** pRightsEntry, unsigned long ulIdentifiant, char * pszRDBName, char * pszRootDirectoryName, char * pszEntryName, long bAddEnabled );

    long    RDBServerRightsOpen                  ( void )
    {
        ServerRights.lNbRightsEntry   = 0;
        ServerRights.pRightsEntry     = NULL;

        CriticalSectionCreate ( &ServerRights.CriticalSectionRights );

        return 0;
    }


    long RDBServerRightsClose                    ( SRDB ** pRDBBase, long NbMaxBases )
    {
        long i;

        RDBServerRightsFreeAllFlags ( pRDBBase, NbMaxBases );

        CriticalSectionDelete ( &ServerRights.CriticalSectionRights );

        for (i=0;i<ServerRights.lNbRightsEntry;i++)
        {
            KERNEL_FREE ( ServerRights.pRightsEntry[i].pStringMemory );
        }
        KERNEL_FREE ( ServerRights.pRightsEntry );

        ServerRights.lNbRightsEntry   = 0;
        ServerRights.pRightsEntry     = NULL;

        return 0;
    }

    long    RDBServerRightsSetSupervisor         ( unsigned long ulIdentifiant, char * pszRDBName, char * pszRootDirectoryName, char * pszEntryName, long bFlag )
    {
        RIGHTSENTRY * pRightsEntry;

        CriticalSectionEnter ( &ServerRights.CriticalSectionRights );

        //Recherche l'entree
        _RDBServerRightsFindOrAdd ( &pRightsEntry, ulIdentifiant, pszRDBName, pszRootDirectoryName, pszEntryName, 1 );

        //Fixe le drapeau
        pRightsEntry->bFlagSupervisor = bFlag;

        CriticalSectionLeave ( &ServerRights.CriticalSectionRights );

        return 0;
    }

    long    RDBServerRightsSetReadOnly           ( unsigned long ulIdentifiant, char * pszRDBName, char * pszRootDirectoryName, char * pszEntryName, long bFlag )
    {
        RIGHTSENTRY * pRightsEntry;

        CriticalSectionEnter ( &ServerRights.CriticalSectionRights );

        //Recherche l'entree
        _RDBServerRightsFindOrAdd ( &pRightsEntry, ulIdentifiant, pszRDBName, pszRootDirectoryName, pszEntryName, 1 );

        //Fixe le drapeau
        pRightsEntry->lFlagReadOnlyCount += (bFlag)?(1):(-1);

        CriticalSectionLeave ( &ServerRights.CriticalSectionRights );

        return 0;
    }

    long    RDBServerRightsGetReadOnly           ( unsigned long ulIdentifiant, char * pszRDBName, char * pszRootDirectoryName, char * pszEntryName, long * pbFlag )
    {
        RIGHTSENTRY     *pRightsEntry;
        long            lReturn;

        CriticalSectionEnter ( &ServerRights.CriticalSectionRights );

        //Recherche l'entree
        lReturn = _RDBServerRightsFindOrAdd ( &pRightsEntry, ulIdentifiant, pszRDBName, pszRootDirectoryName, pszEntryName, 0 );

        //Recuperation du drapeau
        if (!lReturn)
        {
            if (pRightsEntry->lFlagReadOnlyCount>0) *pbFlag = 1;
            else                                    *pbFlag = 0;
        }

        CriticalSectionLeave ( &ServerRights.CriticalSectionRights );

        return lReturn;
    }


    long    RDBServerRightsFreeFlags             ( unsigned long ulIdentifiant, SRDB ** pRDBBase, long NbMaxBases )
    {
        long i;
        SRDB                **pRDB;
        NETWORKDATALINK     NDLClient;
        long                lRetour;

        CriticalSectionEnter ( &ServerRights.CriticalSectionRights );

        NDLClient.Identifiant = ulIdentifiant;

        for (i=0;i<ServerRights.lNbRightsEntry;i++)
        {
            if ((ServerRights.pRightsEntry[i].ulIdentifiant==ulIdentifiant)||(ulIdentifiant==0xFFFFFFFF))
            {
                //Point d'acces de la base
                NDLClient.Identifiant = ServerRights.pRightsEntry[i].ulIdentifiant;
                pRDB = _RDBServerGetBase ( ServerRights.pRightsEntry[i].pszRDBName, pRDBBase, NbMaxBases, &NDLClient );

                //Si le programme n'a pas libere tous les ReadsCount
                if (ServerRights.pRightsEntry[i].lFlagReadOnlyCount>0)
                {
                    lRetour  = RDBReadsCountSet ( pRDB, ServerRights.pRightsEntry[i].pszRootDirectoryName, ServerRights.pRightsEntry[i].pszEntryName, 0 );
                    //if (lRetour) FlagReadUpdate ( pRDB, ServerRights.pRightsEntry[i].pszRootDirectoryName, ServerRights.pRightsEntry[i].pszEntryName, 0 );
                }

                //Si le programme n'a pas libere le supervisor
                if (ServerRights.pRightsEntry[i].bFlagSupervisor)
                {
                    lRetour = RDBSupervisorSet ( pRDB, ServerRights.pRightsEntry[i].pszRootDirectoryName, ServerRights.pRightsEntry[i].pszEntryName, 0 );
                }

                //Ferme le point d'acces a la base
                _RDBServerReleaseBase ( pRDB, &NDLClient );
                //RDBEntry_KillSupervisor ( pRDB, ulIdentifiant );

                //Liberation de cette entree de droits
                KERNEL_FREE ( ServerRights.pRightsEntry[i].pStringMemory );

                memcpy ( &ServerRights.pRightsEntry[i],
                         &ServerRights.pRightsEntry[ServerRights.lNbRightsEntry-1],
                         sizeof(RIGHTSENTRY) );
                
                ServerRights.lNbRightsEntry--;
                ServerRights.pRightsEntry = (RIGHTSENTRY *)KERNEL_REALLOC ( ServerRights.pRightsEntry, ServerRights.lNbRightsEntry * sizeof(RIGHTSENTRY) );
                i--;
            }
        }
        
        CriticalSectionLeave ( &ServerRights.CriticalSectionRights );   

        return 0;
    }

    long    RDBServerRightsFreeAllFlags          ( SRDB ** pRDBBase, long NbMaxBases )
    {
        RDBServerRightsFreeFlags ( 0xFFFFFFFF, pRDBBase, NbMaxBases );

        return 0;
    }



    long    _RDBServerRightsFindOrAdd ( RIGHTSENTRY ** pRightsEntry, unsigned long ulIdentifiant, char * pszRDBName, char * pszRootDirectoryName, char * pszEntryName, long bAddEnabled )
    {
        long i;
        char szRDBName[256];
        char szRootDirectoryName[256];
        char szEntryName[256];
        long bFoundEntry;

        //Passage en majuscule pour comparaison rapide
        strcpy ( szRDBName, pszRDBName );
        strcpy ( szRootDirectoryName, pszRootDirectoryName );
        strcpy ( szEntryName, pszEntryName );
        strucasefast ( szRDBName );
        strucasefast ( szRootDirectoryName );
        strucasefast ( szEntryName );


        bFoundEntry = 0;
        for (i=0;i<ServerRights.lNbRightsEntry;i++)
        {
            if (ServerRights.pRightsEntry[i].ulIdentifiant==ulIdentifiant)
            {
                if ( (strcmp(ServerRights.pRightsEntry[i].pszRDBName,             szRDBName           )==0) &&
                     (strcmp(ServerRights.pRightsEntry[i].pszRootDirectoryName,   szRootDirectoryName )==0) &&
                     (strcmp(ServerRights.pRightsEntry[i].pszEntryName,           szEntryName         )==0) )
                {
                    //Une entree existante a ete trouvee
                    *pRightsEntry = &ServerRights.pRightsEntry[i];
                    bFoundEntry = 1;
                }
            }
        }

        if (!bFoundEntry)
        {
            if (bAddEnabled)
            {
                ServerRights.pRightsEntry = (RIGHTSENTRY *)KERNEL_REALLOC ( ServerRights.pRightsEntry, (ServerRights.lNbRightsEntry+1)*sizeof(RIGHTSENTRY) );
                *pRightsEntry = &ServerRights.pRightsEntry[ServerRights.lNbRightsEntry];
            
                memset ( *pRightsEntry, 0, sizeof(RIGHTSENTRY) );

                (*pRightsEntry)->ulIdentifiant          = ulIdentifiant;

                //Allocation de l'adresse RDB
                (*pRightsEntry)->pStringMemory          = (char *)KERNEL_ALLOC ( (unsigned long)(strlen(szRDBName) + strlen(szRootDirectoryName) + strlen(szEntryName) + 3) );
                (*pRightsEntry)->pszRDBName             = (*pRightsEntry)->pStringMemory;
                (*pRightsEntry)->pszRootDirectoryName   = (*pRightsEntry)->pStringMemory + strlen(szRDBName) + 1;
                (*pRightsEntry)->pszEntryName           = (*pRightsEntry)->pStringMemory + strlen(szRDBName) + strlen(szRootDirectoryName) + 2;

                //Copie de l'adresse RDB
                strcpy ( (*pRightsEntry)->pszRDBName,               szRDBName );
                strcpy ( (*pRightsEntry)->pszRootDirectoryName,     szRootDirectoryName );
                strcpy ( (*pRightsEntry)->pszEntryName,             szEntryName );

                ServerRights.lNbRightsEntry++;
            }
            else return -1;
        }


        return 0;
    }
#endif

