/*
===================================================================
Copyright (C) 2009-2010 Editions JOCATOP / Christophe Gossa
 
This file is part of Jocareso.

Jocareso is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Jocareso is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jocareso.  If not, see http://www.gnu.org/licenses/


Editions JOCATOP
Route de Saint Saturnin
84310 MORIERES
FRANCE

Christophe Gossa
christophe.gossa@jocatop.fr
===================================================================
*/


#ifdef _WIN32
    //#define WINVER 0x0500
    #define _WIN32_WINNT 0x0500
#endif

#include "criticalsection.h"

#ifdef _WIN32

    //Hack to WINVER 500
    //#ifdef WINVER
        //#undef WINVER
    //#endif
    
    //#ifdef _WIN32_WINNT
        //#undef _WIN32_WINNT
    //#endif


    #include <windows.h>
    #include <windowsx.h>
    #include <winuser.h>
    #include <tchar.h>
    #include <stdio.h>
    #include <string.h>
    #include <conio.h>
    #include <direct.h>
    #include <stdlib.h>
    #include <commctrl.h>
    #include <fcntl.h>
    #include <io.h>
    #include <time.h>
    #include <math.h>
    #include <commdlg.h>
    #include <malloc.h>
    #include <winbase.h>
    #include <limits.h>
    
    
    //#if (sizeof(CRITICAL_SECTION)>(4*CRITICALSECTION_SIZE))
    //#error "CRITICALSECTION.dwData is not enough big !"
    //#endif
    
    long CriticalSectionEnter ( CRITICALSECTION * pCriticalSection )
    {
        CRITICAL_SECTION *pwin32CriticalSection;
        pwin32CriticalSection = (CRITICAL_SECTION *)pCriticalSection->dwData;
        
        if (pCriticalSection->lCreated==1)
            EnterCriticalSection ( pwin32CriticalSection );
        return 0;
    }

    long CriticalSectionLeave ( CRITICALSECTION * pCriticalSection )
    {
        CRITICAL_SECTION *pwin32CriticalSection;
        pwin32CriticalSection = (CRITICAL_SECTION *)pCriticalSection->dwData;
    
        if (pCriticalSection->lCreated==1)
            LeaveCriticalSection ( pwin32CriticalSection );
        return 0;
    }

    long CriticalSectionCreate ( CRITICALSECTION * pCriticalSection )
    {
        CRITICAL_SECTION *pwin32CriticalSection;
        
        if (CRITICALSECTION_SIZE*4<sizeof(CRITICAL_SECTION))
            exit (1);
        
        pwin32CriticalSection = (CRITICAL_SECTION *)pCriticalSection->dwData;
        

        //InitializeCriticalSection ( pwin32CriticalSection );
        
        /*
            Cette fonction de "Hack" permet d'outre-passer le bug de Windows 2000 lorsqu'il y a peu de memoire.
            Sous Windows 2000, lorque le bit 31 est fixe, le systeme effectue une pre-allocation ce qui garantie
            un fonctionnement de la section critique sans faille, meme s'il ne reste que peu de memoire.

            La valeur 0x1000 (4096) pour le spin count est standard (le systeme utilise une valeur par defaut de 4000).
        */
        InitializeCriticalSectionAndSpinCount ( pwin32CriticalSection, 0x80001000 );
        pCriticalSection->lCreated = 1;
        return 0;
    }

    long CriticalSectionDelete ( CRITICALSECTION * pCriticalSection )
    {
        CRITICAL_SECTION *pwin32CriticalSection;
        pwin32CriticalSection = (CRITICAL_SECTION *)pCriticalSection->dwData;
    
        if (pCriticalSection->lCreated==1)
        {
            DeleteCriticalSection ( pwin32CriticalSection );
            memset ( pCriticalSection, 0, sizeof(CRITICALSECTION) );
        }
        return 0;
    }

#else   
    #include <stdio.h>
    #include <string.h>
    #include <pthread.h>
    #include <unistd.h>
    #include <semaphore.h>
    #include <errno.h>

    long CriticalSectionEnter ( CRITICALSECTION * pCriticalSection )
    {
        pthread_mutex_t *pcs_mutex;
        pcs_mutex = (pthread_mutex_t *)pCriticalSection->dwData;
        
        if (pCriticalSection->lCreated==1)
            pthread_mutex_lock( pcs_mutex );

        return 0;
    }

    long CriticalSectionLeave ( CRITICALSECTION * pCriticalSection )
    {
        pthread_mutex_t *pcs_mutex;
        pcs_mutex = (pthread_mutex_t *)pCriticalSection->dwData;
    
        if (pCriticalSection->lCreated==1)
            pthread_mutex_unlock( pcs_mutex );
            
        return 0;
    }


    long CriticalSectionCreate ( CRITICALSECTION * pCriticalSection )
    {
        pthread_mutex_t *pcs_mutex;
        
        if (CRITICALSECTION_SIZE*4<sizeof(pthread_mutex_t))
            exit (1);
            
        pcs_mutex = (pthread_mutex_t *)pCriticalSection->dwData;
    
        pthread_mutex_init ( pcs_mutex, NULL );
        pCriticalSection->lCreated = 1;
        return 0;
    }

    
    long CriticalSectionDelete ( CRITICALSECTION * pCriticalSection )
    {
        if (pCriticalSection->lCreated==1)
            memset ( pCriticalSection, 0, sizeof(CRITICALSECTION) );
            
        return 0;
    }
    
#endif

