/*
===================================================================
Copyright (C) 2009-2010 Editions JOCATOP / Christophe Gossa
 
This file is part of Jocareso.

Jocareso is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Jocareso is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jocareso.  If not, see http://www.gnu.org/licenses/


Editions JOCATOP
Route de Saint Saturnin
84310 MORIERES
FRANCE

Christophe Gossa
christophe.gossa@jocatop.fr
===================================================================
*/


#include "jocareso.h"
#include <stdio.h>
#include "RDBServerClientInterface.h"

#ifdef _WIN32
    #include <signal.h>
    #include <conio.h>
#else
    //#include <iostream.h>
    #include <signal.h>
    
    int _kbhit ( void )
    {
    
        struct timeval tv;
        fd_set read_fd;

        tv.tv_sec=0;
        tv.tv_usec=0;
        FD_ZERO(&read_fd);
        FD_SET(0,&read_fd);

        if(select(1, &read_fd, NULL, NULL, &tv) == -1)
            return 0;

        if(FD_ISSET(0,&read_fd))
            return 1;

        return 0;
    }

    int _getch_2 ( void )
    {
        struct termios oldt, newt;
        int ch;

        tcgetattr( STDIN_FILENO, &oldt );
        newt = oldt;
        newt.c_lflag &= ~( ICANON | ECHO );
        tcsetattr( STDIN_FILENO, TCSANOW, &newt );
        ch = getchar();
        tcsetattr( STDIN_FILENO, TCSANOW, &oldt );
        return ch;
    }
    
    int _getch ( void )
    {
        static struct termios term, back;
        int ret=-1;
 
        tcgetattr (0, &term);       /* On recupere les info du terminal */
        memcpy (&back, &term, sizeof(term));
 
        term.c_lflag &= ~(ICANON|ECHO); /* On modifie les options du terminal */
        term.c_cc[VTIME] = 0;       /* Temps d'attente infini */
        term.c_cc[VMIN]  = 1;       /* Nombre minimun de caractere */
 
        tcsetattr(0, TCSANOW, &term);   /* Modif des attribut de stdin */
        ret = getchar();                /* Attente d'un caractere */
        tcsetattr(0, TCSANOW, &back);   /* Restauration des attributs */
 
        return ret;
    }

#endif

    
    

    
    void signalExitApp ( int n )
    {
        GlobalData.lExitApplication = 1;
    }


    long JOCARESO_Application ( void )
    {
        long i;
        long lLoop;
        long lKey;
        long lPort;
        long lFound;
        char szTmp[1024];
        long lSaveCounter;

        // Checking the R/W working directory access...
        if (VerifyReadWriteModify ( ))
        {
            sprintf ( szTmp, "Vous ne pouvez pas correctement utiliser "DAEMON_NAME". Votre ordinateur limite les droits d'acces aux fichiers de "DAEMON_NAME".\n\nSolution 1 - Choisissez un autre dossier d'installation\nSolution 2 - Quittez et attribuez des droits moins restrictifs (ex. Controle total pour Tout le monde).\n\nContactez votre revendeur ou votre installateur pour plus de renseignements et pour resoudre le probleme.\n\nSouhaitez-vous continuer sans pouvoir enregistrer vos donnees ?" );
            COM_Printf ( (char *)"Probleme : Droits insuffisants\n\n%s\n", szTmp );
            return -1;
        }
        
        //Run on a single instance only !
        if (SYS_SingleInstanceOpen ( ))
        {
            COM_Printf ( (char *)"Erreur : Vous ne pouvez pas lancer plusieurs instances de cette application." );
            return -1;
        }
    
        //********************************************************************************************************
        // Set global vars
        memset ( &GlobalData, 0, sizeof(GlobalData) );        
        ConfigSetDefault ( );
        GlobalData.RDBBaseNb        = 512;
        
        CriticalSectionCreate ( &GlobalData.csConfigAccess );
        CriticalSectionCreate ( &GlobalData.CriticalSection );
        
        ConfigLoad ( );
        //ServerClientInterface_MakePKey ( CommandData.szPassword );
        if (CommandData.iForceAdminPasswordReinit)
            memset ( ConfigDataEx.bPasswordAdminSHA512, 0, 64 );
            
        

        GlobalData.lExitApplication = 0;
        strcpy ( GlobalData.szExitApplicationWhy, "" );
        
        ListBaseOpen ( );

        
        
        

        //Disable Power saving under Windows
        ScreenSaverManagerOpen ( &GlobalData.ScreenSaver );
        ScreenSaverManagerSet ( &GlobalData.ScreenSaver, 1, 0 );

        //Initialise le temps
        CriticalSectionCreate ( &GlobalData.CriticalSectionCurrentTime );
        CriticalSectionEnter ( &GlobalData.CriticalSectionCurrentTime );
        time ( &GlobalData.ltimeCurrentTime );
        CriticalSectionLeave ( &GlobalData.CriticalSectionCurrentTime );

        //Get current params...
        //if (strlen(CommandData.szPassword)) strcpy ( szTmp, "Enabled" );
        //else strcpy ( szTmp, "Disabled" );
        COM_Printf ( (char *)"Params-RootPath: %s", CommandData.szRootPath );
		SysGetLogPath(szTmp);
		COM_Printf ( (char *)"Params-SubPath: %s", szTmp);
        COM_Printf ( (char *)"Params-Port: %s", CommandData.szBasePort );
        
        //COM_Printf ( (char *)"Params-ListenMode: %s", CommandData.szListenMode );
        //COM_Printf ( (char *)"Params-Password: %s", szTmp );
		if (strlen(CommandData.szBasePort))
			lPort = atol ( CommandData.szBasePort );
		else
			lPort = 0;
        
        // Ports checking
        CriticalSectionEnter ( &GlobalData.CriticalSection );
        CriticalSectionEnter ( &GlobalData.csConfigAccess );

		// Default port = 2000
		//ConfigData.NetworkPort[0] = 2000;
        if ( (lPort>=20) && (lPort<65536) )
			ConfigData.NetworkPort[0] = lPort;

		if (ConfigData.NetworkPort[0]==0)
			ConfigData.NetworkPort[0] = 2000;

        for (i=1;i<16;i++)
            ConfigData.NetworkPort[i] = 0;


		/*
		
        
        if ( (lPort>=20) && (lPort<65536) )
        {
            //Find the same port...
            lFound = -1;
            for (i=0;i<16;i++)
            {
                if (ConfigData.NetworkPort[i]==lPort) lFound = -2;
                else if ( (!ConfigData.NetworkPort[i]) && (lFound!=-2) ) lFound = i;
            }
            
            if (lFound>=0)
                ConfigData.NetworkPort[lFound] = lPort;
        }
        
        for (i=0;i<16;i++)
        {
            if (ConfigData.NetworkPort[i]) i = 20;
        }
        if (i<20)
            ConfigData.NetworkPort[0] = 2000;
		*/
            
        for (i=0;i<16;i++)
        {
            if (ConfigData.NetworkPort[i])
                COM_Printf ( (char *)"Listen slot #%d : TCP %d Port", i, ConfigData.NetworkPort[i] );
        }
            
        CriticalSectionLeave ( &GlobalData.CriticalSection );
        CriticalSectionLeave ( &GlobalData.csConfigAccess );

        ThreadCreate ( 
            &GlobalData.ThreadMain, 
            ThreadMainProc, 
            0, 
            NULL );
        ThreadSetName ( &GlobalData.ThreadMain, "Main" );

        ThreadCreate ( 
            &GlobalData.ThreadWaitForConnection, 
            ThreadWaitForConnectionProc, 
            0, 
            NULL );
        ThreadSetName ( &GlobalData.ThreadWaitForConnection, "WFC" );

        GlobalData.ThreadServerListNbEntry = 1; //ConfigData.lNbThreadServer;
        for (i=0;i<GlobalData.ThreadServerListNbEntry;i++)
        {
            ThreadCreate ( 
                &GlobalData.ThreadServerList[i], 
                ThreadServerProc,
                0,
                NULL );
                ThreadSetName ( &GlobalData.ThreadServerList[i], "SP" );
        }

        //Get Host params
        GetLocalMachineName ( GlobalData.szServerComputerName, GlobalData.szServerHostName );

#ifdef _WIN32
        signal ( SIGINT, signalExitApp );
        signal ( SIGILL, signalExitApp );
        signal ( SIGFPE, signalExitApp );
        signal ( SIGSEGV, signalExitApp );
        signal ( SIGTERM, signalExitApp );
        signal ( SIGBREAK, signalExitApp );
        signal ( SIGABRT, signalExitApp );
#else
        //Pour éviter de se faire fermer comme un mal propre, il est plus utile d'utiliser le signal handling de linux
        signal ( SIGHUP, signalExitApp );
        signal ( SIGQUIT, signalExitApp );
        signal ( SIGTRAP, signalExitApp );                
        signal ( SIGIOT, signalExitApp );
        signal ( SIGBUS, signalExitApp );
        signal ( SIGKILL, signalExitApp );                
        signal ( SIGSTOP, signalExitApp );
        signal ( SIGABRT, signalExitApp );
        signal ( SIGFPE, signalExitApp );
        signal ( SIGILL, signalExitApp );
        signal ( SIGINT, signalExitApp );
        signal ( SIGSEGV, signalExitApp );                
        signal ( SIGTERM, signalExitApp );
#endif     
        
        //Boucle d'attente
        COM_Printf ( (char *)"  Daemon started." );
        COM_Printf ( (char *)"  's' => stop." );
        lLoop = 1;
        lSaveCounter = 0;
        do {
            //Compteur pour la sauvegarde toute les 3'24
            lSaveCounter=(lSaveCounter+1)&2047;
            if (!lSaveCounter) ConfigSave ( );
        
            if (_kbhit())
            {
                lKey = _getch();
                if (lKey=='s') lLoop = 0;   
            }
            SleepSet ( 100 );
            if (GlobalData.lExitApplication)
            {
                if (GlobalData.lExitApplication==1) lLoop = 0;
                else GlobalData.lExitApplication--;
            }
#ifdef _WIN32            
            if (_Win32Service_ServiceGetStopEvent()) lLoop = 0;
#else
            if (_LinuxDaemon_GetStopEvent()) lLoop = 0;
#endif
        } while (lLoop);
        COM_Printf ( (char *)"  Daemon stopped." );
        
        ThreadDelete ( &GlobalData.ThreadMain, 500 );
        ThreadDelete ( &GlobalData.ThreadWaitForConnection, 500 );

        for (i=0;i<GlobalData.ThreadServerListNbEntry;i++)
        {
            ThreadDelete ( &GlobalData.ThreadServerList[i], 500 );
        }

        CriticalSectionDelete ( &GlobalData.CriticalSectionCurrentTime );

        ListBaseClose ( );
        CriticalSectionDelete ( &GlobalData.CriticalSection );
        //KERNEL_FREE ( GlobalData.pscisClient );

        ScreenSaverManagerClose ( &GlobalData.ScreenSaver );
        
        if (GlobalData.lExitApplication<0)
        {
            COM_Printf ( GlobalData.szExitApplicationWhy );
        }
        
        ConfigSave ( );
        SYS_SingleInstanceClose ( );
        
		Z_MemoryClose();
        CriticalSectionDelete ( &GlobalData.csConfigAccess );
        
        return 0;
    }
    
    
#ifdef _WIN32
    #include "Winsvc.h"
    
    char * JOCARESO_GetTextError ( DWORD dwError );
    /*long JOCARESO_ServiceManagerConnect ( void );
    long JOCARESO_ServiceManagerDysconnect ( void );
    long JOCARESO_ServiceAppOpen ( void );    
    long JOCARESO_ServiceAppClose ( void );
    long JOCARESO_ServiceInstall ( char * pszBinaryPath );
    long JOCARESO_ServiceUninstall ( void );
    long JOCARESO_ServiceSetState ( long lStateNeed );
    long _JOCARESO_ServiceStart ( void );
    long _JOCARESO_ServiceStop ( void );
        
    SC_HANDLE WIN32_schManager;
    SC_HANDLE WIN32_schService;
    */

    char * JOCARESO_GetTextError ( DWORD dwError )
    {
        char * pszMessage;
        static char szErrorMessage[2048];
        

        FormatMessageA ( 
            FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM,      //dwFlags
            NULL,                                                           //lpSource
            dwError,                                                        //dwMessageId
            MAKELANGID(LANG_NEUTRAL,SUBLANG_NEUTRAL),                       //dwLanguageId (Default language)
            (LPSTR)&pszMessage,                                             //lpBuffer
            0,                                                              //nSize
            NULL );                                                         //Arguments

        sprintf ( szErrorMessage, "Error %d : %s", dwError, pszMessage );
        
        LocalFree ( pszMessage );
        
        return szErrorMessage;
    }
    
    /*
    long JOCARESO_ServiceManagerConnect ( void )
    {
        WIN32_schManager = OpenSCManager ( NULL, NULL, SC_MANAGER_ALL_ACCESS );
        
        if (!WIN32_schManager)
        {
            COM_Printf ( "%s - Can't open SCManager", JOCARESO_GetTextError ( GetLastError() ) );
            return -1;
        }
        return 0;    
    }
    
    long JOCARESO_ServiceManagerDysconnect ( void )
    {
        if (!WIN32_schManager)
            return -1;
        
        CloseServiceHandle ( WIN32_schManager );
        WIN32_schManager = NULL;
        return 0; 
    }
    
    long JOCARESO_ServiceAppOpen ( void )
    {
        if (!WIN32_schManager)
            return -1;

        WIN32_schService = OpenServiceA ( 
            WIN32_schManager,               // SCM database 
            DAEMON_NAME,                    // name of service 
            SERVICE_ALL_ACCESS );           // need change config access     
        
        if (!WIN32_schService)
        {
            COM_Printf ( "%s - Can't open service", JOCARESO_GetTextError ( GetLastError() ) );
            return -1;
        }
        return 0;        
    }

    long JOCARESO_ServiceAppClose ( void )
    {
        if (!WIN32_schManager)
            return -1;
            
        if (!WIN32_schService)
            return -1;

        CloseServiceHandle ( WIN32_schService );
        WIN32_schService = NULL;
        return 0;        
    }
    
    long JOCARESO_ServiceGetState ( void )
    {
        if (!WIN32_schManager)
            return -1;
    
        //GetServiceDisplayName (
        //    WIN32_scHandleManager,
        //    DAEMONNAME,
        //    szDisplayName,
            
        
        return 0;
    }
    
    long JOCARESO_ServiceInstall ( char * pszBinaryPath )
    {
        SERVICE_DESCRIPTIONA sd;
        char szDesc[256];
        
        if (!WIN32_schManager)
            return -1;
    
        WIN32_schService = CreateServiceA ( 
            WIN32_schManager,               // SCM database 
            DAEMON_NAME,                    // name of service 
            DAEMON_NAME,                    // service name to display 
            SERVICE_ALL_ACCESS,             // desired access 
            SERVICE_WIN32_OWN_PROCESS,      // service type 
            SERVICE_AUTO_START,             // start type 
            SERVICE_ERROR_NORMAL,           // error control type 
            pszBinaryPath,                  // path to service's binary 
            NULL,                           // no load ordering group 
            NULL,                           // no tag identifier 
            NULL,                           // no dependencies 
            NULL,                           // LocalSystem account 
            NULL );                         // no password 

        if (!WIN32_schService)
        {
            COM_Printf ( "%s - Can't create service", JOCARESO_GetTextError ( GetLastError() ) );
            return -1;
        }

    
        strcpy ( szDesc, DAEMON_DESC );        
        sd.lpDescription = szDesc;
        if (!ChangeServiceConfig2A (
            WIN32_schService,          
            SERVICE_CONFIG_DESCRIPTION,
            &sd ) )
        {
            COM_Printf ( "%s - Can't add a service description", JOCARESO_GetTextError ( GetLastError() ) );
        }

        CloseServiceHandle ( WIN32_schService );
        WIN32_schService = NULL;
        return 0;
    }
    
    long JOCARESO_ServiceUninstall ( void )
    {
        long lRetour;
        
        if (!WIN32_schManager)
            return -1;
    
        if (JOCARESO_ServiceAppOpen())
            return -1;

        if (!DeleteService ( WIN32_schService ) )
        {
            COM_Printf ( "%s - Can't uninstall service", JOCARESO_GetTextError ( GetLastError() ) );
            lRetour = -1;
        }
        else lRetour = 0;
        
        JOCARESO_ServiceAppClose ( );
        
        return lRetour;
    }
    
    long JOCARESO_ServiceSetState ( long lStateNeed )
    {
        long lRetour;
        DWORD dwStartType;
        long lModeFunction;
        
        if (!WIN32_schManager)
            return -1;
    
        if (JOCARESO_ServiceAppOpen())
            return -1;
            
        lModeFunction = 0;
        switch (lStateNeed)
        {
            case 0:
                dwStartType = SERVICE_DISABLED;
                break;
                
            case 1:
                dwStartType = SERVICE_DEMAND_START;
                break;          
                
            case 2:
                dwStartType = SERVICE_AUTO_START;
                break;
                
            case 3: //start                
                lModeFunction = 1;
                break;
                
            case 4: //stop
                lModeFunction = 2;
                break;
                
            
        }
                
                
        switch (lModeFunction)
        {
            case 0:        
                if (!ChangeServiceConfigA (
                    WIN32_schService,
                    SERVICE_NO_CHANGE,
                    dwStartType,
                    SERVICE_NO_CHANGE,
                    NULL,
                    NULL,
                    NULL,
                    NULL,
                    NULL,
                    NULL,
                    NULL ) )
                {
                    COM_Printf ( "%s - Can't set a new service state", JOCARESO_GetTextError ( GetLastError() ) );
                    lRetour = -1;
                }
                else lRetour = 0;
                break;
                
            case 1:
                if (_JOCARESO_ServiceStart()) 
                {
                    COM_Printf ( "Error : Can't start service" );
                    lRetour = -1;
                }
                else lRetour = 0;
                break;
                
            case 2:
                if (_JOCARESO_ServiceStop())
                {
                    COM_Printf ( "Error : Can't stop service" );
                    lRetour = -1;
                }
                else lRetour = 0;
                break;
        }

        JOCARESO_ServiceAppClose ( );
        
        return lRetour;
    }

    long _JOCARESO_ServiceStart ( void )
    {
        SERVICE_STATUS_PROCESS ssStatus; 
        DWORD dwOldCheckPoint; 
        DWORD dwStartTickCount;
        DWORD dwWaitTime;
        DWORD dwBytesNeeded;
        
        // Attempt to start the service.
        if (!StartService(
            WIN32_schService,  // handle to service 
            0,           // number of arguments 
            NULL) )      // no arguments 
        {
            COM_Printf ( "%s - Can't start service", JOCARESO_GetTextError ( GetLastError() ) );
            return -1; 
        }
        else COM_Printf ( "Service start pending..." );


        // Check the status until the service is no longer start pending. 
        if (!QueryServiceStatusEx( 
                WIN32_schService,             // handle to service 
                SC_STATUS_PROCESS_INFO, // info level
                (LPBYTE) &ssStatus,             // address of structure
                sizeof(SERVICE_STATUS_PROCESS), // size of structure
                &dwBytesNeeded ) )              // if buffer too small
        {
            return 0; 
        }
 
        // Save the tick count and initial checkpoint.
        dwStartTickCount = GetTickCount();
        dwOldCheckPoint = ssStatus.dwCheckPoint;

        while (ssStatus.dwCurrentState == SERVICE_START_PENDING) 
        { 
            // Do not wait longer than the wait hint. A good interval is 
            // one-tenth the wait hint, but no less than 1 second and no 
            // more than 10 seconds. 
            dwWaitTime = ssStatus.dwWaitHint / 10;

            if( dwWaitTime < 1000 )         dwWaitTime = 1000;
            else if ( dwWaitTime > 10000 )  dwWaitTime = 10000;

            Sleep ( dwWaitTime );

            // Check the status again. 
            if (!QueryServiceStatusEx( 
                WIN32_schService,             // handle to service 
                SC_STATUS_PROCESS_INFO, // info level
                (LPBYTE) &ssStatus,             // address of structure
                sizeof(SERVICE_STATUS_PROCESS), // size of structure
                &dwBytesNeeded ) )              // if buffer too small
                break; 
     
            if ( ssStatus.dwCheckPoint > dwOldCheckPoint )
            {
                // The service is making progress.
                dwStartTickCount = GetTickCount();
                dwOldCheckPoint = ssStatus.dwCheckPoint;
            }
            else
            {
                if(GetTickCount()-dwStartTickCount > ssStatus.dwWaitHint)
                {
                    // No progress made within the wait hint.
                    break;
                }
            }
        } 

        // Determine whether the service is running.
        if (ssStatus.dwCurrentState == SERVICE_RUNNING) 
        {
            COM_Printf ( "Service started successfully." ); 
        }
        else 
        { 
            COM_Printf ( "Error : Service not started.");
            //printf("  Current State: %d\n", ssStatus.dwCurrentState); 
            //printf("  Exit Code: %d\n", ssStatus.dwWin32ExitCode); 
            //printf("  Check Point: %d\n", ssStatus.dwCheckPoint); 
            //printf("  Wait Hint: %d\n", ssStatus.dwWaitHint); 
            return -1;
        } 
        return 0;
    }    
    
    long _JOCARESO_ServiceStop ( void )
    {
        SERVICE_STATUS_PROCESS ssp;
        
        DWORD dwStartTime = GetTickCount();
        DWORD dwBytesNeeded;
        DWORD dwTimeout = 30000; // 30-second time-out

        // Make sure the service is not already stopped.
        if ( !QueryServiceStatusEx( 
                WIN32_schService, 
                SC_STATUS_PROCESS_INFO,
                (LPBYTE)&ssp, 
                sizeof(SERVICE_STATUS_PROCESS),
                &dwBytesNeeded ) )
        {
            COM_Printf("QueryServiceStatusEx failed (%d)\n", GetLastError()); 
            return -1;
        }

        if ( ssp.dwCurrentState == SERVICE_STOPPED )
        {
            COM_Printf("Service is already stopped.\n");
            return 0;
        }

        // If a stop is pending, wait for it.

        while ( ssp.dwCurrentState == SERVICE_STOP_PENDING ) 
        {
            COM_Printf("Service stop pending...\n");
            Sleep( ssp.dwWaitHint );
            if ( !QueryServiceStatusEx( 
                     WIN32_schService, 
                     SC_STATUS_PROCESS_INFO,
                     (LPBYTE)&ssp, 
                     sizeof(SERVICE_STATUS_PROCESS),
                     &dwBytesNeeded ) )
            {
                COM_Printf("QueryServiceStatusEx failed (%d)\n", GetLastError()); 
                return -1;
            }

            if ( ssp.dwCurrentState == SERVICE_STOPPED )
            {
                COM_Printf("Service stopped successfully.\n");
                return 0;
            }

            if ( GetTickCount() - dwStartTime > dwTimeout )
            {
                COM_Printf("Service stop timed out.\n");
                return -1;
            }
        }

        // If the service is running, dependencies must be stopped first.

        //StopDependentServices();

        // Send a stop code to the service.

        if ( !ControlService( 
                WIN32_schService, 
                SERVICE_CONTROL_STOP, 
                (LPSERVICE_STATUS) &ssp ) )
        {
            COM_Printf( "ControlService failed (%d)\n", GetLastError() );
            return -1;
        }

        // Wait for the service to stop.

        while ( ssp.dwCurrentState != SERVICE_STOPPED ) 
        {
            Sleep( ssp.dwWaitHint );
            if ( !QueryServiceStatusEx( 
                    WIN32_schService, 
                    SC_STATUS_PROCESS_INFO,
                    (LPBYTE)&ssp, 
                    sizeof(SERVICE_STATUS_PROCESS),
                    &dwBytesNeeded ) )
            {
                COM_Printf( "QueryServiceStatusEx failed (%d)\n", GetLastError() );
                return -1;
            }

            if ( ssp.dwCurrentState == SERVICE_STOPPED )
                break;

            if ( GetTickCount() - dwStartTime > dwTimeout )
            {
                COM_Printf( "Wait timed out\n" );
                return -1;
            }
        }
        COM_Printf("Service stopped successfully\n");

        return 0;    
    }    
*/
#endif
