/*
===================================================================
Copyright (C) 2009-2010 Editions JOCATOP / Christophe Gossa
 
This file is part of Jocareso.

Jocareso is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Jocareso is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jocareso.  If not, see http://www.gnu.org/licenses/


Editions JOCATOP
Route de Saint Saturnin
84310 MORIERES
FRANCE

Christophe Gossa
christophe.gossa@jocatop.fr
===================================================================
*/

#include "jocareso.h"
#include "stdarg.h"


GLOBALDATA      GlobalData;
CONFIGDATA      ConfigData;
CONFIGDATAEX    ConfigDataEx;
COMMANDDATA     CommandData;



#ifdef _WIN32
    #pragma data_seg("Shared")
    jocareso_align_spec(32) long SYS_lInstanceCounter = 0; //Need to align this data to 4 byte boundary
    #pragma data_seg()
    #pragma comment(linker, "/section:Shared,rws")

#else
    #include <sys/file.h>
    static FILE * SYS_hFileFlock;
    static FILE * SYS_hFilePID;
    static char SYS_szPathFileFlock[512];
    static char SYS_szPathFilePID[512];
#endif
    static char SYS_szHomePath[256];
    static char SYS_szCurrentPath[256];    
    static char SYS_szInstallPath[256];
    static char SYS_szExecRun[1024];
    static long SYS_lHomePathInit = 0;
    static long SYS_lCurrentPathInit = 0;
    static long SYS_lRunSimpleMode = 0; //Vrai si on utilise le mode "run"

    void SYS_RunSimpleModeSet ( long lMode )
    {
        SYS_lRunSimpleMode = lMode;
    }

    long SYS_RunSimpleModeGet ( void )
    {
        return SYS_lRunSimpleMode;
    }

#ifdef _WIN32
	void SysGetLogPath(char * szTmp)
	{
		sprintf(szTmp, "C:\\%s-%i.%i.%i", DAEMON_NAME, JOCARESO_VERSION_GEN, JOCARESO_VERSION_MAJOR, JOCARESO_VERSION_MINOR);
	}
#else
	void SysGetLogPath(char * szTmp)
	{
		sprintf(szTmp, "/var/lib/%s", DAEMON_NAME);
	}
#endif
    
    long SYS_SingleInstanceGetPID ( long * lPID )
    {
#ifdef _WIN32
        return 0;
#else
        char szTmp[128];
		SysGetLogPath(szTmp);
		sprintf ( SYS_szPathFilePID, "%s/pid-single-instance.tmp", szTmp);

        SYS_hFilePID = fopen ( SYS_szPathFilePID, "rt" );
        if (SYS_hFilePID)
        {
            memset ( szTmp, 0, 32 );
            fgets ( szTmp, 31, SYS_hFilePID );
            fclose ( SYS_hFilePID );

            if (lPID) *lPID = atoi(szTmp);
            return 0;
        }
        return -1;

#endif
    }

    long SYS_SingleInstanceOpen ( void )
    {
#ifdef _WIN32
        BOOL bFirstInstance;
        
        bFirstInstance = (InterlockedIncrement(&SYS_lInstanceCounter)==1)?(1):(0);
        
        if (!bFirstInstance)
        {
            InterlockedDecrement(&SYS_lInstanceCounter);
            return -1;
        }
#else        
        FILE * hFileTest;
		char szLogPath[128];
		SysGetLogPath(szLogPath);
        
        sprintf ( SYS_szPathFileFlock, "%s/flock-single-instance.tmp", szLogPath );
        sprintf ( SYS_szPathFilePID, "%s/pid-single-instance.tmp", szLogPath );
        
/*        hFileTest = fopen ( SYS_szPathFileFlock, "rb" );
        if (hFileTest)
        {
            fclose ( hFileTest );
            return -1;
        }*/
        
        SYS_hFileFlock = fopen ( SYS_szPathFileFlock, "wb" );
        
        if (SYS_hFileFlock==NULL)
            return -1;
            
        if (flock ( fileno(SYS_hFileFlock), LOCK_EX|LOCK_NB )==-1)
        {
            fclose ( SYS_hFileFlock );
            return -1;
        }
        
        SYS_hFilePID = fopen ( SYS_szPathFilePID, "wt" );
        if (SYS_hFilePID)
        {
            fprintf ( SYS_hFilePID, "%d", getpid() );
            fclose ( SYS_hFilePID );
        }
        else
        {
            flock ( fileno(SYS_hFileFlock), LOCK_UN );
            fclose ( SYS_hFileFlock );
            remove ( SYS_szPathFileFlock );
            return -1;
        }
        
#endif        
        return 0;
    }
    
    long SYS_SingleInstanceClose ( void )
    {
#ifdef _WIN32
        InterlockedDecrement(&SYS_lInstanceCounter);
#else    
        flock ( fileno(SYS_hFileFlock), LOCK_UN );
        fclose ( SYS_hFileFlock );
        remove ( SYS_szPathFileFlock );
        remove ( SYS_szPathFilePID );
#endif
        return 0;
    }
    
    char * SYS_GetCurrentPath ( void )
    {
        if (!SYS_lCurrentPathInit)
        {
            SYS_lCurrentPathInit = 1;
#ifdef _WIN32
            GetCurrentDirectoryA ( 256, SYS_szCurrentPath );
#else            
            getcwd ( SYS_szCurrentPath, 256 );
#endif
        }
        return SYS_szCurrentPath;            
    }

    char * SYS_GetHomePath ( void )
    {
#ifndef _WIN32
        char * psz;
#endif        
        if (!SYS_lHomePathInit)
        {
            SYS_lHomePathInit = 1;
#ifdef _WIN32
            strcpy ( SYS_szHomePath, "" );
#else
            psz = getenv("HOME");
            if (psz!=NULL)
                strncpy ( SYS_szHomePath, psz, 256 );
            else
                strcpy ( SYS_szHomePath, "" );
#endif            
        }            
        return SYS_szHomePath;            
    }
    
    char * SYS_GetInstallPath ( void )
    {
        return SYS_szInstallPath;
    }
    
    void SYS_SetInstallPath ( char * pszPath )
    {
        strcpy ( SYS_szInstallPath, pszPath );    
    }
    
    void SYS_SetExecRunApp ( char * psz )
    {
        strcpy ( SYS_szExecRun, psz );
    }
    
    char * SYS_GetExecRunApp ( void )
    {
        return SYS_szExecRun;
    }




    static FILE    *COM_pFileLog;
    static long     COM_lOutputMode = 0; //0 = off, 1 = screen, 2 = file
    static char     COM_szOutputFile[1024];
    
    void COM_GetMode ( long * plOutputMode )
    {
        if (plOutputMode) *plOutputMode = COM_lOutputMode;
    }
    
    void COM_Open ( long lOutputMode, char * pszPath )
    {
        //char szFileName[1024];
        
        COM_lOutputMode = 0;
        
        switch (lOutputMode)
        {
            case 1: //Screen
                COM_lOutputMode = lOutputMode;
                break;
            
            case 2: //File
                sprintf ( COM_szOutputFile, "%s", pszPath );//
                strsupprallright ( COM_szOutputFile, '/' );
                strsupprallright ( COM_szOutputFile, '\\' );
                strcat ( COM_szOutputFile, "/log.txt" );
                COM_lOutputMode = lOutputMode;
                break;
            
            case 0:
            default:
                break;
        }
    }
    
    void COM_Close ( void )
    {

    }
    
    void COM_Printf ( char * fmt, ... )
    {
        static long lEnter=0;
        va_list     vaList;
        char		szMessage[1024];
		struct tm   *tmTime;
		time_t      tTime;
        
        if (!fmt) return;
        
        while (lEnter);
        lEnter = 1;
        
        va_start ( vaList, fmt );
#ifdef _WIN32        
        _vsnprintf ( szMessage, 1023, fmt, vaList );
#else
        vsnprintf ( szMessage, 1023, fmt, vaList );
#endif
        va_end ( vaList );
        
        switch (COM_lOutputMode)
        {
            case 1: //Screen
                printf ( szMessage );
                printf ( "\n" );
                break;
            
            case 2: //File
			    time ( &tTime );
			    tmTime = localtime ( &tTime );
                COM_pFileLog = fopen ( COM_szOutputFile, "a+t" ); //a+t" );
                if (COM_pFileLog)
                {			    
                    fprintf ( COM_pFileLog, "%d-%02d-%02d %02d:%02d:%02d : %s\n", 
                        tmTime->tm_year+1900,
                        tmTime->tm_mon+1,
                        tmTime->tm_mday,
                        tmTime->tm_hour,
                        tmTime->tm_min,
                        tmTime->tm_sec,
                        szMessage );
                    fclose ( COM_pFileLog );
                }
                break;
                
            case 0:
            default:
                break;
        }
        
        lEnter = 0;
    }
