/*
===================================================================
Copyright (C) 2009-2010 Editions JOCATOP / Christophe Gossa
 
This file is part of Jocareso.

Jocareso is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Jocareso is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jocareso.  If not, see http://www.gnu.org/licenses/


Editions JOCATOP
Route de Saint Saturnin
84310 MORIERES
FRANCE

Christophe Gossa
christophe.gossa@jocatop.fr
===================================================================
*/


#include "jocareso.h"

#ifdef _WIN32    

#define MAX_HOSTNAME_LEN        128
#define MAX_DOMAIN_NAME_LEN     128
#define MAX_SCOPE_ID_LEN        256

typedef struct {
    char String[4 * 4];
} IP_ADDRESS_STRING, *PIP_ADDRESS_STRING, IP_MASK_STRING, *PIP_MASK_STRING;


typedef struct _IP_ADDR_STRING {
    struct _IP_ADDR_STRING* Next;
    IP_ADDRESS_STRING IpAddress;
    IP_MASK_STRING IpMask;
    DWORD Context;
} IP_ADDR_STRING, *PIP_ADDR_STRING;

typedef struct {
    char HostName[MAX_HOSTNAME_LEN + 4];
    char DomainName[MAX_DOMAIN_NAME_LEN + 4];
    PIP_ADDR_STRING CurrentDnsServer;
    IP_ADDR_STRING DnsServerList;
    UINT NodeType;
    char ScopeId[MAX_SCOPE_ID_LEN + 4];
    UINT EnableRouting;
    UINT EnableProxy;
    UINT EnableDns;
} FIXED_INFO, *PFIXED_INFO;


typedef DWORD (WINAPI *GetNetworkParamsProc) ( PFIXED_INFO pFixedInfo, PULONG pOutBufLen );

void GetLocalMachineName ( char * pszComputerName, char * pszHostName )
{

    HMODULE                 hLibrary;
    GetNetworkParamsProc    GetNetworkParams;
    char                    *pszBuffer;
    DWORD                   nSize;
    long                    Retour;


    //Nom par defaut
    strcpy ( pszComputerName, "***Nom introuvable" );
    strcpy ( pszHostName, "***Nom introuvable" );

    nSize = MAX_COMPUTERNAME_LENGTH+1;
    pszBuffer = (char *)Z_Alloc ( nSize );

    Retour = GetComputerNameA ( pszBuffer, &nSize );
    if (Retour) strcpy ( pszComputerName, pszBuffer );
    Z_Free ( pszBuffer );

	//Charge la librairie IPHLPAPI
	hLibrary = LoadLibraryA ( "Iphlpapi.dll" );
	if (!hLibrary) return;

	GetNetworkParams = (GetNetworkParamsProc)GetProcAddress(hLibrary, "GetNetworkParams");
    if (GetNetworkParams)
    {
        FIXED_INFO      *pFixedInfo;
        ULONG           ulOutBufLen;
        DWORD           dwRetVal;
        //IP_ADDR_STRING  *pIPAddr;

        pFixedInfo = (FIXED_INFO *) GlobalAlloc( GPTR, sizeof( FIXED_INFO ) );
        ulOutBufLen = sizeof( FIXED_INFO );
   
        if( ERROR_BUFFER_OVERFLOW == GetNetworkParams( pFixedInfo, &ulOutBufLen ) )
        {
            GlobalFree ( pFixedInfo );
            pFixedInfo = (FIXED_INFO *)GlobalAlloc( GPTR, ulOutBufLen );
        }

        if ( dwRetVal = GetNetworkParams( pFixedInfo, &ulOutBufLen ) )
        {
            //printf( "Call to GetNetworkParams failed. Return Value: %08x\n", dwRetVal );
        }
        else
        {
            strcpy ( pszHostName, pFixedInfo->HostName );
/*---------------------------------------------------------------------
            printf( "Host Name: %s\n", FixedInfo -> HostName );
            printf( "Domain Name: %s\n", FixedInfo -> DomainName );
      
            printf( "DNS Servers:\n" );
            printf( "\t%s\n", FixedInfo -> DnsServerList.IpAddress.String );
      
            pIPAddr = FixedInfo -> DnsServerList.Next;
            while ( pIPAddr ) {
                printf( "\t%s\n", pIPAddr ->IpAddress.String );
                pIPAddr = pIPAddr ->Next;
            }
-----------------------------------------------------------------------*/
        }
        GlobalFree ( pFixedInfo );

    }


    FreeLibrary ( hLibrary );
}
#else
void GetLocalMachineName ( char * pszComputerName, char * pszHostName )
{
    char *          lpBuffer;
    DWORD           nSize;
    long            Retour;

    nSize = MAXHOSTNAMELEN;
    lpBuffer = (char *)Z_Alloc ( nSize );
    Retour = gethostname ( lpBuffer, nSize );
    Retour = (Retour!=-1)?(1):(0);

    if (Retour)
    {
        strcpy ( pszHostName, lpBuffer );
        strcpy ( pszComputerName, lpBuffer );
    }
    else
    {
        strcpy ( pszHostName, "" );
        strcpy ( pszComputerName, "" );
    }
    
    Z_Free ( lpBuffer );
}
#endif


