/*
===================================================================
Copyright (C) 2009-2010 Editions JOCATOP / Christophe Gossa
 
This file is part of Jocareso.

Jocareso is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Jocareso is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jocareso.  If not, see http://www.gnu.org/licenses/


Editions JOCATOP
Route de Saint Saturnin
84310 MORIERES
FRANCE

Christophe Gossa
christophe.gossa@jocatop.fr
===================================================================
*/


#include <stdio.h>
#include <malloc.h>
#include <memory.h>

#ifdef _WIN32
    #include <windows.h>
#else   
    #include <stdio.h>
    #include <string.h>
    #include <pthread.h>
    #include <unistd.h>
#endif


#ifdef _WIN32
    typedef unsigned __int64        QWORD;
    
    #ifndef _BASETSD_H_
    typedef unsigned __int64        UINT64;
    typedef __int64                 INT64;
    #endif
#else
    typedef unsigned long           DWORD;
    typedef unsigned long long      QWORD;
    typedef unsigned long long      UINT64;
    typedef long long               INT64;
#endif


typedef struct {
    void            *pMemoryArea;
    unsigned long   ulMemorySize;
} Z_MEMORYITEM;

unsigned long   Z_Memory_dwInit             = 0;
unsigned long   Z_Memory_dwCriticalSection  = 0;
unsigned long   Z_Memory_dwMemoryInUse      = 0;
Z_MEMORYITEM    *Z_Memory_pItemList         = NULL;
unsigned long   Z_Memory_dwItemCount        = 0;
unsigned long   Z_Memory_dwItemAllocated    = 0;

void Z_MemoryMicroSleep ( unsigned long ulMicroSeconds );
void Z_MemoryEnter ( void );
void Z_MemoryLeave ( void );
void Z_MemoryInit ( void );
void Z_MemoryClose( void );
void * Z_Alloc ( unsigned long ulByteSize );
void * Z_Realloc ( void * p, unsigned long ulByteSize );
void Z_Free ( void * p );
void COM_Printf ( char * fmt, ... );

void Z_MemoryMicroSleep ( unsigned long ulMicroSeconds )
{
#ifdef _WIN32
    QWORD   qwFrequency;

    QWORD   qwTimeWait;
    QWORD   qwTimeCurrent;
    QWORD   qwTimeOffset;
    
    DWORD   dwSleep; 

    QueryPerformanceFrequency ( (LARGE_INTEGER *)&qwFrequency );
    QueryPerformanceCounter ( (LARGE_INTEGER *)&qwTimeOffset );
    qwTimeWait = ( ((QWORD)ulMicroSeconds) * qwFrequency ) / 1000000;

    if (ulMicroSeconds>=2000) dwSleep = ulMicroSeconds/1000;
    else                      dwSleep = 0;

    do {
        //Mesure le temps courant
        QueryPerformanceCounter ( (LARGE_INTEGER *)&qwTimeCurrent );
        qwTimeCurrent -= qwTimeOffset;

        //Passe la main au scheduler
        Sleep ( dwSleep );
        dwSleep = 0;

    } while (qwTimeCurrent<qwTimeWait);
#else
    usleep ( ulMicroSeconds );
#endif
}


void Z_MemoryEnter ( void )
{
#ifdef _WIN32
	__asm {

		mov			ecx, 10

		__EnterTest:

		lock bts	Z_Memory_dwCriticalSection, 0
		jnc			__EnterDone

		push		ecx
		push		ecx
		call		Z_MemoryMicroSleep
		add			esp, 4
		pop			ecx
		add			ecx, ecx

		cmp			ecx, 2000
		jbe			__EnterTest

		mov			ecx, 2000
		jmp			__EnterTest

		__EnterDone:
	}
#else
	asm ( 
		"mov		$10, %ecx\n"

		"__EnterTest:\n"

		"lock bts	$0, Z_Memory_dwCriticalSection\n"
		"jnc			__EnterDone\n"

		"push		%ecx\n"
		"push		%ecx\n"
		"call		Z_MemoryMicroSleep\n"
		"add		$4, %esp\n"
		"pop		%ecx\n"
		"add		%ecx, %ecx\n"

		"cmp		$2000, %ecx\n"
		"jbe		__EnterTest\n"

		"mov		$2000, %ecx\n"
		"jmp		__EnterTest\n"

		"__EnterDone:\n"
		);
#endif
}

void Z_MemoryLeave ( void )
{
#ifdef _WIN32
	__asm {
		lock btr	Z_Memory_dwCriticalSection, 0
	}
#else
	asm ( "lock btr $0, Z_Memory_dwCriticalSection" );
#endif
}


/*
void Z_MemoryEnter_Cbug ( void )
{
    DWORD dwSleepTime;

    dwSleepTime = 10;

    do {
        if (!Z_Memory_dwCriticalSection)
        {
            Z_Memory_dwCriticalSection++;
            if (Z_Memory_dwCriticalSection==1)
                return;

            Z_Memory_dwCriticalSection--;
        }

        Z_MemoryMicroSleep ( dwSleepTime );

        dwSleepTime += dwSleepTime;
        if (dwSleepTime>2000) dwSleepTime = 2000;
    } while (1);
}

void Z_MemoryLeave_Cbug ( void )
{
    Z_Memory_dwCriticalSection--;
}
*/



void Z_MemoryInit ( void )
{
    if (Z_Memory_dwInit) return;

    Z_MemoryEnter ( );

    Z_Memory_dwInit             = 1;
    Z_Memory_pItemList          = (Z_MEMORYITEM *)malloc ( 1024 * sizeof(Z_MEMORYITEM) );
    Z_Memory_dwItemCount        = 0;
    Z_Memory_dwItemAllocated    = 1024;

    memset ( Z_Memory_pItemList, 0, 1024 * sizeof(Z_MEMORYITEM) );

    Z_MemoryLeave ( );
}

void Z_MemoryClose( void )
{
	if(!Z_Memory_dwInit) return;

#if !defined(_JOCADMIN) && !defined(_REMOTEDBWIN32)
	COM_Printf("Memory items still in use : %d items", Z_Memory_dwItemCount);
	COM_Printf("Memory still in use : %d bytes", Z_Memory_dwMemoryInUse);
#endif
}

void * Z_Alloc ( unsigned long ulByteSize )
{
    void *p;

    if (!Z_Memory_dwInit) Z_MemoryInit ( );
    
    //Verification de la taille d'allocation
    if (!ulByteSize) return NULL;
    //Pas de verification, on a le droit d'allouer ZERO
    
    //MMU:Enter
    Z_MemoryEnter ( );

    //Allocation
#ifdef _WIN32
	p = GlobalAlloc(GPTR, ulByteSize & 0x7FFFFFFF);
#else
    p = malloc ( ulByteSize&0x7FFFFFFF );
#endif

    //Pas d'erreur ?
    if (!p)
    {
        Z_MemoryLeave ( );
        return NULL;
    }

    //Mise a zero ?
    if ((p)&&(!(ulByteSize&0x80000000)))
        memset ( p, 0, ulByteSize );

    if (Z_Memory_dwItemCount==Z_Memory_dwItemAllocated)
    {
        Z_Memory_dwItemAllocated += 1024;
        Z_Memory_pItemList = (Z_MEMORYITEM *)realloc ( Z_Memory_pItemList, Z_Memory_dwItemAllocated * sizeof(Z_MEMORYITEM) );
        //COM_Printf ( "ItemAllocated : %d\n", Z_Memory_dwItemAllocated );
    }

    Z_Memory_pItemList[Z_Memory_dwItemCount].pMemoryArea    = p;
    Z_Memory_pItemList[Z_Memory_dwItemCount].ulMemorySize   = ulByteSize&0x7FFFFFFF;
    Z_Memory_dwItemCount    += 1;
    Z_Memory_dwMemoryInUse  += ulByteSize&0x7FFFFFFF;

    //COM_Printf ( "Alloc:%d => %d", ulByteSize, Z_Memory_dwMemoryInUse );

    //MMU:Leave
    Z_MemoryLeave ( );

    return p;
}

void * Z_Realloc ( void * p, unsigned long ulByteSize )
{
    void *new_p;
    DWORD dwIndex, dwItem;

    if (!Z_Memory_dwInit) Z_MemoryInit ( );

    //Le pointeur etait sur ZERO ?
    if (!p) return Z_Alloc ( ulByteSize );

    //On fixe ZERO ?
    if (!(ulByteSize&0x7FFFFFFF))
    {
        Z_Free ( p );
        return NULL;
    }

    //MMU:Enter
    Z_MemoryEnter ( );

    //Recherche de l'item
    dwItem = 0xFFFFFFFF;
    for (dwIndex=0;dwIndex<Z_Memory_dwItemCount;dwIndex++)
    {
        if (Z_Memory_pItemList[dwIndex].pMemoryArea==p)
        {
            dwItem = dwIndex;
            dwIndex = Z_Memory_dwItemCount;
        }
    }

    if (dwItem!=0xFFFFFFFF)
    {
#ifdef _WIN32
		//new_p = GlobalReAlloc(p, ulByteSize & 0x7FFFFFFF, GPTR);
		new_p = GlobalAlloc(GPTR, ulByteSize&0x7FFFFFFF);
		memcpy(new_p, p, min(Z_Memory_pItemList[dwItem].ulMemorySize,ulByteSize & 0x7FFFFFFF));
		GlobalFree(p);

        Z_Memory_dwMemoryInUse  -= Z_Memory_pItemList[dwItem].ulMemorySize;
        Z_Memory_dwMemoryInUse  += ulByteSize&0x7FFFFFFF;
        Z_Memory_pItemList[dwItem].pMemoryArea    = new_p;
        Z_Memory_pItemList[dwItem].ulMemorySize   = ulByteSize&0x7FFFFFFF;
#else
		new_p = realloc ( p, ulByteSize&0x7FFFFFFF );

        Z_Memory_dwMemoryInUse  -= Z_Memory_pItemList[dwItem].ulMemorySize;
        Z_Memory_dwMemoryInUse  += ulByteSize&0x7FFFFFFF;
        Z_Memory_pItemList[dwItem].pMemoryArea    = new_p;
        Z_Memory_pItemList[dwItem].ulMemorySize   = ulByteSize&0x7FFFFFFF;
#endif
    }
    else new_p = NULL;

    //COM_Printf ( "Realloc:%d => %d", ulByteSize, Z_Memory_dwMemoryInUse );

    //MMU:Leave
    Z_MemoryLeave ( );

    return new_p;
}

void Z_Free ( void * p )
{
    DWORD dwIndex, dwItem;

    if (!Z_Memory_dwInit) Z_MemoryInit ( );

    if (!p) return;


    //MMU:Enter
    Z_MemoryEnter ( );

    //Recherche de l'item
    dwItem = 0xFFFFFFFF;
    for (dwIndex=0;dwIndex<Z_Memory_dwItemCount;dwIndex++)
    {
        if (Z_Memory_pItemList[dwIndex].pMemoryArea==p)
        {
            dwItem = dwIndex;
            dwIndex = Z_Memory_dwItemCount;
        }
    }

    if (dwItem!=0xFFFFFFFF)
    {
        Z_Memory_dwMemoryInUse  -= Z_Memory_pItemList[dwItem].ulMemorySize;

        dwIndex = Z_Memory_pItemList[dwItem].ulMemorySize;

        if ( (Z_Memory_dwItemCount>1) &&
             (dwItem!=(Z_Memory_dwItemCount-1)) )
            memcpy ( 
                &Z_Memory_pItemList[dwItem],
                &Z_Memory_pItemList[Z_Memory_dwItemCount-1],
                sizeof(Z_MEMORYITEM) );

        Z_Memory_dwItemCount--;
    }

    //COM_Printf ( "Free:%d => %d", dwIndex, Z_Memory_dwMemoryInUse );
#ifdef _WIN32
	GlobalFree(p);
#else
    free ( p );
#endif

    //MMU:Leave
    Z_MemoryLeave ( );
}
