/*
===================================================================
Copyright (C) 2009-2010 Editions JOCATOP / Christophe Gossa
 
This file is part of Jocareso.

Jocareso is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Jocareso is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jocareso.  If not, see http://www.gnu.org/licenses/


Editions JOCATOP
Route de Saint Saturnin
84310 MORIERES
FRANCE

Christophe Gossa
christophe.gossa@jocatop.fr
===================================================================
*/


//  For Windows only (Visual C++ 2008)
//  Suppress all warnings about unsecured CRT functions
#ifdef _WIN32
#define _CRT_SECURE_NO_WARNINGS
#define jocareso_align_spec(x) _declspec(align(x))
#endif

//#ifndef _JOCARESO_HEADER
#define _JOCARESO_HEADER
//#endif


#ifdef _WIN32
    // C RunTime Header Files
    #include <conio.h>
    #include <direct.h>
    #include <io.h>
    #include <malloc.h>
    #include <math.h>
    #include <memory.h>
    #include <stdio.h>
    #include <stdlib.h>
    #include <string.h>
    #include <tchar.h>
    #include <time.h>
#else
    // C RunTime Header Files
    #include <malloc.h>
    #include <math.h>
    #include <memory.h>
    #include <stdio.h>
    #include <stdlib.h>
    #include <string.h>
    #include <time.h>
    #include <unistd.h>
    #include <termios.h>
    #include <sys/signal.h>
    #include <sys/types.h>
    #include <dirent.h>
#endif

    //API IP Helper
    //#include <iphlpapi.h>
    //#pragma comment ( lib, "Iphlpapi.lib" )

    //********************************************************************************************************
    //Gestion de la memoire    
    void * Z_Alloc ( unsigned long ulByteSize );
    void Z_Free ( void *p );
    void * Z_Realloc ( void * p, unsigned long ulByteSize );

    #define KERNEL_ALLOC(n)         Z_Alloc(n)
    #define KERNEL_REALLOC(p,n)     Z_Realloc(p,n)
    #define KERNEL_FREE(p)          Z_Free(p)


    //External Header Files
    #include "RDB.h"
    #include "sleep.h"
    #include "screensaver.h"
    #include "tick.h"
    #include "thread.h"
    #include "criticalsection.h"
    
    //#include "C:\Projets\EXtensions\OSIndependent\OSIndependent.h"
    #include "strex.h"
    #include "types.h"
    #include "win32-service.h"
    #include "RDBServerClientInterface.h"



    void ConfigLoad ( void );
    void ConfigSave ( void );
    void ConfigSetDefault ( void );


    //********************************************************************************************************
    //Les Threads
    long        ThreadMainProc                  ( void * pProcData );
    long        ThreadWaitForConnectionProc     ( void * pProcData );
    long        ThreadServerProc                ( void * pProcData );

    //********************************************************************************************************
    //ListBaseManagement
    void ListBaseOpen ( void );
    void ListBaseClose ( void );
    void ListBaseFreeClient ( unsigned long ulIdentifiant );


    //Nom du serveur
    void GetLocalMachineName ( char * pszComputerName, char * pszHostName );

    //Test acces
    long VerifyReadWriteModify ( void );
	long copyFile(const char * pszDestination, const char * pszSource, long lForceCopy);



    typedef struct {
        NETWORKDATALINK         NDLClient;
        time_t                  ltimeLastAccessTimeWatchDog;
        long                    bClientUseWathdog;
        SCISTATE                sciState;
    } CLIENTENTRY;

    typedef struct {

        CRITICALSECTION         CriticalSectionCurrentTime;
        time_t                  ltimeCurrentTime;

        //Permet d'obtenir des courbes de donnees entrantes et sortantes (remis a zero une fois par seconde)
        DWORD                   dwTotalDataSend;
        DWORD                   dwTotalDataReceive;
        CRITICALSECTION         CriticalSectionTotalData;


        //CRITICAL_SECTION        CriticalSection;
        CRITICALSECTION         CriticalSection;
        CRITICALSECTION         csConfigAccess;

        THREAD                  ThreadMain;                  //Thread generale
        THREAD                  ThreadWaitForConnection;     //Thread de gestion des connections
        THREAD                  ThreadServerList[16];        //Threads de service RDB - 16 Thread maxi
        long                    ThreadServerListNbEntry;    //Nombre de threads de service RDB (1 a 16)

        long                    ServerLaunched;
        NETWORKDATALINK         NDLServer[16];      //16 serveurs maxi
        long                    lListenPortState[16];

        CLIENTENTRY             *pClientEntry;    //NETWORKDATALINK         *pNDLClient;        
        long                    lNbClients;

        SRDB                    *pRDBBase[512];     //512 bases maxi
        long                    RDBBaseNb;          //toujours 512

        long                    SessionExclusive;
        long                    ClientUpdated;
        char                    szServerComputerName[64];
        char                    szServerHostName[64];
        char                    szServerIP[64];

        //Informations sur les connexions
        //SCISTATE                * pscisClient;
        SCISTATE                pscisClientTable[4096];
        long                    lSCISClientCount;


        SCREENSAVER             ScreenSaver;
        
        long                    lExitApplication;
        char                    szExitApplicationWhy[256];
        
        //char                    szPassword[256];

    } GLOBALDATA;


    typedef struct {
        long                    NetworkPort[16];        //Jusqu'a 16 ports
        long                    lNbThreadServer;        //Nombre de Threads pour gerer le service RDB (min 1 ; max 16)
    } CONFIGDATA;
    
    typedef struct {
        unsigned char           bPasswordSHA512[64];
        char                    szDescription[96];
        long                    lFlag;                  //bit0 = enabled (in use!)
    } PASSWORDID;
    
    typedef struct {
        unsigned char           bPasswordAdminSHA512[64];    //Mot de passe admin
        PASSWORDID              TableUserPassword[64];  //Jusqu' 64 passwords utilisateurs
    } CONFIGDATAEX;
    
    typedef struct {
        char                    szBasePort[16];
        //char                    szPassword[256];
        //char                    szListenMode[256];
        char                    szRootPath[1024];
        long                    iSuperUser; //si (VRAI) alors, on est dj en super user
        
        long                    iForceAdminPasswordReinit;         // -x
        long                    iDisableRemoteAdminPasswordReinit; // -y
        long                    iDisableRemoteAdminPasswordChange; // -w
        long                    iDisableRemoteUpdate;              // -z
    } COMMANDDATA;


    extern GLOBALDATA   GlobalData;
    extern CONFIGDATA   ConfigData;
    extern CONFIGDATAEX ConfigDataEx;
    extern COMMANDDATA  CommandData;


    //HT = HyperThreading / MultiThreading
    //RSE = Rights Security Enforcement (Supervisor/ReadCount)
    //WD = Watchdog
    //#define RDBSERVER_TITRE         "RDBServer"
    //#define RDBSERVER_VERSION       "v1.24 \n"__DATE__
    //#define RDBSERVER_VERSION       "v1.23b HT+RSE+WD \n"__DATE__
    #define DAEMON_NAME         "jocareso"
#ifdef _WIN32    
    #define DAEMON_DESC         "Win32 service for Editions JOCATOP network applications"
#else
    #define DAEMON_DESC         "Linux daemon for Editions JOCATOP network applications"
#endif

#ifdef _WIN32    
    #define DAEMON_TYPE         "win32"
    #define SYSTEM_PATHROOT     "C:/"
#else
    #define DAEMON_TYPE         "linux"
    #define DAEMON_EXENAME      "jocareso-linux.386"
    #define SYSTEM_PATHROOT     "/usr/sbin/"
    #define LINUX_INITD         "/etc/init.d/"DAEMON_NAME
    #define LINUX_PATHEXE       SYSTEM_PATHROOT""DAEMON_EXENAME
    
    #define PERSMISSION_LINUXFILE   (S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH)

	void SysGetLogPath(char *);
#endif

    /*
        History line
        
        v0.01
        Initial linux-daemon and win32-service release, without any install !
        
        v0.02.1
        Renamed as 'Jocatys'
        win32-service full cli (jocatys -i)
        linux-daemon full cli (sudo jocatys -i)

		v0.02.2
		Renamed as 'JOCARESO'
		Removed date limit (for free testing)

        v0.02.3
        Fix a memory alloc bug (forgot multiple KERNEL_FREE in RDBServerMessage.cpp)

        v0.02.4
        New param "-k/-K" used to set running in ROOT mode (Linux DEBIAN only)

        v0.02.5
        Bugfix : Memory leak found in Z_Free (free has accidentely suppressed from jocareso-memory.c)
        
        v0.02.6
        Enhancement : Best-effort install for Kwartz server !
        
        v0.03.0
        New secure protocol with user and admin password (incompatible with older jocareso)
        Updated protocol...
        
        v0.03.1
        New protocol release (0x00000301), added RDB_ACCESS_CLIENT_INTERFACE_SAM_RDBLoadFilePacket and
        RDB_ACCESS_CLIENT_INTERFACE_SAM_RDBSaveFilePacket functions : Load and Save RDB clusterized in
        small packets.
        

    
    
    */
    
    #define JOCARESO_PROTOCOL_RELEASE               0x00000301
    #define JOCARESO_PROTOCOL_RELEASE_0x00000300    0x00000300
    #define JOCARESO_VERSION_GEN                    0
    #define JOCARESO_VERSION_MAJOR                  3
    #define JOCARESO_VERSION_MINOR                  1
    
    #define DAEMON_VERSION_TEXTINFO     ""DAEMON_TYPE" build "__DATE__" "__TIME__
    
    //#define DAEMON_VERSION      "v0.3.0-"DAEMON_TYPE" build "__DATE__                         
    
    #define DAEMON_PATHNAME     "jocareso"
    #define DAEMON_COPYRIGHT    "Original programming by Christophe Gossa\nSupport by Gavroche Richier\n(c) 2009-2010 Editions JOCATOP"

    #define DAEMON_PASSWORD     "admin"
    
    long JOCARESO_Application ( void );
    
    long SYS_SingleInstanceOpen ( void );
    long SYS_SingleInstanceClose ( void );
    long SYS_SingleInstanceGetPID ( long * lPID );
    char * SYS_GetCurrentPath ( void );
    char * SYS_GetHomePath ( void );
    char * SYS_GetInstallPath ( void );
    void SYS_SetInstallPath ( char * pszPath );
    void SYS_RunSimpleModeSet ( long lMode );
    long SYS_RunSimpleModeGet ( void );
    
    char * SYS_GetExecRunApp ( void );
    void SYS_SetExecRunApp ( char * psz );

    
    void COM_Open ( long lOutputMode, char * pszPath );
    void COM_Close ( void );
    void COM_Printf ( char * fmt, ... );
    void COM_GetMode ( long * plOutputMode );
    long ParseConsoleToArg ( char * szLine, int * piArgCount, char *** pszArgValue );
    
#ifdef _WIN32
    //VOID WINAPI win32_ServiceMain ( DWORD dwArgc, LPTSTR *lpszArgv );
    
    char * JOCARESO_GetTextError ( DWORD dwError );

    /*
    long JOCARESO_ServiceManagerConnect ( void );
    long JOCARESO_ServiceManagerDysconnect ( void );
    long JOCARESO_ServiceAppOpen ( void );    
    long JOCARESO_ServiceAppClose ( void );
    long JOCARESO_ServiceInstall ( char * pszBinaryPath );
    long JOCARESO_ServiceUninstall ( void );
    long JOCARESO_ServiceSetState ( long lStateNeed );
    
    long JOCARESO_ServiceGetStopEvent ( void );
    long JOCARESO_ServiceReleaseStopEvent ( void );
    */

#else
    long _LinuxDaemon_GetStopEvent ( void );
    long _LinuxDaemon_DoApplicationStop ( void );
/*    long _LinuxDaemon_DoApplicationInstall ( char * szParams );
    long _LinuxDaemon_DoApplicationUninstall ( void );*/
#endif


    // MAIN functions
    long main_getpaths ( void );
    void main_getparams ( int iArgCount, char * pszArg[], long * plRunMode );
    void main_todo ( long lMode );
    void main_todoStandAlone ( void );
    
    //AUTO UPDATE !!!
    long jocareso_autoupdate ( 
        BYTE * pbFileData, DWORD dwFileSize, char * pszJocaresoUpdateFile, 
        char * pszScriptData, char * pszJocaresoScriptFile );


    #define JOCARESO_ERROR_UPDATE_INCORRECTFILESIZE     (-100)
    #define JOCARESO_ERROR_UPDATE_INCORRECTFILEHASH     (-101)
    #define JOCARESO_ERROR_UPDATE_INCORRECTFILEOS       (-102)
    #define JOCARESO_ERROR_UPDATE_WRITEAPPLICATION      (-103)
    #define JOCARESO_ERROR_UPDATE_WRITECOMMAND          (-104)
    #define JOCARESO_ERROR_UPDATE_STARTCOMMAND          (-105)

    #define JOCARESO_UPDATE_FILEOS_LETTER_WIN32     ('W')
    #define JOCARESO_UPDATE_FILEOS_LETTER_LINUX     ('L')


    #ifdef _WIN32
    #define JOCARESO_UPDATE_FILEOS_LETTER           JOCARESO_UPDATE_FILEOS_LETTER_WIN32
    #else
    #define JOCARESO_UPDATE_FILEOS_LETTER           JOCARESO_UPDATE_FILEOS_LETTER_LINUX
    #endif

	// Memory functions
	void Z_MemoryClose( void );
