/*
===================================================================
Copyright (C) 2009-2010 Editions JOCATOP / Christophe Gossa
 
This file is part of Jocareso.

Jocareso is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Jocareso is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jocareso.  If not, see http://www.gnu.org/licenses/


Editions JOCATOP
Route de Saint Saturnin
84310 MORIERES
FRANCE

Christophe Gossa
christophe.gossa@jocatop.fr
===================================================================
*/


//  For Windows only (Visual C++ 2008)
//  Suppress all warnings about unsecured CRT functions
#ifdef _WIN32
#define _CRT_SECURE_NO_WARNINGS
#endif


#ifdef _WIN32
    #include <tchar.h>
    #include <conio.h>
    #include <direct.h>
    #include <io.h>

    #define jocareso_chdir(x)    _chdir(x)
    #define jocareso_mkdir(x,r)  _mkdir(x)
#else
	#include <errno.h>
	#include <sys/stat.h>
	#ifdef HAVE_CONFIG_H
	#include <config.h>
	#endif

	//#include <iostream>
	//#include <cstdlib>
    #include <limits.h>
    #include <unistd.h>

    #define jocareso_chdir(x)    chdir(x)
    #define jocareso_mkdir(x,r)  mkdir(x,r)
#endif




#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <time.h>
#include <math.h>

// C RunTime Header Files
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>


#include "jocareso.h"

/*
    Les parametres :
    -sPASSWORD
    -lLISTEN_MODE
    -pPORT

*/


char const *szHelp_EN[] = {    
"Options :",
//"  -s[Security Password]",
//"  -l[Interface listen Mode]",
"  -p[Port]",
"  -r[Root path]",
"  -x = Force admin password reinit",
"  -y = Disable remote admin password reinit",
"  -w = Disable remote admin password update",
"  -z = Disable remote update",
"  -i = Install service/daemon",
"  -u = Uninstall service/daemon",
"  -? or -h or -H = English help",
"  -A ou -a = Aide en francais",
"  -k ou -K = Always started in ROOT mode",
"  start = Start the service/daemon",
"  stop = Stop the service/daemon",
"  run = Run as application",
"",
//"* Security password",
//"  Type your own password to use interface with this deamon",
//"",
/*"* Interface listen mode (default=localhost)",
"  lan = accept interface over all the network",
"  localhost = accept interface over this computer only",
"  127.0.0.1 = same as localhost",
"  x.x.x.x = accept interface from this ip only (and localhost)",
"",*/
"* Port (default=2000)",
"  You could use other value instead of 2000, but not recommanded.",
"",
"* Root path",
"  You could change base root path.",
"  default Windows = C:/",
"  default Linux = /",
"",
"@" };

char const *szHelp_FR[] = {    
"Options :",
//"  -s[Mot de passe de securite]",
//"  -l[Mode d'ecoute pour l'interface]",
"  -p[Port]",
"  -r[Chemin de base]",
"  -x = Forcer la reinitialisation du mot de passe admin",
"  -y = Desactiver la reinitialisation a distance du mot de passe admin",
"  -w = Desactiver la modification a distance du mot de passe admin",
"  -z = Desactiver la mise a jour a distance",
"  -i = Installer le service/demon",
"  -u = Desinstaller le service/demon",
"  -? or -h or -H = English help",
"  -A ou -a = Aide en francais",
"  -k ou -K = Indique un lancement en mode ROOT",
"  start = Lancer le service/demon",
"  stop = Arreter le service/demon",
"  run = Demarrer comme une application",
"",
//"* Mot de passe de securite",
//"  Inscrivez votre propre mot de passe pour l'acces au demon via l'interface",
//"",
/*"* Mode d'ecoute pour l'interface (defaut=localhost)",
"  lan = accepte l'interface a partir de tout le reseau",
"  localhost = accepte l'interface a partir de cet ordinateur uniquement",
"  127.0.0.1 = comme localhost",
"  x.x.x.x = accepte l'interface a partir de l'ip specifiee (et de localhost)",
"",*/
"* Port (defaut=2000)",
"  Il est possible de choisir une valeur differente de 2000, mais ce n'est pas recommande.",
"",
"* Chemin de base",
"  Vous pouvez modifier le chemin de base.",
"  Par defaut, sous Windows = C:/",
"  Par defaut, sous Linux = /usr/bin/",
"",
"@" };




#define _RUNMODE_NOTHING        0
#define _RUNMODE_INSTALL        1
#define _RUNMODE_UNINSTALL      2
#define _RUNMODE_START          3
#define _RUNMODE_STOP           4
#define _RUNMODE_HELP_EN        5
#define _RUNMODE_HELP_FR        6
#define _RUNMODE_APPLICATION    7

long main_getpaths ( void )
{
    char szHomePath[1024];
    char szBasePath[1024];
    char szTmp[1024];
    char szRootPath[1024];
    
    if (strlen(CommandData.szRootPath))
        strcpy ( szRootPath, CommandData.szRootPath );
    else
        strcpy ( szRootPath, SYSTEM_PATHROOT );
        
    strcat ( szRootPath, (char *)"/" );
    strcreplace ( szRootPath, '\\', '/' );
    strreplace ( szRootPath, (char *)"//", (char *)"/" );
    strsupprallright ( szRootPath, '/' );
    strcat ( szRootPath, (char *)"/" );
    
    
    strcpy ( szHomePath, SYS_GetHomePath ( ) );
    

    //Get default user path
    strcpy ( szBasePath, SYS_GetCurrentPath ( ) );
    //COM_Printf ( "BasePath = %s\nHomePath = %s\nRootPath = %s", szBasePath, szBasePath, szRootPath );
    if (jocareso_chdir(szRootPath)<0)
    {
        COM_Printf ( (char *)"Erreur : Impossible de se deplacer sur la racine (%s).", szRootPath );
        return -1;
    }

	SysGetLogPath(szTmp);
    if (jocareso_chdir(szTmp)<0)  //DAEMON_PATHNAME
    { 
        //COM_Printf ( (char *)"Erreur : Impossible de se deplacer sur (%s).", szTmp );
        if (jocareso_mkdir ( szTmp, 0777 )<0)
        {
            COM_Printf ( (char *)"Erreur : Impossible de creer le dossier %s", szTmp );
        }
        if (jocareso_chdir(szTmp)<0)
        {
            COM_Printf ( (char *)"Erreur : Impossible de se deplacer sur le dossier %s.", szTmp );
            return -1;
        }
    }

	SysGetLogPath(szTmp);
    SYS_SetInstallPath ( szTmp );
    COM_Printf ( (char *)"# Dossier install : %s\n", SYS_GetInstallPath ( ) );
    
    //Go to initial path
    jocareso_chdir ( szBasePath );
    
    return 0;
}

void main_getparams ( int iArgCount, char * pszArg[], long * plRunMode )
{
    int iArgIndex;
    long lParamTypePosition;
    long lRunMode;
    char szTmp[1024];
    
    SYS_SetExecRunApp ( pszArg[0] );
    
    //Get current params
    memset ( &CommandData, 0, sizeof(COMMANDDATA) );
    strcpy ( CommandData.szBasePort,    "2000" );
    //strcpy ( CommandData.szListenMode,  "0.0.0.0" );
    //strcpy ( CommandData.szPassword,    "" );
    strcpy ( CommandData.szRootPath,    SYSTEM_PATHROOT );
    CommandData.iSuperUser = 0;
    
    lRunMode      = _RUNMODE_NOTHING;
    for (iArgIndex=1;iArgIndex<iArgCount;iArgIndex++)
    {
        strcpy ( szTmp, pszArg[iArgIndex] );

        switch (szTmp[0])
        {
            case '-':
            case '\\':
            case '/':
                lParamTypePosition = 1;
                break;
                
            default:
                lParamTypePosition = 0;
                break;
        }
        
        if (lParamTypePosition)
        {
            switch (szTmp[lParamTypePosition])
            {
                //Force admin password reinit
                case 'x':
                case 'X':
                    CommandData.iForceAdminPasswordReinit = 1;
                    break;
                    
                //Disable remote admin password reinit
                case 'y':
                case 'Y':
                    CommandData.iDisableRemoteAdminPasswordReinit = 1;
                    break;

                //Disable remote admin password reinit
                case 'w':
                case 'W':
                    CommandData.iDisableRemoteAdminPasswordChange = 1;
                    break;
                    
                //Disable remote update
                case 'z':
                case 'Z':
                    CommandData.iDisableRemoteUpdate = 1;
                    break;
            
                //Security Access
                case 's':
                case 'S': 
                    //strcpy ( CommandData.szPassword, szTmp+lParamTypePosition+1 );
                    break;
                    
                //Listen Mode
                case 'l':
                case 'L':
                    //strcpy ( CommandData.szListenMode, szTmp+lParamTypePosition+1 );
                    break;
                
                //Port
                case 'p':
                case 'P':
                    strcpy ( CommandData.szBasePort, szTmp+lParamTypePosition+1 );
                    break;
                
                //Base root path    
                case 'r':
                case 'R':
					if(iArgIndex + 1 >= iArgCount)
					{
						COM_Printf("-r doit tre suivi d'un chemin");
						exit(-1);
					}
                    // strcpy ( CommandData.szRootPath, szTmp+lParamTypePosition+1 );
					strcpy ( CommandData.szRootPath, pszArg[iArgIndex + 1]);
                    break;

                case 'k':
                case 'K':
                    CommandData.iSuperUser = 1;
                    break;
                    
                case '?':
                case 'h':
                case 'H':
                    lRunMode = _RUNMODE_HELP_EN;
                    break;
                    
                case 'a':
                case 'A':
                    lRunMode = _RUNMODE_HELP_FR;
                    break;
                    
                case 'i':
                case 'I':
                    lRunMode = _RUNMODE_INSTALL;
                    break;
                    
                case 'u':
                case 'U':
                    lRunMode = _RUNMODE_UNINSTALL;
                    break;
                
            }
        }
        else
        {
            if (stricmp(szTmp,"start")==0)
                lRunMode = _RUNMODE_START;
            
            if (stricmp(szTmp,"stop")==0)
                lRunMode = _RUNMODE_STOP;

            if (stricmp(szTmp,"install")==0)
                lRunMode = _RUNMODE_INSTALL;
            
            if (stricmp(szTmp,"uninstall")==0)
                lRunMode = _RUNMODE_UNINSTALL;

            if (stricmp(szTmp,"run")==0)
                lRunMode = _RUNMODE_APPLICATION;


        }
    }
    
    if (plRunMode)
        *plRunMode = lRunMode;
} 


int main(int argc, char* argv[])
{
    char szTmp[1024], szTmp2[1024], szParams[1024];
    long lRunMode;
    long lReturn;

#ifdef _WIN32    
    _Win32Service_SetNameDescProc ( 
        DAEMON_NAME,
        DAEMON_DESC,
        main_todoStandAlone );
#endif

    COM_Open ( 1, NULL );

    // Get params
    main_getparams ( argc, argv, &lRunMode );
    
#ifndef _WIN32
	char szUsrBinDir[18] = "/usr/bin/jocareso";
	char szVarLibDir[18] = "/var/lib/jocareso";

    DIR			    *pUsrBinDir;
	DIR				*pVarLibDir;
    struct dirent   *pDirEntry;

	if(pUsrBinDir = opendir(szUsrBinDir))
	{
		if(!(pVarLibDir = opendir(szVarLibDir)))
		{
			if(errno == ENOENT)
			{
				long iMkReturn;

				if(iMkReturn = mkdir(szVarLibDir, S_IREAD | S_IWRITE))
				{
					COM_Printf("Impossible de creer le dossier %s", szVarLibDir);
					exit(iMkReturn);
				}
				if(!(pVarLibDir = opendir(szVarLibDir)))
				{
					COM_Printf("Impossible d'ouvrir le dossier %s", szVarLibDir);
					exit(errno);
				}
			}
			else
			{
				COM_Printf("Impossible d'ouvrir le dossier %s", szVarLibDir);
				exit(errno);
			}
		}

		while((pDirEntry = readdir(pUsrBinDir)) != NULL)
		{
			char * szRDBFileExtension;
			char szFileOldPath[128];
			char szFileNewPath[128];

			szRDBFileExtension = strstr(pDirEntry->d_name, ".rdb");

			if(szRDBFileExtension)
			{
				long iLnReturn;

				sprintf(szFileOldPath, "%s/%s", szUsrBinDir, pDirEntry->d_name);
				sprintf(szFileNewPath, "%s/%s", szVarLibDir, pDirEntry->d_name);

				iLnReturn = copyFile(szFileNewPath, szFileOldPath, 0);

				if(iLnReturn < 0)
				{
					COM_Printf("Impossible de deplacer le fichier %s a %s :", szFileOldPath, szFileNewPath);
					perror(NULL);
				}
			}
		}
	}
#endif
//Redo_console:
    // Show version number
    COM_Printf ( (char *)"%s (v%d.%d.%d-%s)", DAEMON_NAME, JOCARESO_VERSION_GEN, JOCARESO_VERSION_MAJOR, JOCARESO_VERSION_MINOR, DAEMON_VERSION_TEXTINFO );
    COM_Printf ( (char *)"%s\n", DAEMON_COPYRIGHT );
    
    /*
    {
        long i;
        for (i=0;i<argc;i++)
        {    
            COM_Printf ( "arg %d => %s", i, argv[i] );
        }
    }
    */
    
    
    //Set new mask
    //umask(0);
    
    
    //####################################################################################################
    //Install daemon / service
    
    if (lRunMode==_RUNMODE_INSTALL)
        main_getpaths ( );

    switch (lRunMode)
    {
        case _RUNMODE_NOTHING:
            //exit ( 0 );
            break;
            
        case _RUNMODE_HELP_FR:
        case _RUNMODE_HELP_EN:
            {
                long i;
                char **szHelp;
                
                if (lRunMode==_RUNMODE_HELP_FR) szHelp = (char **)szHelp_FR;
                else                            szHelp = (char **)szHelp_EN;
                
                for (i=0;szHelp[i][0]!='@';i++)
                {
                    COM_Printf ( (char *)"%s", szHelp[i] );
                }
                return 0;
            }
            break;
            
        case _RUNMODE_INSTALL:
            szParams[0] = 0;
            
            /*if (strlen(CommandData.szPassword))
            {
                strcat ( szParams, " -s" );
                strcat ( szParams, CommandData.szPassword );
            }
            
            if (strlen(CommandData.szListenMode))
            {
                strcat ( szParams, " -l" );
                strcat ( szParams, CommandData.szListenMode );
            }*/
            
            if (strlen(CommandData.szBasePort))
            {
                strcat ( szParams, " -p" );
                strcat ( szParams, CommandData.szBasePort );
            }

            if (strlen(CommandData.szRootPath))
            {
                strcat ( szParams, " -r" );
                strcat ( szParams, CommandData.szRootPath );
            }

            if (CommandData.iDisableRemoteAdminPasswordChange)
                strcat ( szParams, " -w" );            

            if (CommandData.iDisableRemoteAdminPasswordReinit)
                strcat ( szParams, " -y" );

            if (CommandData.iDisableRemoteUpdate)
                strcat ( szParams, " -z" );

#ifdef _WIN32
            if (!GetModuleFileNameA ( NULL, szTmp, 1023 ))
            {
                COM_Printf ( (char *)"Error : Can't get service name." );
                exit ( 0 );
            }
            
            sprintf ( szTmp2, "\"%s\" %s", szTmp, szParams );
            if (_Win32Service_DoApplicationInstall ( szTmp2 ))
                COM_Printf ( (char *)"Error : Can't install." );
#else
/*            if (_LinuxDaemon_DoApplicationInstall ( szParams ))
                COM_Printf ( (char *)"Error : Can't install." );*/
#endif
            exit(0);
            break;
            
        case _RUNMODE_UNINSTALL:
#ifdef _WIN32        
            if (_Win32Service_DoApplicationUninstall())
                COM_Printf ( (char *)"Error : Can't uninstall." );
#else
            if (_LinuxDaemon_DoApplicationStop())
                COM_Printf ( (char *)"Error : Can't stop." );
            
/*            if (_LinuxDaemon_DoApplicationUninstall())
                COM_Printf ( (char *)"Error : Can't uninstall." );*/
#endif
            exit ( 0 );
            break;
            
        case _RUNMODE_START:
#ifdef _WIN32        
            if (_Win32Service_DoApplicationStart())
                COM_Printf ( (char *)"Error : Can't start." );
                
            exit ( 0 );
#else
            {
                long lPID;
                if (!SYS_SingleInstanceGetPID(&lPID))
                {
                    COM_Printf ( (char *)"Daemon already running." );
                    exit ( 0 );
                }
                else COM_Printf ( (char *)"Daemon started." );
            }
            //_LinuxDaemon_DoApplicationStop();
                        
#endif
            //long JOCARESO_ServiceSetState ( long lStateNeed );
            
            break;
            
        case _RUNMODE_STOP:
#ifdef _WIN32        
            if (_Win32Service_DoApplicationStop())
                COM_Printf ( (char *)"Error : Can't stop." );
#else
            COM_Printf ( "Try to stop..." );
            if (_LinuxDaemon_DoApplicationStop())
                COM_Printf ( (char *)"Error : Can't stop.\nTry 'sudo ./"DAEMON_NAME" stop'" );
            else
                COM_Printf ( "Stopped." );            
#endif
            exit (0);
            break;

        case _RUNMODE_APPLICATION:
            SYS_RunSimpleModeSet ( 1 );
            main_todo ( 1 );
            exit (0);
            break;
    }
    
    //printf ( "Current path : %s\n", SYS_GetCurrentPath ( ) );
    
    //printf ( "Home path : %s\n", SYS_GetHomePath ( ) );
        
    //return 0;
    //################################
    //Forking the parent process
    //pid_t pid, sid;
    
    //Fork off the parent process
    /*
    pid = fork();
    if (pid < 0)
    {
        printf ( "pid < 0\n" );
        exit(EXIT_FAILURE);
    }
    // If we got a good PID, then we can exit the parent process.
    if (pid > 0) 
    {
        printf ( "pid > 0\n" );    
        exit(EXIT_SUCCESS);
    }
    */
    /*
    //Change the file mode mask
    umask(0);    
    
    //Create a new SID for the child process
    sid = setsid ( );
    if (sid < 0) 
    {
        printf ( "sid failure\n" );
        exit(EXIT_FAILURE);     
    }
    */
    
    

    lReturn = 0;   
#ifdef _WIN32
    {
        SERVICE_TABLE_ENTRYA DispatchTable[] = 
        { 
            { "", (LPSERVICE_MAIN_FUNCTIONA)_Win32Service_ServiceMain }, 
            { NULL, NULL } 
        };    
    
        // This call returns when the service has stopped. 
        // The process should simply terminate when the call returns.
        if (!StartServiceCtrlDispatcherA(DispatchTable)) 
        { 
            COM_Printf ( "%s - Can't start service control dispatcher.", JOCARESO_GetTextError ( GetLastError() ) );
            lReturn = -1;
        }
    }
#else
    {
        int iDescriptor;
        int iDescriptorMax;
        
        if (fork()==0)
        {
            setsid();
            
            iDescriptorMax = sysconf(_SC_OPEN_MAX); //OPEN_MAX

            /*for (iDescriptor=0;iDescriptor<iDescriptorMax;iDescriptor++)
                close(iDescriptor);*/
            
            jocareso_chdir ( (char *)"/" );
        
            main_todoStandAlone ( );
            return 0;
        }
        else lReturn = -1;
    }
#endif
    {
        long lOutputMode;
        COM_GetMode ( &lOutputMode );
    
    /*
    if ((lReturn)&&(lOutputMode==1))
    {
        char szConsole[1024];
        int iArgCount;
        char ** pszArg;
        
        printf ( "Console : " );
        gets ( szConsole );
        ParseConsoleToArg ( szConsole, &iArgCount, &pszArg );
        
        main_getparams ( iArgCount, pszArg, &lRunMode );
        
        Z_Free ( pszArg );
        
        goto Redo_console;
        
        
    }
    */
    


    /*
    COM_Open ( 2, SYS_GetInstallPath ( ) );
    JOCARESO_Application ( );
    COM_Close ( );
    */
    
        COM_Close ( );
    }
    return 0;
}

void main_todo ( long lMode )
{
    if (!main_getpaths ( ))
    {
        COM_Open ( lMode, SYS_GetInstallPath ( ) );
        JOCARESO_Application ( );
        COM_Close ( );
    }
    else COM_Printf ( (char *)"Error - Can't set default path." );
}

void main_todoStandAlone ( void )
{
    main_todo ( 2 );
}


long ParseConsoleToArg ( char * szLine, int * piArgCount, char *** pszArgValue )
{
    long i;
    long lValueIndexStart[32];
    long lValueIndexLength[32];
    long lValueIndexCount;
    long lValueIndexMode;
    char szTmp[1024];
    char ** pszArg;
    long lAllocSize, lOffset;

    //First fictive arg.
    lValueIndexCount = 1;
    lValueIndexStart[0] = 0;
    lValueIndexLength[0] = 0;
    
    lValueIndexMode = 0;
    for (i=0;szLine[i];i++)
    {
        if ((lValueIndexMode==0)&&(szLine[i]!=32)) //Found something
        {
            lValueIndexMode = 1;
            lValueIndexStart[lValueIndexCount] = i;
            lValueIndexLength[lValueIndexCount] = 0;
        }
        
        if (szLine[i]==34) //Found quotation mark
            lValueIndexMode = 3 - lValueIndexMode;
            //1 = no quotation
            //2 = quotation
 
        if ((lValueIndexMode==1)&&(szLine[i]==32))
        {
            lValueIndexMode = 0;
            lValueIndexCount++;
        }
 
        if (lValueIndexMode)
            lValueIndexLength[lValueIndexCount] += 1;
            

    }
    if (lValueIndexMode)
        lValueIndexCount++;
        
    //Get allocation size
    lAllocSize = 0;
    for (i=0;i<lValueIndexCount;i++)
    {
        lAllocSize += lValueIndexLength[i]+1+sizeof(char*);
    }
    pszArg = (char **)Z_Alloc ( lAllocSize );
    
    //Process allocation
    lOffset = sizeof(char *) * lValueIndexCount;
    for (i=0;i<lValueIndexCount;i++)
    {
        memcpy ( szTmp, szLine+lValueIndexStart[i], lValueIndexLength[i] );
        szTmp[lValueIndexLength[i]] = 0;
        pszArg[i] = (char *)pszArg + lOffset;
        strcpy ( pszArg[i], szTmp );
        lOffset += lValueIndexLength[i]+1;
    }
    
    *piArgCount = lValueIndexCount;
    *pszArgValue = pszArg;
    return 0;
}

