/*
===================================================================
Copyright (C) 2009-2010 Editions JOCATOP / Christophe Gossa
 
This file is part of Jocareso.

Jocareso is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Jocareso is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jocareso.  If not, see http://www.gnu.org/licenses/


Editions JOCATOP
Route de Saint Saturnin
84310 MORIERES
FRANCE

Christophe Gossa
christophe.gossa@jocatop.fr
===================================================================
*/

////////////////////////////////////////////////////////////////////////
//
//  Change log.
//
//  Revision 1.00 2006/05/15  8:22  Christophe Gossa
//  Initial release for Win32 only.
//
//  Revision 1.01 2006/08/29 12:19  Christophe Gossa
//  Removed SetThreadExecutionState to virtual function for Win 95/NT.
//
////////////////////////////////////////////////////////////////////////

#include "screensaver.h"
                                    
#ifdef _WIN32
#include <windows.h>
#include <windowsx.h>
#include <winuser.h>
#include <tchar.h>
#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <direct.h>
#include <stdlib.h>
#include <commctrl.h>
#include <fcntl.h>
#include <io.h>
#include <time.h>
#include <math.h>
#include <commdlg.h>
#include <malloc.h>
#include <winbase.h>


long    ScreenSaverManagerOpen ( SCREENSAVER * pScreenSaver )
{
    BOOL            bScreenSaveActive;
    BOOL            bPowerOffActive;
    DWORD           dwPowerOffTimeOut;
    BOOL            bLowPowerActive;

    if (!pScreenSaver) return -1;

    //Test de l'etat de l'ecran de veille
    SystemParametersInfo ( SPI_GETSCREENSAVEACTIVE, 0, &bScreenSaveActive, SPIF_UPDATEINIFILE );
    pScreenSaver->bScreenSaveActive = (long)bScreenSaveActive;

    //Test de l'etat de la mise en veille prolongee
    SystemParametersInfo ( SPI_GETPOWEROFFACTIVE, 0, &bPowerOffActive, SPIF_UPDATEINIFILE );
    pScreenSaver->bPowerOffActive = (long)bPowerOffActive;
    
    SystemParametersInfo ( SPI_GETPOWEROFFTIMEOUT, 0, &dwPowerOffTimeOut, SPIF_UPDATEINIFILE );
    pScreenSaver->dwPowerOffTimeOut = (unsigned long)dwPowerOffTimeOut;
    
    SystemParametersInfo ( SPI_GETLOWPOWERACTIVE, 0, &bLowPowerActive, SPIF_UPDATEINIFILE );
    pScreenSaver->bLowPowerActive = (long)bLowPowerActive;

    return 0;
}

long    ScreenSaver_SetThreadExecutionState_Win32 ( long lMode )
{
    typedef EXECUTION_STATE (WINAPI * _SetThreadExecutionState_DLLProc) ( EXECUTION_STATE esFlags );
    _SetThreadExecutionState_DLLProc    pSetThreadExecutionState;
    HMODULE                             hLibrary;
    EXECUTION_STATE                     esReturn;

    esReturn = 0;

    hLibrary = LoadLibraryA ( "Kernel32.DLL" );
    if (hLibrary)
    {            
        pSetThreadExecutionState = (_SetThreadExecutionState_DLLProc) GetProcAddress ( hLibrary, "SetThreadExecutionState" );
        if (pSetThreadExecutionState)
        {
            esReturn = pSetThreadExecutionState ( lMode );   
        }
        FreeLibrary ( hLibrary );
    }
    
    return (long)esReturn;
}

long    ScreenSaverManagerSet ( SCREENSAVER * pScreenSaver, long bEnableScreenSaver, long bEnablePowerSaver )
{
    if (!pScreenSaver) return -1;

    //ScreenSaver
    if (bEnableScreenSaver)
    {
        SystemParametersInfo ( SPI_SETSCREENSAVEACTIVE, (BOOL)pScreenSaver->bScreenSaveActive, NULL, SPIF_UPDATEINIFILE );
        SystemParametersInfo ( SPI_SETSCREENSAVEACTIVE, (BOOL)pScreenSaver->bScreenSaveActive, NULL, SPIF_SENDWININICHANGE );
    }
    else
    {
        SystemParametersInfo ( SPI_SETSCREENSAVEACTIVE, 0, NULL, SPIF_UPDATEINIFILE );
        SystemParametersInfo ( SPI_SETSCREENSAVEACTIVE, 0, NULL, SPIF_SENDWININICHANGE );
    }

    //PowerSaver
    if (bEnablePowerSaver)
    {
        SystemParametersInfo ( SPI_SETPOWEROFFTIMEOUT, (DWORD)pScreenSaver->dwPowerOffTimeOut, NULL, SPIF_UPDATEINIFILE );
        SystemParametersInfo ( SPI_SETPOWEROFFTIMEOUT, (DWORD)pScreenSaver->dwPowerOffTimeOut, NULL, SPIF_SENDWININICHANGE );

        SystemParametersInfo ( SPI_SETPOWEROFFACTIVE, 1, NULL, SPIF_UPDATEINIFILE );
        SystemParametersInfo ( SPI_SETPOWEROFFACTIVE, 1, NULL, SPIF_SENDWININICHANGE );

        SystemParametersInfo ( SPI_SETLOWPOWERACTIVE, (BOOL)pScreenSaver->bLowPowerActive, NULL, SPIF_UPDATEINIFILE );
        SystemParametersInfo ( SPI_SETLOWPOWERACTIVE, (BOOL)pScreenSaver->bLowPowerActive, NULL, SPIF_SENDWININICHANGE );

        //Le systeme repart dans son mode normal
        ScreenSaver_SetThreadExecutionState_Win32 ( 0 );
    }
    else
    {
        SystemParametersInfo ( SPI_SETPOWEROFFACTIVE, 0, NULL, SPIF_UPDATEINIFILE );
        SystemParametersInfo ( SPI_SETPOWEROFFACTIVE, 0, NULL, SPIF_SENDWININICHANGE );

        SystemParametersInfo ( SPI_SETPOWEROFFTIMEOUT, 0, NULL, SPIF_UPDATEINIFILE );
        SystemParametersInfo ( SPI_SETPOWEROFFTIMEOUT, 0, NULL, SPIF_SENDWININICHANGE );

        SystemParametersInfo ( SPI_SETLOWPOWERACTIVE, 0, NULL, SPIF_UPDATEINIFILE );
        SystemParametersInfo ( SPI_SETLOWPOWERACTIVE, 0, NULL, SPIF_SENDWININICHANGE );

        //Force le systeme a rester en marche !!!
        ScreenSaver_SetThreadExecutionState_Win32 ( ES_CONTINUOUS | ES_DISPLAY_REQUIRED | ES_SYSTEM_REQUIRED );
    }

    return 0;
}



long    ScreenSaverManagerClose ( SCREENSAVER * pScreenSaver )
{
    if (!pScreenSaver) return -1;

    //Ecran de veille actif
    SystemParametersInfo ( SPI_SETSCREENSAVEACTIVE, (BOOL)pScreenSaver->bScreenSaveActive, NULL, SPIF_UPDATEINIFILE );
    SystemParametersInfo ( SPI_SETSCREENSAVEACTIVE, (BOOL)pScreenSaver->bScreenSaveActive, NULL, SPIF_SENDWININICHANGE );

    //Mise en veille prolongee
    SystemParametersInfo ( SPI_SETPOWEROFFTIMEOUT, (DWORD)pScreenSaver->dwPowerOffTimeOut, NULL, SPIF_UPDATEINIFILE );
    SystemParametersInfo ( SPI_SETPOWEROFFTIMEOUT, (DWORD)pScreenSaver->dwPowerOffTimeOut, NULL, SPIF_SENDWININICHANGE );
    SystemParametersInfo ( SPI_SETPOWEROFFACTIVE, (BOOL)pScreenSaver->bPowerOffActive, NULL, SPIF_UPDATEINIFILE );
    SystemParametersInfo ( SPI_SETPOWEROFFACTIVE, (BOOL)pScreenSaver->bPowerOffActive, NULL, SPIF_SENDWININICHANGE );
    SystemParametersInfo ( SPI_SETLOWPOWERACTIVE, (BOOL)pScreenSaver->bLowPowerActive, NULL, SPIF_UPDATEINIFILE );
    SystemParametersInfo ( SPI_SETLOWPOWERACTIVE, (BOOL)pScreenSaver->bLowPowerActive, NULL, SPIF_SENDWININICHANGE );

    return 0;
}


#else
    //#error Need to implement Platform-dependent Cursor.
    long    ScreenSaverManagerOpen                             ( SCREENSAVER * pScreenSaver )
    {
        return 0;
    }

    long    ScreenSaverManagerSet                              ( SCREENSAVER * pScreenSaver, long bEnableScreenSaver, long bEnablePowerSaver )
    {
        return 0;
    }

    long    ScreenSaverManagerClose                            ( SCREENSAVER * pScreenSaver )
    {
        return 0;
    }
#endif
