/*
===================================================================
Copyright (C) 2009-2010 Editions JOCATOP / Christophe Gossa
 
This file is part of Jocareso.

Jocareso is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Jocareso is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jocareso.  If not, see http://www.gnu.org/licenses/


Editions JOCATOP
Route de Saint Saturnin
84310 MORIERES
FRANCE

Christophe Gossa
christophe.gossa@jocatop.fr
===================================================================
*/

/*

2010-02-23 15:45
strreplace and strireplace functions are probably bugged.
Need to fix them.

Christophe Gossa

*/


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

static long lStrInit = 0;
static char cListStrUCase[256];

unsigned long table_cvt_windows1252[7*128];
unsigned long table_cvt_ascii7bit[7*128];
unsigned long table_cvt_codepage850[7*128];

char * stristr ( char * pszString1, char * pszString2 );
char * strichr ( char * pszString, int cChar );



void strcutonlyascii ( char * psz );
void strcutonlylistchar ( char * psz, char * pszListChar );
void strsupprstr ( char * pszIO, char * pszPattern );
void strsupprchar ( char * pszIO, char cChar );
void strsupprlistchar ( char * pszIO, char * pszListChar );
void strkeeplistchar ( char * pszIO, char * pszListChar );

char * strucase ( char * psz );
void strucasefast ( char * psz );

long strucasecmp ( const char * pszSource, const char * pszTarget );
void strreplace ( char * pszIO, char * pszPattern, char * pszReplacement );
void strireplace ( char * pszIO, char * pszPattern, char * pszReplacement );
void strcreplace ( char * psz, char cOriginal, char cReplace );
void strsupprallright ( char * psz, char cChar );
char * strsupprallleft ( char * psz, char cChar );
char * strgetfileextension ( char * pszSource );
char * strgetfilenameandextension ( char * pszSource );
void strkeeppath ( char * pszIO );
void * memmem ( void * pBuffer, void * pPattern, long lBufferLength, long lPatternLength );
void str2wc ( wchar_t * pwcTarget, char * pszSource );
int strisalpha ( int iChar );

void _strInit ( void );
    
    

/*
===================================================================

Init all internal vars...

===================================================================
*/
void _strInit ( void )
{   
    long i,j;
    
    unsigned long * pTable;

    if (lStrInit) return;
    lStrInit = 1;

    for (i=0;i<256;i++)
    {
        j = i*7;
        if (i<128)
        {
            pTable = table_cvt_ascii7bit;
            j = i*7;
        }
        else
        {
            pTable = table_cvt_windows1252;
            j = (i-128)*7;
        }
        
        if (pTable[j+3]!=-1) //Offset+3 pour avoir uniquement la majuscule
            cListStrUCase[i] = (char)pTable[j+3];
        else
            cListStrUCase[i] = 32;
    }
}

/*
===================================================================

Return the first occurence of pszString2 in pszString1

===================================================================
*/
char * stristr ( char * pszString1, char * pszString2 )
{
    char * psz;
    char * pszSub1;
    char * pszSub2;

    if (!(*pszString2))
        return pszString1;
    

    _strInit ( );
        
    psz = pszString1;
    while (*psz)
    {
        pszSub1 = psz;
        pszSub2 = pszString2;
        while ( (*pszSub1) && (*pszSub2) && (!(cListStrUCase[(unsigned char)*pszSub1]-cListStrUCase[(unsigned char)*pszSub2])) )
        {
            pszSub1++;
            pszSub2++;
        }
        
        if (!(*pszSub2))
            return psz;
            
        psz++;
    }
    
    return NULL;
}
        

/*
===================================================================

Return the first occurence of cChar in pszString

===================================================================
*/
char * strichr ( char * pszString, int cChar )
{
    char * psz;
    char c;
    
    _strInit ( );
        
    psz = pszString;
    c = cListStrUCase[cChar&0xFF];

    while (*psz && cListStrUCase[*psz] != c)
            psz++;

    if (cListStrUCase[*psz] == c)
            return psz;

    return NULL;
}

    
/*
===================================================================

Keep only left ascii part [9;32...255]

===================================================================
*/
void strcutonlyascii ( char * psz )
{
    long i;
    for (i=0;((unsigned)psz[i]>=32)||(psz[i]==9);i++);
    psz[i] = 0;
}

/*
===================================================================

Cut the string at the first occurence of listed pszListChar chars

===================================================================
*/
void strcutonlylistchar ( char * psz, char * pszListChar )
{
    long i;
    for (i=0;psz[i];i++)
    {
        if (strchr(pszListChar,psz[i])==0)
        {
            psz[i] = 0;
            return;
        }
    }
}

/*
===================================================================

Remove pszPattern from pszIO

===================================================================
*/
void strsupprstr ( char * pszIO, char * pszPattern )
{
    char * psz;

    do {
        psz = strstr ( pszIO, pszPattern );
        if (psz) memmove ( psz, psz+strlen(pszPattern), (strlen(psz)+1)-(strlen(pszPattern)+1)+1 );
    } while (psz);
}

/*
===================================================================

Remove cChar from pszIO

===================================================================
*/
void strsupprchar ( char * pszIO, char cChar )
{
    char szTmp[2];
    szTmp[0] = cChar;
    szTmp[1] = 0;
    strsupprstr ( pszIO, szTmp );
}

/*
===================================================================

Remove a list of chars from pszIO

===================================================================
*/
void strsupprlistchar ( char * pszIO, char * pszListChar )
{
    char szTmp[2];
    long i;
    szTmp[1] = 0;
    for (i=0;pszListChar[i];i++)
    {
        szTmp[0] = pszListChar[i];
        strsupprstr ( pszIO, szTmp );
    }
}


/*
===================================================================

Keep only the listed chars pszListChar from pszIO

===================================================================
*/
void strkeeplistchar ( char * pszIO, char * pszListChar )
{
    char szTmp[256];
    long i;

    //Generation de la liste des caracteres a supprimer
    for (i=32;i<255;i++) szTmp[i-32] = (char)i;
    szTmp[i-32] = 0;

    //Suppression des caracteres que l'on veut conserver
    strsupprlistchar ( szTmp, pszListChar );

    //Suppression des caracteres
    strsupprlistchar ( pszIO, szTmp );
}


/*
===================================================================

Go to uppercase

===================================================================
*/
char * strucase ( char * psz )
{
	int i;
	_strInit ( );
    
    for (i=0;psz[i]!=0;i++)
    {
		psz[i] = cListStrUCase[(unsigned char)psz[i]];
    }
	return psz;
}

/*
===================================================================

Go to uppercase

===================================================================
*/
void strucasefast ( char * psz )
{
    strucase ( psz );
}

/*
===================================================================

Compare without regarding case between Source and Target

===================================================================
*/
long strucasecmp ( const char * pszSource, const char * pszTarget )
{
    long ret;
    unsigned char bSource, bTarget;

    _strInit ( );
    ret = 0;

    do {
        bSource = (unsigned char)cListStrUCase[(unsigned char)*pszSource];
        bTarget = (unsigned char)cListStrUCase[(unsigned char)*pszTarget];
        pszSource++;
        pszTarget++;
    } while ( !(ret=bSource-bTarget) && bTarget && bSource );

    if (ret<0)      ret = -1;
    else if (ret>0) ret =  1;

    return ret;
}

/*
===================================================================

Search and replace pszPattern with pszReplacement into pszIO
Probably bugged... Need to fix it !

===================================================================
*/
void strreplace ( char * pszIO, char * pszPattern, char * pszReplacement )
{
    char * psz;
    long lPatternLength;
    long lReplacementLength;

    lPatternLength      = (long)strlen(pszPattern);

    if (pszReplacement==NULL) lReplacementLength = 0;
    else                      lReplacementLength = (long)strlen(pszReplacement);
    
    psz = pszIO;
    
    do {
        psz = strstr ( psz, pszPattern );
        if (psz)
        {
            //Deplacement de la chaine !
            memmove ( psz+lReplacementLength, psz+lPatternLength, strlen ( psz+lPatternLength ) + 1 );
            if (pszReplacement) memmove ( psz, pszReplacement, lReplacementLength );
            psz += lReplacementLength;
        }
    } while (psz);
}

/*
===================================================================

Search and replace pszPattern with pszReplacement into pszIO
without regarding case.
Probably bugged... Need to fix it !

===================================================================
*/
void strireplace ( char * pszIO, char * pszPattern, char * pszReplacement )
{
    char * psz;
    long lPatternLength;
    long lReplacementLength;
    
    _strInit ( );

    lPatternLength = (long)strlen(pszPattern);
    if (pszReplacement==NULL) lReplacementLength = 0;
    else                      lReplacementLength = (long)strlen(pszReplacement);
    psz = pszIO;
    do {
        psz = stristr ( psz, pszPattern );
        if (psz)
        {
            //Deplacement de la chaine !
            memmove ( psz+lReplacementLength, psz+lPatternLength, strlen ( psz+lPatternLength ) + 1 );
            if (pszReplacement) memmove ( psz, pszReplacement, lReplacementLength );
            psz += lReplacementLength;
        }
    } while (psz);
}

/*
===================================================================

Search and replace cOriginal with cReplace into pszIO

===================================================================
*/
void strcreplace ( char * psz, char cOriginal, char cReplace )
{
    long i,n;
    n = (long)strlen(psz);
    for (i=0;i<n;i++)
    {
        if (psz[i] == cOriginal) psz[i] = cReplace;
    }
}

/*
===================================================================

Kill all found cChar right characters...

===================================================================
*/
void strsupprallright ( char * psz, char cChar )
{
    long i,n;
    n = (long)strlen(psz);

    if (!n) return;

    for (i=n-1;i>=0;i++)
    {
        if (psz[i]==cChar) psz[i] = 0;
        else               return;
    }
}

/*
===================================================================

Kill all found cChar left characters and apply a left shift...

===================================================================
*/
char * strsupprallleft ( char * psz, char cChar )
{
    long i,n;
    
    n = (long)strlen(psz);

    if (!n) return psz;

    for (i=0;psz[i];i++)
    {
        if (psz[i]!=cChar) return &(psz[i]);
    }
    return &(psz[n]);
}

/*
===================================================================

Get the file extension (searching for the 'DOT' character)

===================================================================
*/
char * strgetfileextension ( char * pszSource )
{
    long i;

    for (i=(long)strlen(pszSource)-1;i>=0;i--)
    {
        if (pszSource[i]=='.')
        {
            return pszSource+i;
        }
    }

    return NULL;
}

/*
===================================================================

Get the file name and extension, after the last slash or back-slash

===================================================================
*/
char * strgetfilenameandextension ( char * pszSource )
{
    long i;

    for (i=(long)strlen(pszSource)-1;i>=0;i--)
    {
        if ((pszSource[i]=='\\')||(pszSource[i]=='/'))
        {
            return pszSource+i+1;
        }
    }

    return pszSource;
}

/*
===================================================================

Keep path only and kill other characters
If no slash or back-slash found, it's a file name !

===================================================================
*/
void strkeeppath ( char * pszIO )
{
    long i;

    for (i=(long)strlen(pszIO)-1;i>=0;i--)
    {
        if ((pszIO[i]=='\\')||(pszIO[i]=='/'))
        {
            pszIO[i+1] = 0;
            return;
        }
    }
    pszIO[0] = 0;
}
    
/*
===================================================================

Find pPattern in pBuffer

===================================================================
*/
void * memmem ( void * pBuffer, void * pPattern, long lBufferLength, long lPatternLength )
{
    char * pSource;
    char * pFinder;
    //long lFinderLength;
    long lFoundLength;

    if ( (!pPattern) || 
            (!pBuffer) || 
            (!lBufferLength) ||
            (lBufferLength<lPatternLength) ) return NULL;

    pSource         = (char *)pBuffer;
    pFinder         = (char *)pPattern;
    lFoundLength    = 0;

    lBufferLength -= (lPatternLength-1);

    do {

        if (*pSource == *pFinder)
        {
            if (!memcmp(pSource,pFinder,lPatternLength))
                return pSource;
        }

        lBufferLength--;
        pSource++;
    } while (lBufferLength);

    return NULL;
}

/*
===================================================================

String to WCHAR (just simple copy)

===================================================================
*/
void str2wc ( wchar_t * pwcTarget, char * pszSource )
{
    long i;
    if ((!pwcTarget)||(!pszSource)) return;

    for (i=0;pszSource[i];i++)
    {
        pwcTarget[i] = (wchar_t)pszSource[i];
    }
    pwcTarget[i] = 0;
}


/*
===================================================================

Is this character is alpha ?

===================================================================
*/
int strisalpha ( int iChar )
{
    unsigned long c;
    
    _strInit ( );
    
    c = (unsigned long)cListStrUCase[((unsigned)iChar)&0xFF];
    
    return ((c>='A')&&(c<='Z'))?(1):(0);
}

/*
===================================================================

auto path completition for Windows and Linux
if char 92 or 47 is founded...

===================================================================
*/
void strautopathcompletition ( char * pszPath, char cDefaultSeparator )
{
    char cPathChar;
    char *psz92, *psz47, *psz;
    long lLen;
    
    if ((cDefaultSeparator==0)||(cDefaultSeparator==127))
        cPathChar = 92; //The default path char separator is for Windows
    else
        cPathChar = cDefaultSeparator;
        
    psz92 = strrchr ( pszPath, 92 ); // searching for '\\'
    psz47 = strrchr ( pszPath, 47 ); // searching for '/'
    psz   = strrchr ( pszPath, cPathChar );

    //don't add anything if the separator isn't found
    if ((!psz47)&&(!psz92)&&(!psz))
        return;

    //Learn for Linux separator char    
    if ((psz47)&&(!psz92))
        cPathChar = 47;
    else if ((psz47)&&(psz92)&&(psz47>psz92))
        cPathChar = 47;

    lLen = (long)strlen(pszPath);
    if (lLen)
    {
        if (pszPath[lLen-1]!=cPathChar)
        {
            pszPath[lLen]=cPathChar;
            pszPath[lLen+1]=0;
        }
    }
}
    
/*
===================================================================

Table code page 850

===================================================================
*/
    
    /*
                -1 si ce n'est pas une lettre et dont le comportement peut varier...
    Valeur     Pas d'accent         Majuscule           Minuscule           ASCII       Commentaire
    8-bit      Meme casse           Acc, pas-acc        Acc, pas-acc        7-bit       de controle
    */
unsigned long table_cvt_ascii7bit[7*128] = {
      0,          0,                  0,   0,             0,   0,             0,
      1,          1,                  1,   1,             1,   1,             1,
      2,          2,                  2,   2,             2,   2,             2,
      3,          3,                  3,   3,             3,   3,             3,
      4,          4,                  4,   4,             4,   4,             4,
      5,          5,                  5,   5,             5,   5,             5,
      6,          6,                  6,   6,             6,   6,             6,
      7,          7,                  7,   7,             7,   7,             7,
      8,          8,                  8,   8,             8,   8,             8,
      9,          9,                  9,   9,             9,   9,             9,
     10,         10,                 10,  10,            10,  10,            10,
     11,         11,                 11,  11,            11,  11,            11,
     12,         12,                 12,  12,            12,  12,            12,
     13,         13,                 13,  13,            13,  13,            13,
     14,         14,                 14,  14,            14,  14,            14,
     15,         15,                 15,  15,            15,  15,            15,
     16,         16,                 16,  16,            16,  16,            16,
     17,         17,                 17,  17,            17,  17,            17,
     18,         18,                 18,  18,            18,  18,            18,
     19,         19,                 19,  19,            19,  19,            19,
     20,         20,                 20,  20,            20,  20,            20,
     21,         21,                 21,  21,            21,  21,            21,
     22,         22,                 22,  22,            22,  22,            22,
     23,         23,                 23,  23,            23,  23,            23,
     24,         24,                 24,  24,            24,  24,            24,
     25,         25,                 25,  25,            25,  25,            25,
     26,         26,                 26,  26,            26,  26,            26,
     27,         27,                 27,  27,            27,  27,            27,
     28,         28,                 28,  28,            28,  28,            28,
     29,         29,                 29,  29,            29,  29,            29,
     30,         30,                 30,  30,            30,  30,            30,
     31,         31,                 31,  31,            31,  31,            31,
     32,         32,                 32,  32,            32,  32,            32,
     33,         33,                 33,  33,            33,  33,            33,
     34,         34,                 34,  34,            34,  34,            34,
     35,         35,                 35,  35,            35,  35,            35,
     36,         36,                 36,  36,            36,  36,            36,
     37,         37,                 37,  37,            37,  37,            37,
     38,         38,                 38,  38,            38,  38,            38,
     39,         39,                 39,  39,            39,  39,            39,
     40,         40,                 40,  40,            40,  40,            40,
     41,         41,                 41,  41,            41,  41,            41,
     42,         42,                 42,  42,            42,  42,            42,
     43,         43,                 43,  43,            43,  43,            43,
     44,         44,                 44,  44,            44,  44,            44,
     45,         45,                 45,  45,            45,  45,            45,
     46,         46,                 46,  46,            46,  46,            46,
     47,         47,                 47,  47,            47,  47,            47,
     48,         48,                 48,  48,            48,  48,            48,
     49,         49,                 49,  49,            49,  49,            49,
     50,         50,                 50,  50,            50,  50,            50,
     51,         51,                 51,  51,            51,  51,            51,
     52,         52,                 52,  52,            52,  52,            52,
     53,         53,                 53,  53,            53,  53,            53,
     54,         54,                 54,  54,            54,  54,            54,
     55,         55,                 55,  55,            55,  55,            55,
     56,         56,                 56,  56,            56,  56,            56,
     57,         57,                 57,  57,            57,  57,            57,
     58,         58,                 58,  58,            58,  58,            58,
     59,         59,                 59,  59,            59,  59,            59,
     60,         60,                 60,  60,            60,  60,            60,
     61,         61,                 61,  61,            61,  61,            61,
     62,         62,                 62,  62,            62,  62,            62,
     63,         63,                 63,  63,            63,  63,            63,
     64,         64,                 64,  64,            64,  64,            64,
    
     65,        'A',                'A', 'A',           'a', 'a',           'A',
     66,        'B',                'B', 'B',           'b', 'b',           'B',     
     67,        'C',                'C', 'C',           'c', 'c',           'C',
     68,        'D',                'D', 'D',           'd', 'd',           'D',
     69,        'E',                'E', 'E',           'e', 'e',           'E',
     70,        'F',                'F', 'F',           'f', 'f',           'F',
     71,        'G',                'G', 'G',           'g', 'g',           'G',
     72,        'H',                'H', 'H',           'h', 'h',           'H',
     73,        'I',                'I', 'I',           'i', 'i',           'I',
     74,        'J',                'J', 'J',           'j', 'j',           'J',
     75,        'K',                'K', 'K',           'k', 'k',           'K',
     76,        'L',                'L', 'L',           'l', 'l',           'L',
     77,        'M',                'M', 'M',           'm', 'm',           'M',
     78,        'N',                'N', 'N',           'n', 'n',           'N',
     79,        'O',                'O', 'O',           'o', 'o',           'O',
     80,        'P',                'P', 'P',           'p', 'p',           'P',
     81,        'Q',                'Q', 'Q',           'q', 'q',           'Q',
     82,        'R',                'R', 'R',           'r', 'r',           'R',
     83,        'S',                'S', 'S',           's', 's',           'S',
     84,        'T',                'T', 'T',           't', 't',           'T',
     85,        'U',                'U', 'U',           'u', 'u',           'U',
     86,        'V',                'V', 'V',           'v', 'v',           'V',
     87,        'W',                'W', 'W',           'w', 'w',           'W',
     88,        'X',                'X', 'X',           'x', 'x',           'X',
     89,        'Y',                'Y', 'Y',           'y', 'y',           'Y',
     90,        'Z',                'Z', 'Z',           'z', 'z',           'Z',
    
     91,         91,                 91,  91,            91,  91,            91,   
     92,         92,                 92,  92,            92,  92,            92,   
     93,         93,                 93,  93,            93,  93,            93,   
     94,         94,                 94,  94,            94,  94,            94,   
     95,         95,                 95,  95,            95,  95,            95,   
     96,         96,                 96,  96,            96,  96,            96,
     
     97,        'a',                'A', 'A',           'a', 'a',           'a',
     98,        'b',                'B', 'B',           'b', 'b',           'b',     
     99,        'c',                'C', 'C',           'c', 'c',           'c',
    100,        'd',                'D', 'D',           'd', 'd',           'd',
    101,        'e',                'E', 'E',           'e', 'e',           'e',
    102,        'f',                'F', 'F',           'f', 'f',           'f',
    103,        'g',                'G', 'G',           'g', 'g',           'g',
    104,        'h',                'H', 'H',           'h', 'h',           'h',
    105,        'i',                'I', 'I',           'i', 'i',           'i',
    106,        'j',                'J', 'J',           'j', 'j',           'j',
    107,        'k',                'K', 'K',           'k', 'k',           'k',
    108,        'l',                'L', 'L',           'l', 'l',           'l',
    109,        'm',                'M', 'M',           'm', 'm',           'm',
    110,        'n',                'N', 'N',           'n', 'n',           'n',
    111,        'o',                'O', 'O',           'o', 'o',           'o',
    112,        'p',                'P', 'P',           'p', 'p',           'p',
    113,        'q',                'Q', 'Q',           'q', 'q',           'q',
    114,        'r',                'R', 'R',           'r', 'r',           'r',
    115,        's',                'S', 'S',           's', 's',           's',
    116,        't',                'T', 'T',           't', 't',           't',
    117,        'u',                'U', 'U',           'u', 'u',           'u',
    118,        'v',                'V', 'V',           'v', 'v',           'v',
    119,        'w',                'W', 'W',           'w', 'w',           'w',
    120,        'x',                'X', 'X',           'x', 'x',           'x',
    121,        'y',                'Y', 'Y',           'y', 'y',           'y',
    122,        'z',                'Z', 'Z',           'z', 'z',           'z',
    
    123,        123,                123, 123,           123, 123,           123,
    124,        124,                124, 124,           124, 124,           124,
    125,        125,                125, 125,           125, 125,           125,
    126,        126,                126, 126,           126, 126,           126,  
    127,        127,                127, 127,           127, 127,           127,
};    

unsigned long table_cvt_windows1252[7*128] = {    
    128,        'E',                128, 'E',           128, 'e',           'E',        //Euro
    129,         -1,                 -1,  -1,            -1,  -1,            -1,        //unknow
    130,         -1,                 -1,  -1,            -1,  -1,            -1,        //unknow
    132,         -1,                 -1,  -1,            -1,  -1,            -1,        //unknow
    133,         -1,                 -1,  -1,            -1,  -1,            -1,        //unknow
    134,         -1,                 -1,  -1,            -1,  -1,            -1,        //unknow
    135,         -1,                 -1,  -1,            -1,  -1,            -1,        //unknow
    136,         -1,                 -1,  -1,            -1,  -1,            -1,        //unknow
    137,         -1,                 -1,  -1,            -1,  -1,            -1,        //unknow
    138,        'S',                138, 'S',           154, 's',           'S',        //S avec ^ a l'envers
    139,         -1,                 -1,  -1,            -1,  -1,            -1,        //unknow
    140,        'O',                140, 'O',           156, 'o',           'O',        //OE avec liaison
    141,         -1,                 -1,  -1,            -1,  -1,            -1,        //unknow
    142,        'Z',                142, 'Z',           158, 'z',           'Z',        //Z avec ^ a l'envers
    143,         -1,                 -1,  -1,            -1,  -1,            -1,        //unknow
    144,         -1,                 -1,  -1,            -1,  -1,            -1,        //unknow
    145,         -1,                 -1,  -1,            -1,  -1,            -1,        //unknow
    146,         -1,                 -1,  -1,            -1,  -1,            -1,        //unknow
    147,         -1,                 -1,  -1,            -1,  -1,            -1,        //unknow
    148,         -1,                 -1,  -1,            -1,  -1,            -1,        //unknow
    149,         -1,                 -1,  -1,            -1,  -1,            -1,        //unknow
    150,         -1,                 -1,  -1,            -1,  -1,            -1,        //unknow
    151,         -1,                 -1,  -1,            -1,  -1,            -1,        //unknow
    152,         -1,                 -1,  -1,            -1,  -1,            -1,        //unknow
    153,         -1,                 -1,  -1,            -1,  -1,            -1,        //unknow
    154,        's',                138, 'S',           154, 's',           's',        //s avec ^ a l'envers
    155,         -1,                 -1,  -1,            -1,  -1,            -1,        //unknow
    156,        'o',                140, 'O',           156, 'o',           'o',        //oe avec liaison
    157,         -1,                 -1,  -1,            -1,  -1,            -1,        //unknow
    158,        'z',                142, 'Z',           158, 'z',           'z',        //z avec ^ a l'envers
    159,        'Y',                159, 'Y',           255, 'y',           'Y',        //Y avec ¨
    160,         -1,                 -1,  -1,            -1,  -1,            -1,        //Espace insecable (NBSP)
    161,         -1,                 -1,  -1,            -1,  -1,            -1,        //! a l'envers
    162,        'c',                'C', 'C',           162, 'c',           'c',        //c barre
    163,        'L',                163, 'L',           'l', 'l',           'L',        //La Livre
    164,         -1,                 -1,  -1,            -1,  -1,            -1,        //Un truc bizarre
    165,        'Y',                165, 'Y',           'y', 'y',           'Y',        //Le Yen
    166,         -1,                 -1,  -1,            -1,  -1,            -1,        //Barre verticale coupee
    167,         -1,                 -1,  -1,            -1,  -1,            -1,        //Paragraphe
    168,         -1,                 -1,  -1,            -1,  -1,            -1,        //Petit trait haut horizontal
    169,         -1,                 -1,  -1,            -1,  -1,            -1,        //(c)
    170,         -1,                 -1,  -1,            -1,  -1,            -1,        //petit a
    171,         -1,                 -1,  -1,            -1,  -1,            -1,        //guillement d'ouverture
    172,         -1,                 -1,  -1,            -1,  -1,            -1,        //
    173,         -1,                 -1,  -1,            -1,  -1,            -1,        //
    174,         -1,                 -1,  -1,            -1,  -1,            -1,        //
    175,         -1,                 -1,  -1,            -1,  -1,            -1,        //
    176,         -1,                 -1,  -1,            -1,  -1,            -1,        //
    177,         -1,                 -1,  -1,            -1,  -1,            -1,        //
    178,         -1,                 -1,  -1,            -1,  -1,            -1,        //
    179,         -1,                 -1,  -1,            -1,  -1,            -1,        //
    180,         -1,                 -1,  -1,            -1,  -1,            -1,        //
    181,        'U',                181, 'U',           181, 'u',           'u',        //micro
    182,         -1,                 -1,  -1,            -1,  -1,            -1,        //
    183,         -1,                 -1,  -1,            -1,  -1,            -1,        //
    184,         -1,                 -1,  -1,            -1,  -1,            -1,        //
    185,         -1,                 -1,  -1,            -1,  -1,            -1,        //
    186,         -1,                 -1,  -1,            -1,  -1,            -1,        //
    187,         -1,                 -1,  -1,            -1,  -1,            -1,        //
    188,         -1,                 -1,  -1,            -1,  -1,            -1,        //
    189,         -1,                 -1,  -1,            -1,  -1,            -1,        //
    190,         -1,                 -1,  -1,            -1,  -1,            -1,        //
    191,         -1,                 -1,  -1,            -1,  -1,            -1,        //
    0xC0,       'A',               0xC0, 'A',          0xE0, 'a',           'A',        //
    0xC1,       'A',               0xC1, 'A',          0xE1, 'a',           'A',        //
    0xC2,       'A',               0xC2, 'A',          0xE2, 'a',           'A',        //
    0xC3,       'A',               0xC3, 'A',          0xE3, 'a',           'A',        //
    0xC4,       'A',               0xC4, 'A',          0xE4, 'a',           'A',        //
    0xC5,       'A',               0xC5, 'A',          0xE5, 'a',           'A',        //
    0xC6,       'A',               0xC6, 'A',          0xE6, 'a',           'A',        //
    0xC7,       'C',               0xC7, 'C',          0xE7, 'c',           'C',        //
    0xC8,       'E',               0xC8, 'E',          0xE8, 'e',           'E',        //
    0xC9,       'E',               0xC9, 'E',          0xE9, 'e',           'E',        //
    0xCA,       'E',               0xCA, 'E',          0xEA, 'e',           'E',        //
    0xCB,       'E',               0xCB, 'E',          0xEB, 'e',           'E',        //
    0xCC,       'I',               0xCC, 'I',          0xEC, 'i',           'I',        //
    0xCD,       'I',               0xCD, 'I',          0xED, 'i',           'I',        //
    0xCE,       'I',               0xCE, 'I',          0xEE, 'i',           'I',        //
    0xCF,       'I',               0xCF, 'I',          0xEF, 'i',           'I',        //
    0xD0,       'D',               0xD0, 'D',          0xD0, 'd',           'D',        //
    0xD1,       'N',               0xD1, 'N',          0xF1, 'n',           'N',        //
    0xD2,       'O',               0xD2, 'O',          0xF2, 'o',           'O',        //
    0xD3,       'O',               0xD3, 'O',          0xF3, 'o',           'O',        //
    0xD4,       'O',               0xD4, 'O',          0xF4, 'o',           'O',        //
    0xD5,       'O',               0xD5, 'O',          0xF5, 'o',           'O',        //
    0xD6,       'O',               0xD6, 'O',          0xF6, 'o',           'O',        //
    0xD7,       '*',               0xD7, '*',          0xD7, '*',           '*',        //
    0xD8,       'O',               0xD8, 'O',          0xF8, 'o',           'O',        //
    0xD9,       'U',               0xD9, 'A',          0xF9, 'a',           'A',        //
    0xDA,       'U',               0xDA, 'A',          0xFA, 'a',           'A',        //
    0xDB,       'U',               0xDB, 'A',          0xFB, 'a',           'A',        //
    0xDC,       'U',               0xDC, 'A',          0xFC, 'a',           'A',        //
    0xDD,       'Y',               0xDD, 'A',          0xFD, 'a',           'A',        //
    0xDE,        -1,                 -1,  -1,            -1,  -1,            -1,        //
    0xDF,       'S',               0xDF, 'S',          0xDF, 's',           'S',        //Le double S allemand
    
    0xE0,       'a',               0xC0, 'A',          0xE0, 'a',           'A',        //
    0xE1,       'a',               0xC1, 'A',          0xE1, 'a',           'A',        //
    0xE2,       'a',               0xC2, 'A',          0xE2, 'a',           'A',        //
    0xE3,       'a',               0xC3, 'A',          0xE3, 'a',           'A',        //
    0xE4,       'a',               0xC4, 'A',          0xE4, 'a',           'A',        //
    0xE5,       'a',               0xC5, 'A',          0xE5, 'a',           'A',        //
    0xE6,       'a',               0xC6, 'A',          0xE6, 'a',           'A',        //
    0xE7,       'c',               0xC7, 'C',          0xE7, 'c',           'C',        //
    0xE8,       'e',               0xC8, 'E',          0xE8, 'e',           'E',        //
    0xE9,       'e',               0xC9, 'E',          0xE9, 'e',           'E',        //
    0xEA,       'e',               0xCA, 'E',          0xEA, 'e',           'E',        //
    0xEB,       'e',               0xCB, 'E',          0xEB, 'e',           'E',        //
    0xEC,       'i',               0xCC, 'I',          0xEC, 'i',           'I',        //
    0xED,       'i',               0xCD, 'I',          0xED, 'i',           'I',        //
    0xEE,       'i',               0xCE, 'I',          0xEE, 'i',           'I',        //
    0xEF,       'i',               0xCF, 'I',          0xEF, 'i',           'I',        //
    0xF0,       'o',               0xF0, 'O',          0xF0, 'o',           'o',        //
    0xF1,       'n',               0xD1, 'N',          0xF1, 'n',           'N',        //
    0xF2,       'o',               0xD2, 'O',          0xF2, 'o',           'O',        //
    0xF3,       'o',               0xD3, 'O',          0xF3, 'o',           'O',        //
    0xF4,       'o',               0xD4, 'O',          0xF4, 'o',           'O',        //
    0xF5,       'o',               0xD5, 'O',          0xF5, 'o',           'O',        //
    0xF6,       'o',               0xD6, 'O',          0xF6, 'o',           'O',        //
    0xF7,        47,               0xF7,  47,          0xF7,  47,            47,        // division
    0xF8,       'o',               0xD8, 'O',          0xF8, 'o',           'O',        //
    0xF9,       'u',               0xD9, 'A',          0xF9, 'a',           'A',        //
    0xFA,       'u',               0xDA, 'A',          0xFA, 'a',           'A',        //
    0xFB,       'u',               0xDB, 'A',          0xFB, 'a',           'A',        //
    0xFC,       'u',               0xDC, 'A',          0xFC, 'a',           'A',        //
    0xFD,       'y',               0xDD, 'A',          0xFD, 'a',           'A',        //
    0xFE,        -1,                 -1,  -1,            -1,  -1,            -1,        //
    0xFF,       'y',               0x9F, 'Y',          0xFF, 'y',           'y',        //Le double S allemand
};
    
unsigned long table_cvt_codepage850[7*128] = {    
    128,        'C',                128, 'C',           135, 'c',           'C',        //Çç
    129,        'u',                154, 'U',           129, 'u',           'u',        //üÜ
    130,        'e',                144, 'E',           130, 'e',           'e',        //éÉ 
    131,        'a',                182, 'A',           131, 'a',           'a',        //âÂ
    132,        'a',                142, 'A',           132, 'a',           'a',        //äÄ
    133,        'a',                183, 'A',           133, 'a',           'a',        //àÀ
    134,        'a',                143, 'A',           134, 'a',           'a',        //åÅ
    135,        'c',                128, 'C',           135, 'c',           'c',        //çÇ
    136,        'e',                210, 'E',           136, 'e',           'e',        //êÊ
    137,        'e',                211, 'E',           137, 'e',           'e',        //ëË
    138,        'e',                212, 'E',           138, 'e',           'e',        //èÈ
    139,        'i',                216, 'I',           139, 'i',           'i',        //ïÏ
    140,        'i',                215, 'I',           140, 'i',           'i',        //îÎ
    141,        'i',                222, 'I',           141, 'i',           'i',        //ìÌ
    142,        'A',                142, 'A',           132, 'a',           'A',        //Ää
    143,        'A',                143, 'A',           134, 'a',           'A',        //Åå
    144,        'E',                144, 'E',           130, 'e',           'E',        //Éé
    145,        'a',                146, 'A',           145, 'a',           'a',        //æÆ
    146,        'A',                146, 'A',           145, 'a',           'A',        //Ææ
    147,        'o',                226, 'O',           147, 'o',           'o',        //ôÔ
    148,        'o',                153, 'O',           148, 'o',           'o',        //öÖ
    149,        'o',                227, 'O',           149, 'o',           'o',        //òÒ
    150,        'u',                234, 'U',           150, 'u',           'u',        //ûÛ
    151,        'u',                235, 'U',           151, 'u',           'u',        //ùÙ
    152,        'y',                'Y', 'Y',           152, 'y',           'y',        //ÿ
    153,        'O',                153, 'O',           148, 'o',           'O',        //Öö
    154,        'U',                154, 'U',           129, 'u',           'U',        //Üü
    155,        'o',                157, 'o',           155, 'o',           'o',        //øØ
    156,        'L',                156, 'L',           'l', 'l',           'L',        //£
    157,        'O',                157, 'O',           155, 'o',           'O',        //Øø
    158,         -1,                 -1,  -1,            -1,  -1,            -1,        //×
    159,         -1,                 -1,  -1,            -1,  -1,            -1,        //ƒ
    160,        'a',                181, 'A',           160, 'a',           'a',        //áÁ
    161,        'i',                214, 'I',           161, 'i',           'i',        //íÍ
    162,        'o',                224, 'O',           162, 'o',           'o',        //óÓ
    163,        'u',                233, 'U',           163, 'u',           'u',        //úÚ
    164,        'n',                165, 'N',           164, 'n',           'n',        //ñÑ
    165,        'N',                165, 'N',           164, 'n',           'N',        //Ññ
    166,        'a',                'A', 'A',           166, 'a',           'a',        //ª
    167,        'o',                'O', 'O',           167, 'o',           'o',        //º
    168,         -1,                 -1,  -1,            -1,  -1,            -1,        //¿
    169,         -1,                 -1,  -1,            -1,  -1,            -1,        //®
    170,         -1,                 -1,  -1,            -1,  -1,            -1,        //¬
    171,         -1,                 -1,  -1,            -1,  -1,            -1,        //½
    172,         -1,                 -1,  -1,            -1,  -1,            -1,        //¼
    173,         -1,                 -1,  -1,            -1,  -1,            -1,        //¡
    174,         -1,                 -1,  -1,            -1,  -1,            -1,        //«
    175,         -1,                 -1,  -1,            -1,  -1,            -1,        //»
    176,         -1,                 -1,  -1,            -1,  -1,            -1,        //░
    177,         -1,                 -1,  -1,            -1,  -1,            -1,        //▒
    178,         -1,                 -1,  -1,            -1,  -1,            -1,        //▓
    179,         -1,                 -1,  -1,            -1,  -1,            -1,        //│
    180,         -1,                 -1,  -1,            -1,  -1,            -1,        //┤
    181,        'A',                181, 'A',           160, 'a',           'a',        //Áá
    182,        'A',                182, 'A',           131, 'a',           'a',        //Ââ
    183,        'A',                183, 'A',           133, 'a',           'a',        //Àà
    184,         -1,                 -1,  -1,            -1,  -1,            -1,        //©
    185,         -1,                 -1,  -1,            -1,  -1,            -1,        //╣
    186,         -1,                 -1,  -1,            -1,  -1,            -1,        //║
    187,         -1,                 -1,  -1,            -1,  -1,            -1,        //╗
    188,         -1,                 -1,  -1,            -1,  -1,            -1,        //╝
    189,        'c',                'C', 'C',           189, 'c',           'c',        //¢
    190,        'Y',                190, 'Y',           'y', 'y',           'Y',        //¥
    191,         -1,                 -1,  -1,            -1,  -1,            -1,        //┐
    192,         -1,                 -1,  -1,            -1,  -1,            -1,        //└
    193,         -1,                 -1,  -1,            -1,  -1,            -1,        //┴
    194,         -1,                 -1,  -1,            -1,  -1,            -1,        //┬
    195,         -1,                 -1,  -1,            -1,  -1,            -1,        //├
    196,         -1,                 -1,  -1,            -1,  -1,            -1,        //─
    197,         -1,                 -1,  -1,            -1,  -1,            -1,        //┼
    198,        'a',                199, 'A',           198, 'a',           'a',        //ãÃ
    199,        'A',                199, 'A',           198, 'a',           'A',        //Ãã
    200,         -1,                 -1,  -1,            -1,  -1,            -1,        //╚
    201,         -1,                 -1,  -1,            -1,  -1,            -1,        //╔
    202,         -1,                 -1,  -1,            -1,  -1,            -1,        //╩
    203,         -1,                 -1,  -1,            -1,  -1,            -1,        //╦
    204,         -1,                 -1,  -1,            -1,  -1,            -1,        //╠
    205,         -1,                 -1,  -1,            -1,  -1,            -1,        //═
    206,         -1,                 -1,  -1,            -1,  -1,            -1,        //╬
    207,         -1,                 -1,  -1,            -1,  -1,            -1,        //¤
    208,        'o',                'O', 'O',           208, 'o',           'o',        //ð
    209,        'D',                209, 'D',           'd', 'd',           'D',        //Ð
    210,        'E',                210, 'E',           136, 'e',           'E',        //Êê
    211,        'E',                211, 'E',           137, 'e',           'E',        //Ëë
    212,        'E',                212, 'E',           138, 'e',           'E',        //Èè
    213,         -1,                 -1,  -1,            -1,  -1,            -1,        //ı
    214,        'I',                214, 'I',           161, 'i',           'I',        //Íí
    215,        'I',                215, 'I',           140, 'i',           'I',        //Îî
    216,        'I',                216, 'I',           139, 'i',           'I',        //Ïï
    217,         -1,                 -1,  -1,            -1,  -1,            -1,        //┘
    218,         -1,                 -1,  -1,            -1,  -1,            -1,        //┌
    219,         -1,                 -1,  -1,            -1,  -1,            -1,        //█
    220,         -1,                 -1,  -1,            -1,  -1,            -1,        //▄
    221,         -1,                 -1,  -1,            -1,  -1,            -1,        //¦
    222,        'I',                222, 'I',           141, 'i',           'I',        //Ìì
    223,         -1,                 -1,  -1,            -1,  -1,            -1,        //▀
    224,        'O',                224, 'O',           162, 'o',           'O',        //Óó
    225,        's',                'S', 'S',           225, 's',           's',        //ß
    226,        'O',                226, 'O',           147, 'o',           'O',        //Ôô
    227,        'O',                227, 'O',           149, 'o',           'O',        //Òò
    228,        'o',                229, 'O',           228, 'o',           'o',        //õÕ
    229,        'O',                229, 'O',           228, 'o',           'O',        //Õõ
    230,        'u',                'U', 'U',           230, 'u',           'u',        //µ
    231,         -1,                 -1,  -1,            -1,  -1,            -1,        //þ
    232,         -1,                 -1,  -1,            -1,  -1,            -1,        //Þ
    233,        'U',                233, 'U',           163, 'u',           'U',        //Úú
    234,        'U',                234, 'U',           150, 'u',           'U',        //Ûû
    235,        'U',                235, 'U',           151, 'u',           'U',        //Ùù
    236,        'y',                237, 'Y',           236, 'y',           'y',        //ýÝ
    237,        'Y',                237, 'Y',           236, 'y',           'Y',        //Ýý
    238,         -1,                 -1,  -1,            -1,  -1,            -1,        //¯
    239,         -1,                 -1,  -1,            -1,  -1,            -1,        //´
    240,         -1,                 -1,  -1,            -1,  -1,            -1,        //­
    241,         -1,                 -1,  -1,            -1,  -1,            -1,        //±
    242,         -1,                 -1,  -1,            -1,  -1,            -1,        //‗
    243,         -1,                 -1,  -1,            -1,  -1,            -1,        //¾
    244,         -1,                 -1,  -1,            -1,  -1,            -1,        //¶
    245,         -1,                 -1,  -1,            -1,  -1,            -1,        //§
    246,         -1,                 -1,  -1,            -1,  -1,            -1,        //÷ => /
    247,         -1,                 -1,  -1,            -1,  -1,            -1,        //¸
    248,         -1,                 -1,  -1,            -1,  -1,            -1,        //°
    249,         -1,                 -1,  -1,            -1,  -1,            -1,        //¨
    250,         -1,                 -1,  -1,            -1,  -1,            -1,        //·
    251,         -1,                 -1,  -1,            -1,  -1,            -1,        //¹
    252,         -1,                 -1,  -1,            -1,  -1,            -1,        //³
    253,         -1,                 -1,  -1,            -1,  -1,            -1,        //²
    254,         -1,                 -1,  -1,            -1,  -1,            -1,        //■
    255,         -1,                 -1,  -1,            -1,  -1,            -1,        // 
};


