/*
===================================================================
Copyright (C) 2009-2010 Editions JOCATOP / Christophe Gossa
 
This file is part of Jocareso.

Jocareso is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Jocareso is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jocareso.  If not, see http://www.gnu.org/licenses/


Editions JOCATOP
Route de Saint Saturnin
84310 MORIERES
FRANCE

Christophe Gossa
christophe.gossa@jocatop.fr
===================================================================
*/


//  For Windows only (Visual C++ 2008)
//  Suppress all warnings about unsecured CRT functions
#ifdef _WIN32
#define _CRT_SECURE_NO_WARNINGS
#endif

#include "sysinfo-processor.h"
#include <stdio.h>
#include <string.h>

#ifndef NULL
#define NULL    0
#endif

typedef unsigned char           BYTE;


/*
#######################################################

Ce code est la propriete exclusive de Christophe Gossa.

Christophe Gossa autorise la societe Editions JOCATOP
a utiliser ce code sans limite.

Ce code fait partie du projet CPUST dont l'auteur,
Christophe Gossa, a stoppe le developpement au debut
de l'annee 2008.

Plus d'informations sur CPUST ici :
http://forum.hardware.fr/hfr/OverclockingCoolingTuning/CPU/monitoring-source-arrete-sujet_259979_1.htm

L'auteur, Christophe Gossa, reste le proprietaire
exclusif de ce code.

Date du code source utilise 2007-03-06

#######################################################

Information additionnelle

Le code a ete modifie pour pouvoir fonctionner au sein
du projet JOCARESO.

Christophe Gossa pour Editions JOCATOP 2009.

#######################################################

Information additionnelle #2

Je (Christophe Gossa) donne l'autorisation à la societe
Editions JOCATOP d'utiliser ce code sous les termes
de la licence GPL v2 afin de l'utiliser au sein du
projet JOCARESO.

Par conséquent, le code ci-présent est sous licence
GNU GPL v2. Se referer au terme du contrat de licence
pour plus d'information sur celle-ci.

Christophe Gossa, 2010-02-23

#######################################################
*/






    //CPU Features
    typedef struct {
        char        szManufacturer[32];             //Fabricant
        char        szManufacturerName[32];         //Nom du fabricant
        char        szDesignation[256];             //Designation
        //long        lGeneration;                    //>=5 => Pentium Class

        long        support_CPUID;
        long        support_MMX;                    //MMX gen-1
        long        support_MMX_EXTENSION;          //MMX gen-2 (AMD only !)
        long        support_SSE;                    //SSE gen-1
        long        support_SSE2;                   //SSE gen-2
        long        support_SSE3;                   //SSE gen-3
        long        support_SSSE3;                  //SSE gen-3 (extension)
        long        support_SSE4a;                  //SSE gen-4 (AMD only !)
        long        support_SSE4d1;                 //SSE gen-4 (Intel only with Penryn and more)
        long        support_SSE4d2;                 //SSE gen-4 (Intel only with Nehalem and more)
        long        support_SSE5;                   //SSE gen-5 (AMD only with Bulldozer)
        long        support_AVX;                    //New extension of Intel/AMD processor (Intel Sandy Bridge and AMD Bulldozer)
        long        support_3DNOW;                  //3DNow gen-1 (AMD only !)
        long        support_3DNOW_EXTENSION;        //3DNow gen-2 (AMD only !)
        long        support_x64;                    //x86-64
        long        support_HTT;                    //Support HyperThreadingTechnology
        long        support_VT;                     //Support VirtualisationTechnology
        long        support_NX;                     //Support No-execute
        long        support_PREFETCH;               //Support pour la fonction prefetch
        long        support_CMOV;                   //Support CMOV
        long        support_CMPXCHG8;               //Support CMPXCHG8
        long        support_RDTSC;                  //Support du RDTSC
        long        support_InvalidTSC;             //Le TSC n'est plus en relation avec la frequence/les cycles

        long        lCacheDataLevel1_KiB;
        long        lCacheCodeLevel1_KiB;
        long        lCacheDataLevel2_KiB;
        long        lCacheDataLevel3_KiB;


        long        lPackage_LocalApicID;
        long        lPackage_LogicalCoreCount;      //LogicalProcessorCount. AMD only ?
        long        lPackage_CmpLegacy;             //AMD only ?
        long        lPackage_NC;                    //Number of CPU cores-1. AMD only ?
        long        lPackage_CoresPerProcessor;
        long        lPackage_ThreadsPerCore;

        long        lIdentity_Family;
        long        lIdentity_Model;
        long        lIdentity_Stepping;
        long        lIdentity_Type;
        long        lIdentity_ExtFamily;
        long        lIdentity_ExtModel;

        long        lIntel_TLBC[16];
        long        lIntel_EnableEaxTLBC;
        long        lIntel_EnableEbxTLBC;
        long        lIntel_EnableEcxTLBC;
        long        lIntel_EnableEdxTLBC;
        long        lIntel_NeedToResolveL2L3_0x40;
        long        lIntel_NeedToResolveL2L3_0x49;


    } CPU_FEATURES;

    CPU_FEATURES
        _cpuFeatures;

    void SysInfo_Processor_Init ( void );


    long       SysInfo_ProcessorGetSpecsA ( long lSpecificationType, unsigned long * pdwReturnedSpecification, char * pszReturnedSpecification )
    {
        static  long            lInit = 0;
                char            *pszText;
                unsigned long   dwValue;
                long            lReturn;

        if (!lInit)
        {
            SysInfo_Processor_Init ( );
            lInit = 1;
        }

        pszText     = NULL;
        dwValue     = 0;

        if (_cpuFeatures.support_CPUID)
        {
            switch (lSpecificationType)
            {
                case SYSINFO_PROCESSOR_GET_MANUFACTURER:                    pszText = _cpuFeatures.szManufacturerName; break;
                case SYSINFO_PROCESSOR_GET_DESIGNATION:                     pszText = _cpuFeatures.szDesignation; break;

                case SYSINFO_PROCESSOR_GET_FEAT_MMX:                        dwValue = _cpuFeatures.support_MMX; break;
                case SYSINFO_PROCESSOR_GET_FEAT_MMX_EXTENSION:              dwValue = _cpuFeatures.support_MMX_EXTENSION; break;
                case SYSINFO_PROCESSOR_GET_FEAT_SSE:                        dwValue = _cpuFeatures.support_SSE; break;
                case SYSINFO_PROCESSOR_GET_FEAT_SSE2:                       dwValue = _cpuFeatures.support_SSE2; break;
                case SYSINFO_PROCESSOR_GET_FEAT_SSE3:                       dwValue = _cpuFeatures.support_SSE3; break;
                case SYSINFO_PROCESSOR_GET_FEAT_SSSE3:                      dwValue = _cpuFeatures.support_SSSE3; break;
                case SYSINFO_PROCESSOR_GET_FEAT_SSE4a:                      dwValue = _cpuFeatures.support_SSE4a; break;
                case SYSINFO_PROCESSOR_GET_FEAT_SSE4d1:                     dwValue = _cpuFeatures.support_SSE4d1; break;
                case SYSINFO_PROCESSOR_GET_FEAT_SSE4d2:                     dwValue = _cpuFeatures.support_SSE4d2; break;
                case SYSINFO_PROCESSOR_GET_FEAT_SSE5:                       dwValue = _cpuFeatures.support_SSE5; break;
                case SYSINFO_PROCESSOR_GET_FEAT_AVX:                        dwValue = _cpuFeatures.support_AVX; break;
                case SYSINFO_PROCESSOR_GET_FEAT_3DNOW:                      dwValue = _cpuFeatures.support_3DNOW; break;
                case SYSINFO_PROCESSOR_GET_FEAT_3DNOW_EXTENSION:            dwValue = _cpuFeatures.support_3DNOW_EXTENSION; break;
                case SYSINFO_PROCESSOR_GET_FEAT_X64:                        dwValue = _cpuFeatures.support_x64; break;
                case SYSINFO_PROCESSOR_GET_FEAT_HTT:                        dwValue = _cpuFeatures.support_HTT; break;
                case SYSINFO_PROCESSOR_GET_FEAT_VT:                         dwValue = _cpuFeatures.support_VT; break;
                case SYSINFO_PROCESSOR_GET_FEAT_NX:                         dwValue = _cpuFeatures.support_NX; break;
                case SYSINFO_PROCESSOR_GET_FEAT_PREFETCH:                   dwValue = _cpuFeatures.support_PREFETCH; break;
                case SYSINFO_PROCESSOR_GET_FEAT_CMOV:                       dwValue = _cpuFeatures.support_CMOV; break;
                case SYSINFO_PROCESSOR_GET_FEAT_CMPXCHG8:                   dwValue = _cpuFeatures.support_CMPXCHG8; break;
                case SYSINFO_PROCESSOR_GET_FEAT_RDTSC:                      dwValue = _cpuFeatures.support_RDTSC; break;
                case SYSINFO_PROCESSOR_GET_FEAT_INVALID_RDTSC:              dwValue = _cpuFeatures.support_InvalidTSC; break;

                case SYSINFO_PROCESSOR_GET_CACHE_L1_CODE:                   dwValue = _cpuFeatures.lCacheCodeLevel1_KiB; break;
                case SYSINFO_PROCESSOR_GET_CACHE_L1_DATA:                   dwValue = _cpuFeatures.lCacheDataLevel1_KiB; break;
                case SYSINFO_PROCESSOR_GET_CACHE_L2_DATA:                   dwValue = _cpuFeatures.lCacheDataLevel2_KiB; break;
                case SYSINFO_PROCESSOR_GET_CACHE_L3_DATA:                   dwValue = _cpuFeatures.lCacheDataLevel3_KiB; break;

                default: break;
            }
            lReturn = 0;
        }
        else lReturn = -1;

        if (pdwReturnedSpecification)
            *pdwReturnedSpecification = dwValue;

        if (pszReturnedSpecification)
        {
            if (pszText) strcpy ( pszReturnedSpecification, pszText );
            else pszReturnedSpecification[0] = 0;        
        }

        return lReturn;
    }


    //CPU Feature Entry	
    typedef struct {
        unsigned long   dwFnCPUID;
        char            szRegister[4];
        long            lBitStart;
        long            lBitCount;
        char            szName[64];
        long            *plOutToStore;
    } CPU_FEATURE_ENTRY;



    CPU_FEATURE_ENTRY
        _cpuFE_Global[] = { { 0x00000000, "ebx",  0, 32, "CPUVendor0",              (long *)(_cpuFeatures.szManufacturer) },
                            { 0x00000000, "edx",  0, 32, "CPUVendor1",              (long *)(_cpuFeatures.szManufacturer+4) },
                            { 0x00000000, "ecx",  0, 32, "CPUVendor2",              (long *)(_cpuFeatures.szManufacturer+8) },

                            { 0x00000001, "eax",  0,  4, "Stepping",                &_cpuFeatures.lIdentity_Stepping },         //OK Intel+AMD
                            { 0x00000001, "eax",  4,  4, "Model",                   &_cpuFeatures.lIdentity_Model },            //OK Intel+AMD
                            { 0x00000001, "eax",  8,  4, "Family",                  &_cpuFeatures.lIdentity_Family },           //OK Intel+AMD
                            { 0x00000001, "eax", 12,  2, "Type",                    &_cpuFeatures.lIdentity_Type },
                            { 0x00000001, "eax", 16,  4, "ExtModel",                &_cpuFeatures.lIdentity_ExtModel },         //OK Intel+AMD
                            { 0x00000001, "eax", 20,  8, "ExtFamily",               &_cpuFeatures.lIdentity_ExtFamily },        //OK Intel+AMD

                            { 0x00000001, "ebx", 24,  8, "LocalApicId",             &_cpuFeatures.lPackage_LocalApicID },       //OK Intel+AMD
                            { 0x00000001, "ebx", 16,  8, "LogicalProcessorCount",   &_cpuFeatures.lPackage_LogicalCoreCount },  //OK Intel+AMD

                            { 0x00000001, "ecx",  0,  1, "SSE3",                    &_cpuFeatures.support_SSE3 },               //OK Intel+AMD
                            { 0x00000001, "ecx", 19,  1, "SSE4.1",                  &_cpuFeatures.support_SSE4d1 },              //OK Intel+AMD
                            { 0x00000001, "ecx", 20,  1, "SSE4.2",                  &_cpuFeatures.support_SSE4d2 },              //OK Intel+AMD

                            { 0x00000001, "edx", 28,  1, "HTT",                     &_cpuFeatures.support_HTT },                //OK Intel+AMD
                            { 0x00000001, "edx", 26,  1, "SSE2",                    &_cpuFeatures.support_SSE2 },               //OK Intel+AMD
                            { 0x00000001, "edx", 25,  1, "SSE",                     &_cpuFeatures.support_SSE },                //OK Intel+AMD
                            { 0x00000001, "edx", 23,  1, "MMX",                     &_cpuFeatures.support_MMX },                //OK Intel+AMD
                            { 0x00000001, "edx", 15,  1, "CMOV",                    &_cpuFeatures.support_CMOV },               //OK Intel+AMD
                            { 0x00000001, "edx",  8,  1, "CMPXCHG8",                &_cpuFeatures.support_CMPXCHG8 },           //OK Intel+AMD
                            { 0x00000001, "edx",  4,  1, "RDTSC",                   &_cpuFeatures.support_RDTSC },              //OK Intel+AMD
                            { 0x80000001, "edx", 29,  1, "x64-64",                  &_cpuFeatures.support_x64 },                //OK Intel+AMD
                            { 0x80000001, "edx", 20,  1, "NX",                      &_cpuFeatures.support_NX },                 //OK Intel+AMD

                            { 0x80000002, "eax",  0, 32, "CPUName0",                (long *)(_cpuFeatures.szDesignation) },
                            { 0x80000002, "ebx",  0, 32, "CPUName1",                (long *)(_cpuFeatures.szDesignation+4) },
                            { 0x80000002, "ecx",  0, 32, "CPUName2",                (long *)(_cpuFeatures.szDesignation+8) },
                            { 0x80000002, "edx",  0, 32, "CPUName3",                (long *)(_cpuFeatures.szDesignation+12) },
                            { 0x80000003, "eax",  0, 32, "CPUName4",                (long *)(_cpuFeatures.szDesignation+16) },
                            { 0x80000003, "ebx",  0, 32, "CPUName5",                (long *)(_cpuFeatures.szDesignation+20) },
                            { 0x80000003, "ecx",  0, 32, "CPUName6",                (long *)(_cpuFeatures.szDesignation+24) },
                            { 0x80000003, "edx",  0, 32, "CPUName7",                (long *)(_cpuFeatures.szDesignation+28) },
                            { 0x80000004, "eax",  0, 32, "CPUName8",                (long *)(_cpuFeatures.szDesignation+32) },
                            { 0x80000004, "ebx",  0, 32, "CPUName9",                (long *)(_cpuFeatures.szDesignation+36) },
                            { 0x80000004, "ecx",  0, 32, "CPUNameA",                (long *)(_cpuFeatures.szDesignation+40) },
                            { 0x80000004, "edx",  0, 32, "CPUNameB",                (long *)(_cpuFeatures.szDesignation+44) },

                            { 0xFFFFFFFF } };



    CPU_FEATURE_ENTRY 
        _cpuFE_INTEL[] = {  { 0x00000001, "ecx",  5,  1, "VM",                      &_cpuFeatures.support_VT },
                            { 0x00000001, "ecx",  9,  1, "SSSE3",                   &_cpuFeatures.support_SSSE3 },
                            { 0x00000001, "edx", 30,  1, "IA64",                    &_cpuFeatures.support_x64 },
                            { 0x00000001, "ecx", 28,  1, "AVX",                     &_cpuFeatures.support_AVX },
                            { 0x00000002, "eax",  8,  8, "TLBC_eax",                &_cpuFeatures.lIntel_TLBC[1] },
                            { 0x00000002, "eax", 16,  8, "TLBC_eax",                &_cpuFeatures.lIntel_TLBC[2] },
                            { 0x00000002, "eax", 24,  8, "TLBC_eax",                &_cpuFeatures.lIntel_TLBC[3] },
                            { 0x00000002, "ebx",  0,  8, "TLBC_ebx",                &_cpuFeatures.lIntel_TLBC[4] },
                            { 0x00000002, "ebx",  8,  8, "TLBC_ebx",                &_cpuFeatures.lIntel_TLBC[5] },
                            { 0x00000002, "ebx", 16,  8, "TLBC_ebx",                &_cpuFeatures.lIntel_TLBC[6] },
                            { 0x00000002, "ebx", 24,  8, "TLBC_ebx",                &_cpuFeatures.lIntel_TLBC[7] },
                            { 0x00000002, "ecx",  0,  8, "TLBC_ecx",                &_cpuFeatures.lIntel_TLBC[8] },
                            { 0x00000002, "ecx",  8,  8, "TLBC_ecx",                &_cpuFeatures.lIntel_TLBC[9] },
                            { 0x00000002, "ecx", 16,  8, "TLBC_ecx",                &_cpuFeatures.lIntel_TLBC[10] },
                            { 0x00000002, "ecx", 24,  8, "TLBC_ecx",                &_cpuFeatures.lIntel_TLBC[11] },
                            { 0x00000002, "edx",  0,  8, "TLBC_edx",                &_cpuFeatures.lIntel_TLBC[12] },
                            { 0x00000002, "edx",  8,  8, "TLBC_edx",                &_cpuFeatures.lIntel_TLBC[13] },
                            { 0x00000002, "edx", 16,  8, "TLBC_edx",                &_cpuFeatures.lIntel_TLBC[14] },
                            { 0x00000002, "edx", 24,  8, "TLBC_edx",                &_cpuFeatures.lIntel_TLBC[15] },
                            { 0x00000002, "eax", 31,  1, "TLBC_eneax",              &_cpuFeatures.lIntel_EnableEaxTLBC },
                            { 0x00000002, "ebx", 31,  1, "TLBC_enebx",              &_cpuFeatures.lIntel_EnableEbxTLBC },
                            { 0x00000002, "ecx", 31,  1, "TLBC_enecx",              &_cpuFeatures.lIntel_EnableEcxTLBC },
                            { 0x00000002, "edx", 31,  1, "TLBC_enedx",              &_cpuFeatures.lIntel_EnableEdxTLBC },
                            { 0x80000006, "ecx", 16, 16, "L2DataKiB",               &_cpuFeatures.lCacheDataLevel2_KiB },
                            { 0x00000004, "eax", 26,  6, "CoresPerPhy",             &_cpuFeatures.lPackage_CoresPerProcessor },

                            { -1 } };


    CPU_FEATURE_ENTRY 
        _cpuFE_AMD[] = {    { 0x80000001, "ecx",  1,  1, "CmpLegacy",               &_cpuFeatures.lPackage_CmpLegacy },
                            { 0x80000001, "ecx",  6,  1, "SSE4a",                   &_cpuFeatures.support_SSE4a },
                            { 0x80000001, "ecx", 11,  1, "SSE5",                    &_cpuFeatures.support_SSE5 },
                            { 0x80000001, "edx", 31,  1, "3DNow!",                  &_cpuFeatures.support_3DNOW },
                            { 0x80000001, "edx", 30,  1, "3DNow!Ext",               &_cpuFeatures.support_3DNOW_EXTENSION },
                            { 0x80000001, "edx", 22,  1, "MmxExt",                  &_cpuFeatures.support_MMX_EXTENSION },
                            
                            { 0x80000005, "ecx", 24,  8, "L1DataKiB",               &_cpuFeatures.lCacheDataLevel1_KiB },
                            { 0x80000005, "edx", 24,  8, "L1CodeKiB",               &_cpuFeatures.lCacheCodeLevel1_KiB },
                            { 0x80000006, "ecx", 16, 16, "L2DataKiB",               &_cpuFeatures.lCacheDataLevel2_KiB },
                            { 0x80000006, "edx", 18, 14, "L3DataKiB",               &_cpuFeatures.lCacheDataLevel3_KiB },
                            { 0x80000008, "ecx",  0,  8, "NC",                      &_cpuFeatures.lPackage_NC },
                            { 0xFFFFFFFF } };

    //CPU Value Detection
    typedef struct {
        long    lDetectedValue;
        long    lWriteValue;
        long    *plObjectToWrite;
    } CPU_VALUE_DETECTION;


    CPU_VALUE_DETECTION
        _cpuVD_INTEL_TLBC[] = {     {   0x00,       0,      NULL },
                                    {   0x06,       8,      &_cpuFeatures.lCacheCodeLevel1_KiB  },
                                    {   0x08,      16,      &_cpuFeatures.lCacheCodeLevel1_KiB  }, 
                                    {   0x0A,       8,      &_cpuFeatures.lCacheDataLevel1_KiB  },
                                    {   0x0C,      16,      &_cpuFeatures.lCacheDataLevel1_KiB  }, 
                                    {   0x22,     512,      &_cpuFeatures.lCacheDataLevel3_KiB  },
                                    {   0x23,    1024,      &_cpuFeatures.lCacheDataLevel3_KiB  }, 
                                    {   0x25,    2048,      &_cpuFeatures.lCacheDataLevel3_KiB  },
                                    {   0x29,    4096,      &_cpuFeatures.lCacheDataLevel3_KiB  },
                                    {   0x2C,      32,      &_cpuFeatures.lCacheDataLevel1_KiB  },
                                    {   0x30,      32,      &_cpuFeatures.lCacheCodeLevel1_KiB  },
                                    {   0x39,     128,      &_cpuFeatures.lCacheDataLevel2_KiB  },
                                    {   0x3A,     192,      &_cpuFeatures.lCacheDataLevel2_KiB  },
                                    {   0x3B,     128,      &_cpuFeatures.lCacheDataLevel2_KiB  },
                                    {   0x3C,     256,      &_cpuFeatures.lCacheDataLevel2_KiB  },
                                    {   0x3D,     384,      &_cpuFeatures.lCacheDataLevel2_KiB  },
                                    {   0x3E,     512,      &_cpuFeatures.lCacheDataLevel2_KiB  },
                                    {   0x40,       1,      &_cpuFeatures.lIntel_NeedToResolveL2L3_0x40 },
                                    {   0x41,     128,      &_cpuFeatures.lCacheDataLevel2_KiB  },
                                    {   0x42,     256,      &_cpuFeatures.lCacheDataLevel2_KiB  },
                                    {   0x43,     512,      &_cpuFeatures.lCacheDataLevel2_KiB  },
                                    {   0x44,    1024,      &_cpuFeatures.lCacheDataLevel2_KiB  },
                                    {   0x45,    2048,      &_cpuFeatures.lCacheDataLevel2_KiB  },
                                    {   0x46,    4096,      &_cpuFeatures.lCacheDataLevel3_KiB  },
                                    {   0x47,    8192,      &_cpuFeatures.lCacheDataLevel3_KiB  },
                                    //{   0x49,    4096,      &ProcessorInfo.lCacheDataLevel3_KiB  }, //Intel Xeon processor MP, Family F, Model 6
                                    //{   0x49,    4096,      &ProcessorInfo.lCacheDataLevel2_KiB  },
                                    {   0x49,       1,      &_cpuFeatures.lIntel_NeedToResolveL2L3_0x49 },
                                    {   0x4A,    6144,      &_cpuFeatures.lCacheDataLevel3_KiB  },
                                    {   0x4B,    8192,      &_cpuFeatures.lCacheDataLevel3_KiB  },
                                    {   0x4C,   12288,      &_cpuFeatures.lCacheDataLevel3_KiB  },
                                    {   0x4D,   16384,      &_cpuFeatures.lCacheDataLevel3_KiB  },

                                    {   0x60,      16,      &_cpuFeatures.lCacheDataLevel1_KiB  },
                                    {   0x66,       8,      &_cpuFeatures.lCacheDataLevel1_KiB  },
                                    {   0x67,      16,      &_cpuFeatures.lCacheDataLevel1_KiB  },
                                    {   0x68,      32,      &_cpuFeatures.lCacheDataLevel1_KiB  },

                                    {   0x70,     -12,      &_cpuFeatures.lCacheCodeLevel1_KiB  }, //Trace cache / K-uops
                                    {   0x71,     -16,      &_cpuFeatures.lCacheCodeLevel1_KiB  }, //Trace cache / K-uops
                                    {   0x72,     -32,      &_cpuFeatures.lCacheCodeLevel1_KiB  }, //Trace cache / K-uops
                                    {   0x73,     -64,      &_cpuFeatures.lCacheCodeLevel1_KiB  }, //Trace cache / K-uops

                                    {   0x78,    1024,      &_cpuFeatures.lCacheDataLevel2_KiB  },
                                    {   0x79,     128,      &_cpuFeatures.lCacheDataLevel2_KiB  },
                                    {   0x7A,     256,      &_cpuFeatures.lCacheDataLevel2_KiB  },
                                    {   0x7B,     512,      &_cpuFeatures.lCacheDataLevel2_KiB  },
                                    {   0x7C,    1024,      &_cpuFeatures.lCacheDataLevel2_KiB  },
                                    {   0x7D,    2048,      &_cpuFeatures.lCacheDataLevel2_KiB  },
                                    {   0x7F,     512,      &_cpuFeatures.lCacheDataLevel2_KiB  },
                                    {   0x82,     256,      &_cpuFeatures.lCacheDataLevel2_KiB  },
                                    {   0x83,     512,      &_cpuFeatures.lCacheDataLevel2_KiB  },
                                    {   0x84,    1024,      &_cpuFeatures.lCacheDataLevel2_KiB  },
                                    {   0x85,    2048,      &_cpuFeatures.lCacheDataLevel2_KiB  },
                                    {   0x86,     512,      &_cpuFeatures.lCacheDataLevel2_KiB  },
                                    {   0x87,    1024,      &_cpuFeatures.lCacheDataLevel2_KiB  },

                                    {     -1 } };


void CPU_FeatureEntryGetList ( CPU_FEATURE_ENTRY * pFeatureEntry )
{
    //long    lExtendedCPUID;
    unsigned long   dwCPUID_Fn;
    long            lRegister;
    unsigned long   dwGR[4], dwMask, dwValue;
    long            i;
    unsigned long   dwMaxCPUID_std;
    unsigned long   dwMaxCPUID_ext;
    long            lEnableOrderCPUID;


    //Support de l'extended CPUID ?
#ifdef _WIN32
    __asm {
            push ebx
            pushfd
            mov eax, 0x00000000
            cpuid
            mov dwMaxCPUID_std, eax

            mov eax, 0x80000000
            cpuid
            mov dwMaxCPUID_ext, eax

            popfd
            pop ebx
    }
#else
    asm ( "pushal" );
    asm ( "pushfl" );
    asm ( "movl $0x00000000, %eax" );
    asm ( "cpuid" );
    asm ( "movl %%eax, %0" : "=m"(dwMaxCPUID_std) );
    asm ( "movl $0x80000000, %eax" );
    asm ( "cpuid" );
    asm ( "movl %%eax, %0" : "=m"(dwMaxCPUID_ext) );
    asm ( "popfl" );
    asm ( "popal" );
#endif

    dwCPUID_Fn = 0xFFFFFFFF;
    for (i=0;pFeatureEntry[i].dwFnCPUID!=0xFFFFFFFF;i++)
    {
        //On evite les appels multiples...
        if (dwCPUID_Fn!=pFeatureEntry[i].dwFnCPUID)
        {
            dwCPUID_Fn          = pFeatureEntry[i].dwFnCPUID;
            lEnableOrderCPUID   = 1;

            if (dwCPUID_Fn&0x80000000)
            {
                if (dwCPUID_Fn>dwMaxCPUID_ext)
                    lEnableOrderCPUID = 0;
            }
            else
            {
                if (dwCPUID_Fn>dwMaxCPUID_std)
                    lEnableOrderCPUID = 0;
            }

                    
            if (lEnableOrderCPUID)
            {
#ifdef _WIN32                
                __asm { 
                    push ebx
                    pushfd
                    mov eax, dwCPUID_Fn
                    xor ecx, ecx
                    cpuid
                    mov dword ptr dwGR,    eax
                    mov dword ptr dwGR+4,  ebx
                    mov dword ptr dwGR+8,  ecx
                    mov dword ptr dwGR+12, edx
                    popfd
                    pop ebx
                }
#else
                asm ( "pushal" );
                asm ( "pushfl" );
                asm ( "movl %0, %%eax" :: "m"(dwCPUID_Fn) );
                asm ( "xorl %ecx, %ecx" );
                asm ( "cpuid" );
                asm ( "movl %%eax, %0" : "=m"((dwGR[0])) );
                asm ( "movl %%ebx, %0" : "=m"((dwGR[1])) );
                asm ( "movl %%ecx, %0" : "=m"((dwGR[2])) );
                asm ( "movl %%edx, %0" : "=m"((dwGR[3])) );
                asm ( "popfl" );
                asm ( "popal" );
#endif
            }
            else
            {
                dwGR[0] = 0;
                dwGR[1] = 0;
                dwGR[2] = 0;
                dwGR[3] = 0;
            }
        }

        if (strcmp(pFeatureEntry[i].szRegister,"eax")==0) lRegister = 0;
        if (strcmp(pFeatureEntry[i].szRegister,"ebx")==0) lRegister = 1;
        if (strcmp(pFeatureEntry[i].szRegister,"ecx")==0) lRegister = 2;
        if (strcmp(pFeatureEntry[i].szRegister,"edx")==0) lRegister = 3;

        if (pFeatureEntry[i].lBitCount==32)
            dwMask = 0xFFFFFFFF;
        else
            dwMask =  ( 1 << pFeatureEntry[i].lBitCount ) - 1;

        dwValue = ( dwGR[lRegister] >> pFeatureEntry[i].lBitStart ) & dwMask;

        *pFeatureEntry[i].plOutToStore = dwValue;
    }
}


void SysInfo_ProcessorGetCodeName ( char * szProcessorCodeName );

void SysInfo_Processor_Init ( void )
{
    long    lCpuidOpcodeSupport;
    long    i, j;

    //Initialise les features
    memset ( &_cpuFeatures, 0, sizeof(CPU_FEATURES) );

    //Test de l'instruction CPUID par une methode douce...
#ifdef _WIN32
    __asm {
            push        ebx
            pushfd
            pop         eax
            mov         ecx,eax
            xor         eax,0x200000
            push        eax
            popfd
            pushfd
            pop         eax
            cmp         ecx,eax
            mov         lCpuidOpcodeSupport, 0
            je          _opsys_get_cpu_type_no_cpuid
            mov         lCpuidOpcodeSupport, 1
        _opsys_get_cpu_type_no_cpuid:
            pop         ebx
    }
#else
    asm ( "pushfl" );
    asm ( "pushal" );
    asm ( "pushfl" );
    asm ( "popl %eax" );
    asm ( "movl %eax, %ecx" );
    asm ( "xorl $0x200000, %eax" );
    asm ( "pushl %eax" );
    asm ( "popfl" );
    asm ( "pushfl" );
    asm ( "popl %eax" );
    asm ( "cmpl %eax, %ecx" );
    asm ( "movl $0, %0" : "=m"(lCpuidOpcodeSupport) );
    asm ( "je _opsys_get_cpu_type_no_cpuid" );
    asm ( "movl $1, %0" : "=m"(lCpuidOpcodeSupport) );
    asm ( "_opsys_get_cpu_type_no_cpuid:" );
    asm ( "popal" );
    asm ( "popfl" );
#endif
    if (lCpuidOpcodeSupport)
    {
        //Les caracteristiques generales...
        _cpuFeatures.support_CPUID = 1;
        CPU_FeatureEntryGetList ( _cpuFE_Global );

        //support_PREFETCH ?
        if (((_cpuFeatures.support_SSE)||(_cpuFeatures.support_SSE2)||(_cpuFeatures.support_3DNOW)))
            _cpuFeatures.support_PREFETCH = 1;

        //##################################################
        //
        //  S'agit-il d'un processeur AMD ?
        //
        if (!memcmp(_cpuFeatures.szManufacturer,"AuthenticAMD",12))
        {
            //Caracteristiques specifiques
            CPU_FeatureEntryGetList ( _cpuFE_AMD );

            //LogicalProcessorCount, CmpLegacy, HTT and NC
            switch ((_cpuFeatures.lPackage_CmpLegacy<<1)|(_cpuFeatures.support_HTT))
            {
                case 0:
                    _cpuFeatures.lPackage_LogicalCoreCount     = 1;
                    _cpuFeatures.lPackage_ThreadsPerCore       = 1;
                    _cpuFeatures.lPackage_CoresPerProcessor    = 1;
                    break;

                case 1:
                    _cpuFeatures.lPackage_CoresPerProcessor    = _cpuFeatures.lPackage_NC+1;
                    _cpuFeatures.lPackage_ThreadsPerCore       = _cpuFeatures.lPackage_LogicalCoreCount / _cpuFeatures.lPackage_CoresPerProcessor;
                    break;
                                        
                case 3:
                    _cpuFeatures.lPackage_CoresPerProcessor    = _cpuFeatures.lPackage_NC+1;
                    _cpuFeatures.lPackage_ThreadsPerCore       = _cpuFeatures.lPackage_LogicalCoreCount / _cpuFeatures.lPackage_CoresPerProcessor;
                    break;

                case 2:
                    _cpuFeatures.lPackage_CoresPerProcessor    = _cpuFeatures.lPackage_NC+1;
                    _cpuFeatures.lPackage_ThreadsPerCore       = _cpuFeatures.lPackage_LogicalCoreCount / _cpuFeatures.lPackage_CoresPerProcessor;
                    break;
            }

            if (_cpuFeatures.lPackage_ThreadsPerCore==1)
                _cpuFeatures.support_HTT = 0;
        }

        //##################################################
        //
        //  S'agit-il d'un processeur INTEL ?
        //
        if (!memcmp(_cpuFeatures.szManufacturer,"GenuineIntel",12))
        {
            //Caracteristiques specifiques
            CPU_FeatureEntryGetList ( _cpuFE_INTEL );

            //Gestion du TBL / Cache
            if (_cpuFeatures.lIntel_EnableEaxTLBC)
                for (i=1;i<4;i++) _cpuFeatures.lIntel_TLBC[i] = 0;

            if (_cpuFeatures.lIntel_EnableEbxTLBC)
                for (i=0;i<4;i++) _cpuFeatures.lIntel_TLBC[i+4] = 0;

            if (_cpuFeatures.lIntel_EnableEcxTLBC)
                for (i=0;i<4;i++) _cpuFeatures.lIntel_TLBC[i+8] = 0;

            if (_cpuFeatures.lIntel_EnableEdxTLBC)
                for (i=0;i<4;i++) _cpuFeatures.lIntel_TLBC[i+12] = 0;

            for (j=1;j<16;j++)
            {
                for (i=0;_cpuVD_INTEL_TLBC[i].lDetectedValue!=-1;i++)
                {
                    if (_cpuVD_INTEL_TLBC[i].lDetectedValue==_cpuFeatures.lIntel_TLBC[j])
                    {
                        if (_cpuVD_INTEL_TLBC[i].plObjectToWrite)
                        {
                            *_cpuVD_INTEL_TLBC[i].plObjectToWrite = _cpuVD_INTEL_TLBC[i].lWriteValue;
                        }
                    }
                }
            }

            _cpuFeatures.lPackage_CoresPerProcessor        += 1;

            if (_cpuFeatures.support_HTT)
            {
                if (_cpuFeatures.lPackage_LogicalCoreCount>_cpuFeatures.lPackage_CoresPerProcessor)
                {
                    _cpuFeatures.lPackage_ThreadsPerCore = _cpuFeatures.lPackage_LogicalCoreCount / _cpuFeatures.lPackage_CoresPerProcessor;
                }
                else
                {
                    _cpuFeatures.support_HTT                = 0;
                    _cpuFeatures.lPackage_ThreadsPerCore    = 1;
                }
            }
            else
            {
                _cpuFeatures.lPackage_ThreadsPerCore        = 1;
                _cpuFeatures.lPackage_CoresPerProcessor     = 1;
            }

            //Resolution de l'ambiguite L2/L3
            if (_cpuFeatures.lIntel_NeedToResolveL2L3_0x49)
            {
                if ((_cpuFeatures.lIdentity_Family==0x0F)&&(_cpuFeatures.lIdentity_Model==0x06))
                    _cpuFeatures.lCacheDataLevel3_KiB = 4096;
                else
                    _cpuFeatures.lCacheDataLevel2_KiB = 4096;
            }

            if (_cpuFeatures.lIntel_NeedToResolveL2L3_0x40)
                _cpuFeatures.lCacheDataLevel3_KiB = 0;
        }

        if (!strlen(_cpuFeatures.szDesignation))
            SysInfo_ProcessorGetCodeName ( _cpuFeatures.szDesignation );

        if (!strlen(_cpuFeatures.szDesignation))
        {
            sprintf ( 
                _cpuFeatures.szDesignation, 
                "TFMS=0x%01x%01x%01x%01x",
                _cpuFeatures.lIdentity_Type,
                _cpuFeatures.lIdentity_Family,
                _cpuFeatures.lIdentity_Model,
                _cpuFeatures.lIdentity_Stepping );
        }

        if      (!strcmp(_cpuFeatures.szManufacturer,"GenuineIntel"))   strcpy ( _cpuFeatures.szManufacturerName, "Intel" );
        else if (!strcmp(_cpuFeatures.szManufacturer,"AuthenticAMD"))   strcpy ( _cpuFeatures.szManufacturerName, "AMD" );
        else if (!strcmp(_cpuFeatures.szManufacturer,"AMDisbetter!"))   strcpy ( _cpuFeatures.szManufacturerName, "AMD" );
        else if (!strcmp(_cpuFeatures.szManufacturer,"CentaurHauls"))   strcpy ( _cpuFeatures.szManufacturerName, "Centaur" );
        else if (!strcmp(_cpuFeatures.szManufacturer,"CyrixInstead"))   strcpy ( _cpuFeatures.szManufacturerName, "Cyrix" );
        else if (!strcmp(_cpuFeatures.szManufacturer,"GenuineTMx86"))   strcpy ( _cpuFeatures.szManufacturerName, "Transmeta" );
        else if (!strcmp(_cpuFeatures.szManufacturer,"TransmetaCPU"))   strcpy ( _cpuFeatures.szManufacturerName, "Transmeta" );
        else if (!strcmp(_cpuFeatures.szManufacturer,"Geode by NSC"))   strcpy ( _cpuFeatures.szManufacturerName, "National Semiconductor" );
        else if (!strcmp(_cpuFeatures.szManufacturer,"NexGenDriven"))   strcpy ( _cpuFeatures.szManufacturerName, "NexGen" );
        else if (!strcmp(_cpuFeatures.szManufacturer,"RiseRiseRise"))   strcpy ( _cpuFeatures.szManufacturerName, "Rise" );
        else if (!strcmp(_cpuFeatures.szManufacturer,"SiS SiS SiS "))   strcpy ( _cpuFeatures.szManufacturerName, "SiS" );
        else if (!strcmp(_cpuFeatures.szManufacturer,"UMC UMC UMC "))   strcpy ( _cpuFeatures.szManufacturerName, "UMC" );
        else if (!strcmp(_cpuFeatures.szManufacturer,"VIA VIA VIA "))   strcpy ( _cpuFeatures.szManufacturerName, "VIA" );
        else                                                            strcpy ( _cpuFeatures.szManufacturerName, "Inconnu" );
    }

}

typedef struct {
    long    lExtFamily;
    long    lExtModel;
    long    lFamily;
    long    lModel;
    long    lType;
    long    lStepping;
    char    szDesignation[64];
    long    lCacheCodeLevel1_KiB;
    long    lCacheDataLevel1_KiB;
    long    lInvalidTSC;
} CPU_NAME_ENTRY;

    #define CMPE        (0x01000000)
    #define CMPA        (0x02000000)
    #define CMPB        (0x04000000)
    #define CMPN        (0x08000000)
    #define CMPNE       (CMPN|CMPE)
    #define CMPNA       (CMPN|CMPA)
    #define CMPNB       (CMPN|CMPB)
    #define CMPAE       (CMPE|CMPA)
    #define CMPBE       (CMPE|CMPB)
    #define CMPMASK     (0x7F000000)
    #define CMPUMASK    (0x80FFFFFF)

    CPU_NAME_ENTRY
    _cpuNE_VIA[] = {  //    ExFam       ExMod       Fam         Mod         Typ     Stepp       Designation                 CodeL1  DataL1    iTSC
                            //######################################################
                        {    0,          0,          6|CMPE,    13|CMPE,     0,       0,        "C7 (Esther)",                0,      0,      0,  },
                        {    0,          0,          6|CMPE,    10|CMPE,     0,       0,        "C7 (Esther)",                0,      0,      0,  },
                        {    0,          0,          6|CMPE,     9|CMPE,     0,       0,        "C3 (Nehemiah)",              0,      0,      0,  },
                        {    0,          0,          6|CMPE,     8|CMPE,     0,       0,        "C3 (Ezra-T)",                0,      0,      0,  },
                        {    0,          0,          6|CMPE,     7|CMPE,     0,       8|CMPAE,  "C3 (Ezra)",                  0,      0,      0,  },
                        {    0,          0,          6|CMPE,     7|CMPE,     0,       8|CMPB,   "C3 (Samuel 2)",              0,      0,      0,  },
                        {    0,          0,          6|CMPE,     6|CMPE,     0,       0,        "Cyrix III (Samuel)",         0,      0,      0,  },
                        {    0,          0,          6|CMPE,     5|CMPE,     0,       1|CMPE,   "6x86MX (M II)",              0,      0,      0,  },
                        {    0,          0,          6|CMPE,     5|CMPE,     0,       0,        "Cyrix III (Joshua)",         0,      0,      0,  },
                        {    0,          0,          6|CMPE,     0|CMPE,     0,       0|CMPE,   "6x86MX (M II)",              0,      0,      0,  },
                        {    0,          0,          6|CMPE,     0|CMPE,     0,       0,        "6x86MX (M II)",              0,      0,      0,  },
                        
                        {   -1 } };                                                                         

    CPU_NAME_ENTRY
    _cpuNE_AMD[] = {  //  ExFam       ExMod       Fam         Mod         Typ     Stepp       Designation                 CodeL1  DataL1    iTSC
                            //######################################################
                        {    0,          0,          6|CMPE,    10|CMPE,     0,       0,        "Athlon XP/MP (Barton/Thorton)", 0,   0,      0,  },
                        {    0,          0,          6|CMPE,     8|CMPE,     0,       0,        "Athlon XP/MP (Thoroughbred)",0,      0,      0,  },
                        {    0,          0,          6|CMPE,     7|CMPE,     0,       0,        "Duron (Morgan)",             0,      0,      0,  },     
                        {    0,          0,          6|CMPE,     6|CMPE,     0,       0,        "Athlon XP/MP (Palomino)",    0,      0,      0,  },     
                        {    0,          0,          6|CMPE,     4|CMPE,     0,       0,        "Athlon K7 (Thunderbird)",    0,      0,      0,  },
                        {    0,          0,          6|CMPE,     3|CMPE,     0,       0,        "Athlon K7 (Spitfire)",       0,      0,      0,  },
                        {    0,          0,          6|CMPE,     2|CMPE,     0,       0,        "Athlon K75",                 0,      0,      0,  },
                        {    0,          0,          6|CMPE,     1|CMPBE,    0,       0,        "Athlon K7",                  0,      0,      0,  },
                        {    0,          0,          5|CMPE,    13|CMPE,     0,       0,        "K6-2+/K6-III+",              0,      0,      0,  },
                        {    0,          0,          5|CMPE,    12|CMPE,     0,       0,        "K6-2+/K6-III+",              0,      0,      0,  },
                        {    0,          0,          5|CMPE,    10|CMPE,     0,       0,        "Geode LX",                   0,      0,      0,  },
                        {    0,          0,          5|CMPE,     9|CMPE,     0,       0,        "K6-III (Sharptooth)",        0,      0,      0,  },
                        {    0,          0,          5|CMPE,     8|CMPE,     0,       0,        "K6-2 (K6-3D)",               0,      0,      0,  },
                        {    0,          0,          5|CMPE,     7|CMPE,     0,       0,        "K6",                         0,      0,      0,  },
                        {    0,          0,          5|CMPE,     6|CMPE,     0,       0,        "K6",                         0,      0,      0,  },
                        {    0,          0,          5|CMPE,     3|CMPBE,    0,       0,        "K5",                         0,      0,      0,  },

                        {   -1 } };



    CPU_NAME_ENTRY
    _cpuNE_INTEL[] = {      
                        //  ExFam       ExMod       Fam         Mod         Typ     Stepp       Designation                 CodeL1  DataL1    iTSC
                            //######################################################
                            //Pentium serie-5
                        {    0,          0,          5|CMPE,     8|CMPE,     0,       0,        "Pentium MMX (P55)",         16,     16,      0,  },
                        {    0,          0,          5|CMPE,     7|CMPE,     0,       0,        "Pentium (P54C)",             8,      8,      0,  },
                        {    0,          0,          5|CMPE,     6|CMPE,     0,       0,        "Pentium OverDrive",          8,      8,      0,  },
                        {    0,          0,          5|CMPE,     4|CMPE,     1|CMPE,  0,        "Pentium MMX OverDrive",     16,     16,      0,  },
                        {    0,          0,          5|CMPE,     4|CMPE,     1|CMPNE, 0,        "Pentium MMX (P55C)",        16,     16,      0,  },
                        {    0,          0,          5|CMPE,     3|CMPE,     0,       0,        "Pentium OverDrive (P24T)",   8,      8,      0,  },
                        {    0,          0,          5|CMPE,     2|CMPE,     1|CMPE,  0,        "Pentium OverDrive",          8,      8,      0,  },
                        {    0,          0,          5|CMPE,     2|CMPE,     1|CMPNE, 0,        "Pentium (P54C)",             8,      8,      0,  },
                        {    0,          0,          5|CMPE,     1|CMPE,     1|CMPE,  0,        "Pentium OverDrive",          8,      8,      0,  },
                        {    0,          0,          5|CMPE,     1|CMPE,     1|CMPNE, 0,        "Pentium (P5)",               8,      8,      0,  },
                        {    0,          0,          5|CMPE,     0|CMPE,     1|CMPE,  0,        "Pentium OverDrive",          8,      8,      0,  },
                        {    0,          0,          5|CMPE,     0|CMPE,     1|CMPNE, 0,        "Pentium (P5)",               8,      8,      0,  },
                        {    0,          0,          5|CMPE,     0,          0,       0,        "Pentium",                    0,      0,      0,  },

                            //######################################################
                            //Itanium..
                        {    0,          0,          7|CMPE,     0,          0,       0,        "Itanium (Merced)",           0,      0,      0,  },

                            //######################################################
                        {    0,       0x1C|CMPE,     6|CMPE,     0,          0,       0,        "Atom",                       0,      0,      0,  },
                        {    0,       0x1A|CMPE,     6|CMPE,     0,          0,       0,        "Nehalem",                    0,      0,      0,  },
                        {    0,       0x17|CMPE,     6|CMPE,     0,          0,       0,        "Penryn",                     0,      0,      0,  },
                        {    0,       0x16|CMPE,     6|CMPE,     0,          0,       0,        "Penryn",                     0,      0,      0,  },

                        {    0,          0,          6|CMPE,    15|CMPE,     0,       0,        "Conroe/Merom",               0,      0,      0,  },
                        {    0,          0,          6|CMPE,    14|CMPE,     0,       0,        "Pentium M (Yonah)",          0,      0,      1,  },
                        {    0,          0,          6|CMPE,    13|CMPE,     0,       0,        "Pentium M (Dothan)",         0,      0,      1,  },
                        {    0,          0,          6|CMPE,    12|CMPE,     0,       0,        "Pentium M (Dothan)",         0,      0,      1,  },

                        {    0,          0,          6|CMPE,    11|CMPE,     0,       0,        "Pentium III-S (Tualatin)",   0,      0,      0,  },
                        {    0,          0,          6|CMPE,    10|CMPE,     0,       0,        "Pentium III Xeon (Cascades)",0,      0,      0,  },
                        {    0,          0,          6|CMPE,     9|CMPE,     0,       0,        "Pentium III (Banias)",       0,      0,      0,  },
                        {    0,          0,          6|CMPE,     8|CMPE,     0,      6|CMPE,    "Pentium III-EB (Coppermine)",0,      0,      0,  },
                        {    0,          0,          6|CMPE,     8|CMPE,     0,      3|CMPE,    "Pentium III-EB (Coppermine)",0,      0,      0,  },
                        {    0,          0,          6|CMPE,     8|CMPE,     0,      2|CMPE,    "Pentium III-E (Coppermine)", 0,      0,      0,  },
                        {    0,          0,          6|CMPE,     8|CMPE,     0,      1|CMPE,    "Pentium III-E (Coppermine)", 0,      0,      0,  },
                        {    0,          0,          6|CMPE,     8|CMPE,     0,       0,        "Pentium III (Coppermine)",   0,      0,      0,  },
                        {    0,          0,          6|CMPE,     7|CMPE,     0,       0,        "Pentium III (Katmai)",       0,      0,      0,  },
                        {    0,          0,          6|CMPE,     6|CMPE,     0,     10|CMPE,    "Mobile Celeron / Pentium II (Dixon)",0,0,    0,  },
                        {    0,          0,          6|CMPE,     6|CMPE,     0,     13|CMPE,    "Mobile Celeron / Pentium II (Dixon)",0,0,    0,  },
                        {    0,          0,          6|CMPE,     6|CMPE,     0,       0,        "Celeron / Pentium II (Dixon / Mendocino)",0,0,0, },
                        {    0,          0,          6|CMPE,     5|CMPE,     0,       0,        "Pentium II (Deschutes)",     0,      0,      0,  },
                        {    0,          0,          6|CMPE,     4|CMPE,     0,       0,        "OverDrive",                  0,      0,      0,  },
                        {    0,          0,          6|CMPE,     3|CMPE,     1|CMPE,  0,        "Pentium II OverDrive",       0,      0,      0,  },
                        {    0,          0,          6|CMPE,     3|CMPE,     1|CMPNE, 0,        "Pentium II (Klamath)",       0,      0,      0,  },
                        {    0,          0,          6|CMPE,     2|CMPBE,    0,       0,        "Pentium Pro (P6)",           0,      0,      0,  },

                        {    1|CMPE,     0,          15,         0,          0,       0,        "Itanium 2 (McKinley)",       0,      0,      0,  },
                        {    1|CMPA,     0,          15,         0,          0,       0,        "Itanium 2 (Madison)",        0,      0,      0,  },

                        {    0|CMPE,     0,          15,         6|CMPE,     0,       0,        "Pentium 4 (Northwood)",      0,      0,      0,  },
                        {    0|CMPE,     0,          15,         5|CMPE,     0,       0,        "Pentium 4 (Northwood)",      0,      0,      0,  },
                        {    0|CMPE,     0,          15,         4|CMPE,     0,       0,        "Pentium 4 (Northwood)",      0,      0,      0,  },
                        {    0|CMPE,     0,          15,         3|CMPE,     0,       0,        "Pentium 4 (Northwood)",      0,      0,      0,  },
                        {    0|CMPE,     0,          15,         2|CMPE,     0,       0,        "Pentium 4 (Northwood)",      0,      0,      0,  },
                        {    0|CMPE,     0,          15,         1|CMPE,     0,       0,        "Pentium 4 (Willamette)",     0,      0,      0,  },
                        {    0|CMPE,     0,          15,         0|CMPE,     0,       0,        "Pentium 4 (Willamette)",     0,      0,      0,  },

                        {   -1 } };

long SysInfo_ProcessorGetCodeName_CompareReturn ( long lValue1, long lValue2, long lFunction )
{
    long lFlags;

    // bit 0 = Equal
    // bit 1 = Above
    // bit 2 = Below
    lFlags = 0;

    if (lFunction&CMPE)
        if (lValue1==lValue2) lFlags |= 1;

    if (lFunction&CMPA)
        if (lValue1> lValue2) lFlags |= 2;

    if (lFunction&CMPB)
        if (lValue1< lValue2) lFlags |= 4;

    if (lFunction&CMPN)
        lFlags = ~lFlags;

    return (lFlags)?(1):(0);
    
}

typedef struct {
    long lCompareDataEntry_Offset;
    long * plCompareDataFeats;
} DATA_COMPARE_LIST;

//Recuperation de l'adresse d'un item de la structure
#define GETOFFSET(s,i) ((size_t)&((s*)NULL)->i)

DATA_COMPARE_LIST
DataCompareList[] = { { GETOFFSET(CPU_NAME_ENTRY,lExtFamily), &_cpuFeatures.lIdentity_ExtFamily },
                      { GETOFFSET(CPU_NAME_ENTRY,lExtModel), &_cpuFeatures.lIdentity_ExtModel },
                      { GETOFFSET(CPU_NAME_ENTRY,lFamily), &_cpuFeatures.lIdentity_Family },
                      { GETOFFSET(CPU_NAME_ENTRY,lModel), &_cpuFeatures.lIdentity_Model },
                      { GETOFFSET(CPU_NAME_ENTRY,lType), &_cpuFeatures.lIdentity_Type },
                      { GETOFFSET(CPU_NAME_ENTRY,lStepping), &_cpuFeatures.lIdentity_Stepping },
                      { -1 } };

void SysInfo_ProcessorGetCodeName ( char * szProcessorCodeName )
{
    CPU_NAME_ENTRY * pNameEntry;
    //char szTmp[32];
    long *plCompareDataEntry;
    long *plCompareDataFeats;

    long lCompareMax;
    long lCompareCount;
    long lIndex;

    if (!strcmp(_cpuFeatures.szManufacturer,"GenuineIntel"))
        pNameEntry=_cpuNE_INTEL;
    else if (!strcmp(_cpuFeatures.szManufacturer,"AuthenticAMD"))
        pNameEntry=_cpuNE_AMD;
    else if ( (!strcmp(_cpuFeatures.szManufacturer,"VIA VIA VIA ")) || 
              (!strcmp(_cpuFeatures.szManufacturer,"CyrixInstead")) )
        pNameEntry=_cpuNE_VIA;
    else
        return;
    

    for (;pNameEntry->lExtFamily!=-1;pNameEntry++)
    {
        lCompareMax     = 0;
        lCompareCount   = 0;

        for (lIndex=0;DataCompareList[lIndex].lCompareDataEntry_Offset!=-1;lIndex++)
        {
            plCompareDataEntry = (long *)(((BYTE *)pNameEntry)+DataCompareList[lIndex].lCompareDataEntry_Offset);
            plCompareDataFeats = DataCompareList[lIndex].plCompareDataFeats;

            if (*plCompareDataEntry)
            {
                lCompareMax   += 1;
                lCompareCount += SysInfo_ProcessorGetCodeName_CompareReturn ( 
                    (*plCompareDataEntry) & CMPUMASK, 
                    (*plCompareDataFeats), 
                    (*plCompareDataEntry) & CMPMASK );
            }
        }

        if (lCompareMax==lCompareCount)
        {
            sprintf ( szProcessorCodeName,
                "%s (TFMS=0x%01x%01x%01x%01x)",
                pNameEntry->szDesignation,
                _cpuFeatures.lIdentity_Type,
                _cpuFeatures.lIdentity_Family,
                _cpuFeatures.lIdentity_Model,
                _cpuFeatures.lIdentity_Stepping );

            _cpuFeatures.support_InvalidTSC = pNameEntry->lInvalidTSC;

            if (!_cpuFeatures.lCacheCodeLevel1_KiB)
                 _cpuFeatures.lCacheCodeLevel1_KiB = pNameEntry->lCacheCodeLevel1_KiB;

            if (!_cpuFeatures.lCacheDataLevel1_KiB)
                 _cpuFeatures.lCacheDataLevel1_KiB = pNameEntry->lCacheDataLevel1_KiB;

            return;
        }
    }
}
