/*
===================================================================
Copyright (C) 2009-2010 Editions JOCATOP / Christophe Gossa
 
This file is part of Jocareso.

Jocareso is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Jocareso is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jocareso.  If not, see http://www.gnu.org/licenses/


Editions JOCATOP
Route de Saint Saturnin
84310 MORIERES
FRANCE

Christophe Gossa
christophe.gossa@jocatop.fr
===================================================================
*/


#include "thread.h"
#include "sleep.h"

#ifdef _WIN32
/*
    Pour Windows uniquement (Visual C++ 2008)
    On supprime tous les warnings en utilisant les fonctions CRT non securisees.
*/
#define _CRT_SECURE_NO_WARNINGS
#endif


#ifdef _WIN32
    #include <windows.h>
    #include <windowsx.h>
    #include <winuser.h>
    #include <tchar.h>
    #include <stdio.h>
    #include <string.h>
    #include <conio.h>
    #include <direct.h>
    #include <stdlib.h>
    #include <commctrl.h>
    #include <fcntl.h>
    #include <io.h>
    #include <time.h>
    #include <math.h>
    #include <commdlg.h>
    #include <malloc.h>
    #include <winbase.h>

    long    ThreadCreate ( THREAD * pThread, long (*pThreadProc) ( void * pProcData ), long lCreateSuspended, void * pParams )
    {
        DWORD dwCreationFlags;
        
        memset ( pThread, 0, sizeof(THREAD) );
        pThread->pUserParams = pParams;

        //Creation et pause ?        
        dwCreationFlags = (lCreateSuspended)?(CREATE_SUSPENDED):(0);

        pThread->hThread = CreateThread ( 
                                NULL,                                   //No security attributes
                                0,                                      //Default Stack size
                                (LPTHREAD_START_ROUTINE)pThreadProc,    //Thread proc
                                (void *)pThread,                        //Input params
                                dwCreationFlags,                        //Creation flag
                                &(pThread->dwThreadID) );               //Used for compatibility with Win95/98/Me

        if (pThread->hThread) return 0;
        else                  return -1;
    }

    long    ThreadCreateEx ( THREAD * pThread, long (*pThreadProc) ( void * pProcData ), long lCreateSuspended, unsigned long dwStackSizeMB, void * pParams )
    {
        DWORD dwCreationFlags;
        
        memset ( pThread, 0, sizeof(THREAD) );
        pThread->pUserParams = pParams;
        
        //Creation et pause ?        
        dwCreationFlags = (lCreateSuspended)?(CREATE_SUSPENDED):(0);

        pThread->hThread = CreateThread ( 
                                NULL,                                   //No security attributes
                                dwStackSizeMB<<20,                      //Default Stack size
                                (LPTHREAD_START_ROUTINE)pThreadProc,    //Thread proc
                                (void *)pThread,                        //Input params
                                dwCreationFlags,                        //Creation flag
                                &(pThread->dwThreadID) );               //Used for compatibility with Win95/98/Me

        if (pThread->hThread) return 0;
        else                  return -1;
    }


    long    ThreadDelete ( THREAD * pThread, long lMaxDelayBeforeKill_ms )
    {
        long    lCount;
        DWORD   dwExitCode;
        DWORD   dwFlags;

        if (!pThread) return -1;
        if (!pThread->hThread) return -1;

        //Destruction de la thread
        lCount = lMaxDelayBeforeKill_ms;
        pThread->bThreadExit = 1;
        do {
            SleepSet ( 10 );
            GetExitCodeThread ( pThread->hThread, &dwExitCode );
            lCount-=10;
        } while ((dwExitCode==STILL_ACTIVE)&&(lCount>0));
        if (dwExitCode==STILL_ACTIVE) TerminateThread ( pThread->hThread, 0 );

        //Fermeture de la Thread
        dwFlags = 0;
        if (GetHandleInformation ( (HANDLE)pThread->hThread, &dwFlags ))
        {
            if (!(dwFlags&HANDLE_FLAG_PROTECT_FROM_CLOSE))
                CloseHandle ( (HANDLE)pThread->hThread );
        }

        memset ( pThread, 0, sizeof(THREAD) );

        return 0;
    }

    #define MS_VC_EXCEPTION 0x406d1388

    typedef struct tagTHREADNAME_INFO
    {
        DWORD dwType;        // must be 0x1000
        LPCSTR szName;       // pointer to name (in same addr space)
        DWORD dwThreadID;    // thread ID (-1 caller thread)
        DWORD dwFlags;       // reserved for future use, most be zero
    } THREADNAME_INFO;

    long    ThreadSetName ( THREAD * pThread, char * pszShortName )
    {
        THREADNAME_INFO info;

        strncpy ( pThread->szName, pszShortName, 9 );
        pThread->szName[9] = 0;

        info.dwType         = 0x1000;
        info.szName         = pThread->szName;
        info.dwThreadID     = pThread->dwThreadID;
        info.dwFlags        = 0;

/*
        __try
        {
            RaiseException(MS_VC_EXCEPTION, 0, sizeof(info) / sizeof(DWORD),(DWORD *)&info);
        }
        __except (EXCEPTION_CONTINUE_EXECUTION)
        {

        }
*/
        return 0;
    }

    long    ThreadGetProcParams ( void * pProcData, void ** pParams )
    {
        THREAD * pThread;
        pThread = (THREAD *)pProcData;
        *pParams = pThread->pUserParams;
        return 0;
    }

    long    ThreadSetPriority ( THREAD * pThread, DWORD dwPriority )
    {
        if (!pThread) return -1;
        if (!pThread->hThread) return -1;

        if (SetThreadPriority ( pThread->hThread, dwPriority )) return 0;
        else                                              return -1;
    }

    long    ThreadSetAffinity ( THREAD * pThread, DWORD dwAffinityMask )
    {
        if (!pThread) return -1;
        if (!pThread->hThread) return -1;
        if (!dwAffinityMask) return -1;

        if (SetThreadAffinityMask ( pThread->hThread, dwAffinityMask )) return 0;
        else return -1;
    }

    long    ThreadSuspend ( THREAD * pThread )
    {
        if (!pThread) return -1;
        if (!pThread->hThread) return -1;

        SuspendThread ( pThread->hThread );
        return 0;
    }
        
    long    ThreadResume ( THREAD * pThread )
    {
        if (!pThread) return -1;
        if (!pThread->hThread) return -1;

        ResumeThread ( pThread->hThread );
        return 0;
    }

    long    ThreadDoWhileAndWaitExit ( void * pProcData )
    {
        THREAD * pThread;
        pThread = (THREAD *)pProcData;
        if (pThread->bThreadExit) return 0;
        else                      return 1;
    }

#else
    #include <pthread.h>
    #include <signal.h>
    #include <memory.h>
    #include <stdio.h>
    #include <stdlib.h>
    
    #define THREAD_STILL_ACTIVE 0x00000103
                     
    static void * _POSIX_RunThread ( void * pData )
    {
        THREAD * pThread;
        long lReturn;
        
        pThread = (THREAD *)pData;
        lReturn = pThread->pThreadProc ( pData );
        
        pthread_mutex_lock( &(pThread->cs_mutex) );
        pThread->dwExitCode = 0;
        pthread_mutex_unlock( &(pThread->cs_mutex) );
        
        pthread_exit ( (void *)0 );
        
        return ((void *)0);
    }
    long    ThreadCreate ( THREAD * pThread, long (*pThreadProc) ( void * pProcData ), long lCreateSuspended, void * pInputParams )
    {
        int iReturn;
        struct sched_param param;
        
        memset ( pThread, 0, sizeof(THREAD) );
        pThread->pUserParams = pInputParams;
        pThread->pThreadProc = pThreadProc;
        pThread->dwExitCode  = THREAD_STILL_ACTIVE;
        
        pthread_mutex_init ( &(pThread->cs_mutex), NULL );
        
        //Set the thread attribute
        pthread_attr_init ( 
            &(pThread->thread_attr) );
        
        pthread_attr_setdetachstate ( 
            &(pThread->thread_attr),
            PTHREAD_CREATE_DETACHED ); //JOINABLE );
/*-----------------------------------------------------
        param.sched_priority = 0;
        pthread_setschedparam ( 
            &(pThread->thread_id),
            0, //Policy (0 for normal thread)
            &param );
-----------------------------------------------------*/
        iReturn = pthread_create ( 
            &(pThread->thread_id),
            &(pThread->thread_attr),
            _POSIX_RunThread,
            (void *)pThread );
        
        return 0;
    }

    long    ThreadCreateEx ( THREAD * pThread, long (*pThreadProc) ( void * pProcData ), long lCreateSuspended, unsigned long dwStackSizeMB, void * pParams )
    {
        return ThreadCreate ( 
            pThread, 
            pThreadProc,
            lCreateSuspended,
            pParams );
    }
    
    long    ThreadDelete ( THREAD * pThread, long lMaxDelayBeforeKill_ms )
    {
        long    lCount;
        DWORD   dwExitCode;
        DWORD   dwFlags;

        if (!pThread) return -1;

        //Destruction de la thread
        lCount = lMaxDelayBeforeKill_ms;
        
        //Send exit order to the Thread
        pthread_mutex_lock( &(pThread->cs_mutex) );
        pThread->bThreadExit = THREAD_STILL_ACTIVE;
        pthread_mutex_unlock( &(pThread->cs_mutex) );
        
        do {
            SleepSet ( 10 );
            
            //Get return ExitCode (like Win32 GetExitCodeThread)
            pthread_mutex_lock( &(pThread->cs_mutex) );
            dwExitCode = pThread->dwExitCode;
            pthread_mutex_unlock( &(pThread->cs_mutex) );
            
            lCount-=10;
        } while ((dwExitCode==THREAD_STILL_ACTIVE)&&(lCount>0));
        
        //Force Thread to be closed...
        if (dwExitCode==THREAD_STILL_ACTIVE)
        {
/*-----------------------------------------------------
#ifdef PTHREAD_CANCEL_ASYNCHRONOUS
            pthread_cancel ( pThread->thread_id );
#else            
            pthread_kill ( pThread->thread_id, SIGKILL );
#endif
-----------------------------------------------------*/
            pthread_kill ( pThread->thread_id, SIGKILL );
        }

        //The Thread is closed...
        memset ( pThread, 0, sizeof(THREAD) );

        return 0;
    }
    
    long    ThreadSetName ( THREAD * pThread, char * pszShortName )
    {
        return 0;
    }    
    
    long    ThreadGetProcParams ( void * pProcData, void ** pParams )
    {
        THREAD * pThread;
        pThread = (THREAD *)pProcData;
        *pParams = pThread->pUserParams;
        return 0;
    }
    
    long    ThreadSetPriority ( THREAD * pThread, DWORD dwPriority )
    {
        if (!pThread) return -1;


        return 0;
    }

    long    ThreadSetAffinity ( THREAD * pThread, DWORD dwAffinityMask )
    {
        if (!pThread) return -1;
        if (!dwAffinityMask) return -1;


        return 0;
    }

    long    ThreadSuspend ( THREAD * pThread )
    {
        if (!pThread) return -1;


        return 0;
    }
        
    long    ThreadResume ( THREAD * pThread )
    {
        if (!pThread) return -1;


        return 0;
    }

    long    ThreadDoWhileAndWaitExit ( void * pProcData )
    {
        THREAD * pThread;
        pThread = (THREAD *)pProcData;
        if (pThread->bThreadExit) return 0;
        else                      return 1;
    }
    
    
#endif
