/*
===================================================================
Copyright (C) 2009-2010 Editions JOCATOP / Christophe Gossa
 
This file is part of Jocareso.

Jocareso is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Jocareso is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jocareso.  If not, see http://www.gnu.org/licenses/


Editions JOCATOP
Route de Saint Saturnin
84310 MORIERES
FRANCE

Christophe Gossa
christophe.gossa@jocatop.fr
===================================================================
*/


#ifndef __THREAD__
#define __THREAD__

    #ifdef _WIN32
        #include <windows.h>
        #include <windowsx.h>
    #else
        #include <pthread.h>
        #include <sys/types.h>
        #ifndef TD_BOOL
            typedef long BOOL;
            #define TD_BOOL
        #endif
    #endif
    
    #include "types.h"


    //Thread
    typedef struct {
        char                szName[16];
        DWORD               dwThreadID;
        BOOL                bThreadExit;
        DWORD               dwExitCode;
        void                *pUserParams;
        
        long                (*pThreadProc) ( void * pProcData );
        
    #ifdef _WIN32
        HANDLE              hThread;
    #else
        pthread_t           thread_id;
        pthread_attr_t      thread_attr;
        pthread_mutex_t     cs_mutex;
    #endif

    } THREAD;
    
    long    ThreadCreate                ( THREAD * pThread, long (*pThreadProc) ( void * pProcData ), long lCreateSuspended, void * pInputParams );
    long    ThreadCreateEx              ( THREAD * pThread, long (*pThreadProc) ( void * pProcData ), long lCreateSuspended, unsigned long dwStackSizeMB, void * pParams );
    long    ThreadDelete                ( THREAD * pThread, long lMaxDelayBeforeKill_ms );
    long    ThreadGetProcParams         ( void * pProcData, void ** pParams );
    long    ThreadSetPriority           ( THREAD * pThread, DWORD dwPriority );
    long    ThreadSetName               ( THREAD * pThread, char * pszShortName );
    long    ThreadSuspend               ( THREAD * pThread );
    long    ThreadResume                ( THREAD * pThread );
    long    ThreadDoWhileAndWaitExit    ( void * pProcData );
    long    ThreadSetAffinity           ( THREAD * pThread, DWORD dwAffinityMask );
    
    #define THREAD_PRIORITYMODE_REALTIME    0
    #define THREAD_PRIORITYMODE_HIGHEST     1
    #define THREAD_PRIORITYMODE_NORMAL      15
    #define THREAD_PRIORITYMODE_LOWEST      30
    #define THREAD_PRIORITYMODE_IDLE        31
    
        
#endif        

