/*
===================================================================
Copyright (C) 2009-2010 Editions JOCATOP / Christophe Gossa
 
This file is part of Jocareso.

Jocareso is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Jocareso is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jocareso.  If not, see http://www.gnu.org/licenses/


Editions JOCATOP
Route de Saint Saturnin
84310 MORIERES
FRANCE

Christophe Gossa
christophe.gossa@jocatop.fr
===================================================================
*/

#include "tick.h"


#ifdef _WIN32
    #include <windows.h>
    #include <windowsx.h>
    #include <winuser.h>
    #include <tchar.h>
    #include <stdio.h>
    #include <string.h>
    #include <conio.h>
    #include <direct.h>
    #include <stdlib.h>
    #include <commctrl.h>
    #include <fcntl.h>
    #include <io.h>
    #include <time.h>
    #include <math.h>
    #include <commdlg.h>
    #include <malloc.h>
    #include <winbase.h>

    long    TickGetCount                                       ( unsigned long * pdwTick )
    {
        *pdwTick = GetTickCount ( );
        return 0;
    }

    unsigned long TickGetCount2                                ( void )
    {
        return GetTickCount ( );
    }
    
    unsigned __int64 TickGetRDTSC2 ( void )
    {
        unsigned long dwRDTSCValue[2];
        __asm {
            push eax
            push edx
            rdtsc
            mov dwRDTSCValue, eax
            mov dwRDTSCValue+4, edx
            pop edx
            pop eax
            
        }
        
        //Generally, the return value is edx:eax (but not always with all compilers and some optimizations)
        return *((unsigned __int64 *)dwRDTSCValue);
    }
    
    void TickGetRDTSC ( unsigned long * pdwLowRDTSC, unsigned long * pdwHighRDTSC )
    {
        __asm {
            push eax
            push edx
            push ecx
            rdtsc
            mov ecx, pdwLowRDTSC
            mov [ecx], eax
            mov ecx, pdwHighRDTSC
            mov [ecx], edx
            pop ecx
            pop edx
            pop eax
        }
    }

#else
    #include <sys/time.h>
    #include <pthread.h>
    #include <sys/types.h>
    
    
    long    TickGetCount                                       ( unsigned long * pdwTick )
    {
        struct timeval tv;
        
        if (gettimeofday(&tv, NULL) != 0)
            *pdwTick = 0;
        else
            *pdwTick = ( (tv.tv_sec * 1000) + (tv.tv_usec / 1000) );
            
        return 0;
    }
    
    unsigned long TickGetCount2                                ( void )
    {
        struct timeval tv;
        
        if (gettimeofday(&tv,NULL)!=0)
            return 0;

        return ( (tv.tv_sec * 1000) + (tv.tv_usec / 1000) );
    }

    
    unsigned long long TickGetRDTSC2 ( void )
    {
        unsigned long dwRDTSCValueLow;
        unsigned long dwRDTSCValueHigh;
        
        asm ( "rdtsc" );
        asm ( "movl %%eax, %0" : "=m"(dwRDTSCValueLow) );
        asm ( "movl %%edx, %0" : "=m"(dwRDTSCValueHigh) );
        
        //Generally, the return value is edx:eax (but not always with all compilers and some optimizations)
        return ((unsigned long long)dwRDTSCValueLow)|(((unsigned long long)dwRDTSCValueHigh)<<32);
    }
    
    void TickGetRDTSC ( unsigned long * pdwLowRDTSC, unsigned long * pdwHighRDTSC )
    {
        unsigned long long qwRDTSCValue;
        
        qwRDTSCValue = TickGetRDTSC2 ( ); //Not very good for precision...
        
        if (pdwLowRDTSC)
            *pdwLowRDTSC = (unsigned long)(qwRDTSCValue&0xFFFFFFFF);
        
        if (pdwHighRDTSC)
            *pdwHighRDTSC = (unsigned long)((qwRDTSCValue>>32)&0xFFFFFFFF);
    }
    
    
#endif
