/*
===================================================================
Copyright (C) 2009-2010 Editions JOCATOP / Christophe Gossa
 
This file is part of Jocareso.

Jocareso is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Jocareso is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Jocareso.  If not, see http://www.gnu.org/licenses/


Editions JOCATOP
Route de Saint Saturnin
84310 MORIERES
FRANCE

Christophe Gossa
christophe.gossa@jocatop.fr
===================================================================
*/

/*
2010-02-25 16:54 Christophe Gossa
GPL v2 update : Partial changing of source code with a new one under GPL.
*/


#ifdef _WIN32

    #define _WIN32_WINNT 0x0500

    #if defined(_MSC_VER) && _MSC_VER >= 1400
    #define _CRT_SECURE_NO_DEPRECATE
    #endif

    #include <windows.h>
    #include <windowsx.h>
    #include <commctrl.h>
    #include <objbase.h>
    #include <shlobj.h>
    #include <stdlib.h>
    #include <stdio.h>
    //#include <wtsApi32.h>
    #include <tchar.h>

    #define OS_VERSION_WIN98    1
    #define OS_VERSION_WINNT    2
    #define OS_VERSION_WIN2K    3

    typedef struct {
        DWORD dwOSVersionInfoSize;
        DWORD dwMajorVersion;
        DWORD dwMinorVersion;
        DWORD dwBuildNumber;
        DWORD dwPlatformId;
        CHAR   szCSDVersion[ 128 ];     // Maintenance string for PSS usage
        WORD wServicePackMajor;
        WORD wServicePackMinor;
        WORD wSuiteMask;
        BYTE bProductType;
        BYTE bReserved;
    } OSVERSIONINFOEX_2;


    SC_HANDLE               _Win32_schManager;
    SC_HANDLE               _Win32_schService;
    char                    _Win32_szServiceName[128]={0};
    char                    _Win32_szServiceDesc[128]={0};
    long                    _Win32_lVersionOS = -1;

    SERVICE_STATUS          _Win32_gSvcStatus; 
    SERVICE_STATUS_HANDLE   _Win32_gSvcStatusHandle; 
    HANDLE                  _Win32_ghSvcStopEvent = NULL;

    void                    (*_Win32_UserToDoMain) ( void );


    long    _Win32Service_GetVersionOS ( long * plVersionOS );
    long    _Win32Service_ServiceManagerConnect ( void );
    long    _Win32Service_ServiceManagerDysconnect ( void );
    long    _Win32Service_ServiceAppOpen ( void );
    long    _Win32Service_ServiceAppClose ( void );
    long    _Win32Service_ServiceGetState ( void );
    long    _Win32Service_ServiceInstall ( char * pszBinaryPath );
    long    _Win32Service_ServiceUninstall ( void );
    long    _Win32Service_ServiceSetState ( long lStateNeed );
    long    _Win32Service_ServiceStart ( void );
    long    _Win32Service_ServiceStop ( void );
    long    _Win32Service_SetNameDescProc ( char * pszServiceName, char * pszServiceDesc, void (*pServiceProc) ( void ) );
    char *  _Win32Service_GetTextError ( DWORD dwError );                                 

    long    _Win32Service_ServiceMain ( DWORD dwArgc, LPTSTR *lpszArgv );
    long    _Win32Service_ServiceGetStopEvent ( void );
    long    _Win32Service_ServiceReleaseStopEvent ( void );
    long    _Win32Service_ServiceInitRun ( DWORD dwArgc, LPTSTR *lpszArgv);

    long    _Win32Service_ReportSvcStatus ( DWORD dwCurrentState, DWORD dwWin32ExitCode, DWORD dwWaitHint );
    void WINAPI _Win32Service_SvcCtrlHandler( DWORD dwCtrl );


    long    _Win32Service_DoApplicationInstall ( char * pszFullPathBin );
    long    _Win32Service_DoApplicationUninstall ( void );
    long    _Win32Service_DoApplicationStart ( void );
    long    _Win32Service_DoApplicationStop ( void );

    void COM_Printf ( char * fmt, ... );


    long _Win32Service_ServiceMain ( DWORD dwArgc, LPTSTR *lpszArgv )
    {
        //Register the handler function for the service
        _Win32_gSvcStatusHandle = 
            RegisterServiceCtrlHandlerA ( 
                _Win32_szServiceName, 
                _Win32Service_SvcCtrlHandler );

        if(!_Win32_gSvcStatusHandle)
        { 
            COM_Printf ( "%s - Can't registering service handler.", _Win32Service_GetTextError(GetLastError()) );
            return -1;
        } 

        //These SERVICE_STATUS members remain as set here
        _Win32_gSvcStatus.dwServiceType = SERVICE_WIN32_OWN_PROCESS; 
        _Win32_gSvcStatus.dwServiceSpecificExitCode = 0;    

        //Report initial status to the SCM
        _Win32Service_ReportSvcStatus( SERVICE_START_PENDING, NO_ERROR, 3000 );

        //Perform service-specific initialization and work.
        _Win32Service_ServiceInitRun( dwArgc, lpszArgv );

        return 0;
    }

    long _Win32Service_ServiceInitRun ( DWORD dwArgc, LPTSTR *lpszArgv)
    {
        //Create an event. The control handler function, SvcCtrlHandler,
        //signals this event when it receives the stop control code.
        _Win32_ghSvcStopEvent = CreateEvent(
                         NULL,    // default security attributes
                         TRUE,    // manual reset event
                         FALSE,   // not signaled
                         NULL);   // no name

        if (_Win32_ghSvcStopEvent==NULL)
        {
            _Win32Service_ReportSvcStatus( SERVICE_STOPPED, NO_ERROR, 0 );
            return -1;
        }

        //Report running status when initialization is complete.
        _Win32Service_ReportSvcStatus( SERVICE_RUNNING, NO_ERROR, 0 );

        // TO_DO: Perform work until service stops.
        //main_getparams ( dwArgc, (char **)lpszArgv, NULL );
        if (_Win32_UserToDoMain)
            _Win32_UserToDoMain ( );
        
        _Win32Service_ServiceReleaseStopEvent ( );
    
        return 0;
    }


    long _Win32Service_ServiceGetStopEvent ( void )
    {
        DWORD dwState;
    
        dwState = WaitForSingleObject ( _Win32_ghSvcStopEvent, 0 );
    
        switch (dwState)
        {
            case WAIT_OBJECT_0: return 1;
            default:            return 0;
        }
    }

    long _Win32Service_ServiceReleaseStopEvent ( void )
    {   
        _Win32Service_ReportSvcStatus( SERVICE_STOPPED, NO_ERROR, 0 );        
        return 0;
    }

    long _Win32Service_ReportSvcStatus ( DWORD dwCurrentState,
                       DWORD dwWin32ExitCode,
                       DWORD dwWaitHint )
    {
        static DWORD dwCheckPoint = 1;

        // Fill in the SERVICE_STATUS structure.
        _Win32_gSvcStatus.dwCurrentState = dwCurrentState;
        _Win32_gSvcStatus.dwWin32ExitCode = dwWin32ExitCode;
        _Win32_gSvcStatus.dwWaitHint = dwWaitHint;

        if (dwCurrentState == SERVICE_START_PENDING)
            _Win32_gSvcStatus.dwControlsAccepted = 0;
        else _Win32_gSvcStatus.dwControlsAccepted = SERVICE_ACCEPT_STOP;

        if ( (dwCurrentState == SERVICE_RUNNING) ||
               (dwCurrentState == SERVICE_STOPPED) )
            _Win32_gSvcStatus.dwCheckPoint = 0;
        else _Win32_gSvcStatus.dwCheckPoint = dwCheckPoint++;

        // Report the status of the service to the SCM.
        SetServiceStatus( _Win32_gSvcStatusHandle, &_Win32_gSvcStatus );

        return 0;
    }


    //
    // Purpose: 
    //   Called by SCM whenever a control code is sent to the service
    //   using the ControlService function.
    //
    // Parameters:
    //   dwCtrl - control code
    // 
    // Return value:
    //   None
    //
    void WINAPI _Win32Service_SvcCtrlHandler( DWORD dwCtrl )
    {
       //Handle the requested control code. 
       switch(dwCtrl) 
       {  
          case SERVICE_CONTROL_STOP: 
             _Win32Service_ReportSvcStatus(SERVICE_STOP_PENDING, NO_ERROR, 0);

             // Signal the service to stop.

             SetEvent(_Win32_ghSvcStopEvent);
             return;
 
          case SERVICE_CONTROL_INTERROGATE: 
             // Fall through to send current status.
             break; 
 
          default: 
             break;
       } 

       _Win32Service_ReportSvcStatus(_Win32_gSvcStatus.dwCurrentState, NO_ERROR, 0);
    }

    

    long    _Win32Service_DoApplicationInstall ( char * pszFullPathBin )
    {
        if (_Win32Service_GetVersionOS ( &_Win32_lVersionOS ))
            return -1;

        switch (_Win32_lVersionOS)
        {
            case 0:
                break;

            case OS_VERSION_WIN98:
                break;

            case OS_VERSION_WINNT:
                break;

            case OS_VERSION_WIN2K:
                if (!_Win32Service_ServiceManagerConnect ( ))
                {
                    _Win32Service_ServiceInstall ( pszFullPathBin );
                    _Win32Service_ServiceSetState ( 3 );
                    _Win32Service_ServiceManagerDysconnect ( );
                    return 0;
                }
                break;

        }

        return -1;
    }


    long    _Win32Service_DoApplicationUninstall ( void )
    {
        if (_Win32Service_GetVersionOS ( &_Win32_lVersionOS ))
            return -1;

        switch (_Win32_lVersionOS)
        {
            case 0:
                break;

            case OS_VERSION_WIN98:
                break;

            case OS_VERSION_WINNT:
                break;

            case OS_VERSION_WIN2K:
                if (!_Win32Service_ServiceManagerConnect ( ))
                {
                    _Win32Service_ServiceSetState ( 4 );
                    _Win32Service_ServiceUninstall ( );
                    _Win32Service_ServiceManagerDysconnect ( );
                    return 0;
                }
                break;

        }

        return -1;
    }
    

    long    _Win32Service_DoApplicationStart ( void )
    {
        if (_Win32Service_GetVersionOS ( &_Win32_lVersionOS ))
            return -1;

        switch (_Win32_lVersionOS)
        {
            case 0:
                break;

            case OS_VERSION_WIN98:
                break;

            case OS_VERSION_WINNT:
                break;

            case OS_VERSION_WIN2K:
                if (!_Win32Service_ServiceManagerConnect ( ))
                {
                    _Win32Service_ServiceSetState ( 3 );
                    _Win32Service_ServiceManagerDysconnect ( );
                    return 0;
                }
                break;

        }

        return -1;
    }


    long    _Win32Service_DoApplicationStop ( void )
    {
        if (_Win32Service_GetVersionOS ( &_Win32_lVersionOS ))
            return -1;

        switch (_Win32_lVersionOS)
        {
            case 0:
                break;

            case OS_VERSION_WIN98:
                break;

            case OS_VERSION_WINNT:
                break;

            case OS_VERSION_WIN2K:
                if (!_Win32Service_ServiceManagerConnect ( ))
                {
                    _Win32Service_ServiceSetState ( 4 );
                    _Win32Service_ServiceManagerDysconnect ( );
                    return 0;
                }
                break;

        }

        return -1;
    }



    long    _Win32Service_SetNameDescProc ( char * pszServiceName, char * pszServiceDesc, void (*pServiceProc) ( void ) )
    {
        if (pszServiceName)
            strcpy ( _Win32_szServiceName, pszServiceName );

        if (pszServiceDesc)
            strcpy ( _Win32_szServiceDesc, pszServiceDesc );

        if (pServiceProc)
            _Win32_UserToDoMain = pServiceProc;
             
        return 0;
    }


    char * _Win32Service_GetTextError ( DWORD dwError )
    {
        char * pszMessage;
        static char szErrorMessage[2048];
        

        FormatMessageA ( 
            FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM,      //dwFlags
            NULL,                                                           //lpSource
            dwError,                                                        //dwMessageId
            MAKELANGID(LANG_NEUTRAL,SUBLANG_NEUTRAL),                       //dwLanguageId (Default language)
            (LPSTR)&pszMessage,                                             //lpBuffer
            0,                                                              //nSize
            NULL );                                                         //Arguments

        sprintf ( szErrorMessage, "Error %d : %s", dwError, pszMessage );
        
        LocalFree ( pszMessage );
        
        return szErrorMessage;
    }


    long _Win32Service_GetVersionOS ( long * plVersionOS )
    {
        OSVERSIONINFOEX_2   osviData;
        BOOL                bOsVersionInfoEx;
        long                lVersion;

        if (_Win32_lVersionOS!=-1)
            lVersion = _Win32_lVersionOS;
        else
        {
            memset ( &osviData, 0, sizeof(OSVERSIONINFOEX_2) );
            osviData.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX_2);
            bOsVersionInfoEx = GetVersionEx ( (OSVERSIONINFO *)&osviData );
            if (!bOsVersionInfoEx)
            {
                osviData.dwOSVersionInfoSize = sizeof ( OSVERSIONINFO );
                if (!GetVersionEx ( (OSVERSIONINFO *)&osviData) ) 
                    return -1;
            }

            switch (osviData.dwPlatformId)
            {
                case VER_PLATFORM_WIN32_WINDOWS:
                    if ((osviData.dwMajorVersion>=4)&&(osviData.dwMinorVersion>=10))
                        lVersion = OS_VERSION_WIN98;
                    else
                        lVersion = 0;
                    break;

                case VER_PLATFORM_WIN32_NT:
                    if (osviData.dwMajorVersion>=5) 
                        lVersion = OS_VERSION_WIN2K;
                    else
                        lVersion = OS_VERSION_WINNT;
                    break;

                default:
                    lVersion = 0;
                    break;
            }
        }

        _Win32_lVersionOS = lVersion;
        *plVersionOS = lVersion;

        return 0;
    }

    long _Win32Service_ServiceManagerConnect ( void )
    {
        _Win32_schManager = OpenSCManager ( NULL, NULL, SC_MANAGER_ALL_ACCESS );
        
        if (!_Win32_schManager)
        {
            COM_Printf ( "%s - Can't open SCManager", _Win32Service_GetTextError(GetLastError()) );
            return -1;
        }
        return 0;    
    }
    
    long _Win32Service_ServiceManagerDysconnect ( void )
    {
        if (!_Win32_schManager)
            return -1;
        
        CloseServiceHandle ( _Win32_schManager );
        _Win32_schManager = NULL;
        return 0; 
    }
    
    long _Win32Service_ServiceAppOpen ( void )
    {
        if (!_Win32_schManager)
            return -1;

        _Win32_schService = OpenServiceA ( 
            _Win32_schManager,              // SCM database 
            _Win32_szServiceName,           // name of service 
            SERVICE_ALL_ACCESS );           // need change config access     
        
        if (!_Win32_schService)
        {
            COM_Printf ( "%s - Can't open service", _Win32Service_GetTextError(GetLastError()) );
            return -1;
        }
        return 0;        
    }

    long _Win32Service_ServiceAppClose ( void )
    {
        if (!_Win32_schManager)
            return -1;
            
        if (!_Win32_schService)
            return -1;

        CloseServiceHandle ( _Win32_schService );
        _Win32_schService = NULL;
        return 0;        
    }
    
    long _Win32Service_ServiceGetState ( void )
    {
        if (!_Win32_schManager)
            return -1;
    
        return 0;
    }
    
    long _Win32Service_ServiceInstall ( char * pszBinaryPath )
    {
        SERVICE_DESCRIPTIONA sd;
        
        if (!_Win32_schManager)
            return -1;
    
        _Win32_schService = CreateServiceA ( 
            _Win32_schManager,              // SCM database 
            _Win32_szServiceName,           // name of service 
            _Win32_szServiceName,           // service name to display 
            SERVICE_ALL_ACCESS,             // desired access 
            SERVICE_WIN32_OWN_PROCESS,      // service type 
            SERVICE_AUTO_START,             // start type 
            SERVICE_ERROR_NORMAL,           // error control type 
            pszBinaryPath,                  // path to service's binary 
            NULL,                           // no load ordering group 
            NULL,                           // no tag identifier 
            NULL,                           // no dependencies 
            NULL,                           // LocalSystem account 
            NULL );                         // no password 

        if (!_Win32_schService)
        {
            COM_Printf ( "%s - Can't create service", _Win32Service_GetTextError(GetLastError()) );
            return -1;
        }

    
        sd.lpDescription = _Win32_szServiceDesc;
        if (!ChangeServiceConfig2A (
            _Win32_schService,          
            SERVICE_CONFIG_DESCRIPTION,
            &sd ) )
        {
            COM_Printf ( "%s - Can't add a service description", _Win32Service_GetTextError(GetLastError()) );
        }

        CloseServiceHandle ( _Win32_schService );
        _Win32_schService = NULL;
        return 0;
    }
    
    long _Win32Service_ServiceUninstall ( void )
    {
        long lRetour;
        
        if (!_Win32_schManager)
            return -1;
    
        if (_Win32Service_ServiceAppOpen())
            return -1;

        if (!DeleteService ( _Win32_schService ) )
        {
            COM_Printf ( "%s - Can't uninstall service", _Win32Service_GetTextError(GetLastError()) );
            lRetour = -1;
        }
        else lRetour = 0;
        
        _Win32Service_ServiceAppClose ( );
        
        return lRetour;
    }
    
    long _Win32Service_ServiceSetState ( long lStateNeed )
    {
        long lRetour;
        DWORD dwStartType;
        long lModeFunction;
        
        if (!_Win32_schManager)
            return -1;
    
        if (_Win32Service_ServiceAppOpen())
            return -1;
            
        lModeFunction = 0;
        switch (lStateNeed)
        {
            case 0:
                dwStartType = SERVICE_DISABLED;
                break;
                
            case 1:
                dwStartType = SERVICE_DEMAND_START;
                break;          
                
            case 2:
                dwStartType = SERVICE_AUTO_START;
                break;
                
            case 3: //start                
                lModeFunction = 1;
                break;
                
            case 4: //stop
                lModeFunction = 2;
                break;
        }
                
                
        switch (lModeFunction)
        {
            case 0:        
                if (!ChangeServiceConfigA (
                    _Win32_schService,
                    SERVICE_NO_CHANGE,
                    dwStartType,
                    SERVICE_NO_CHANGE,
                    NULL,
                    NULL,
                    NULL,
                    NULL,
                    NULL,
                    NULL,
                    NULL ) )
                {
                    COM_Printf ( "%s - Can't set a new service state", _Win32Service_GetTextError(GetLastError()) );
                    lRetour = -1;
                }
                else lRetour = 0;
                break;
                
            case 1:
                if (_Win32Service_ServiceStart()) 
                {
                    COM_Printf ( "Error : Can't start service" );
                    lRetour = -1;
                }
                else lRetour = 0;
                break;
                
            case 2:
                if (_Win32Service_ServiceStop())
                {
                    COM_Printf ( "Error : Can't stop service" );
                    lRetour = -1;
                }
                else lRetour = 0;
                break;
        }

        _Win32Service_ServiceAppClose ( );
        
        return lRetour;
    }

    long _Win32Service_ServiceStart ( void )
    {
        SERVICE_STATUS ssStatus;
        DWORD dwOldCheckPoint; 
        DWORD dwStartTickCount;
        DWORD dwWaitTime;
        
        //Attempt to start the service.
        if (!StartService(
            _Win32_schService,  // handle to service 
            0,           // number of arguments 
            NULL) )      // no arguments 
        {
            COM_Printf ( "%s - Can't start service", _Win32Service_GetTextError(GetLastError()) );
            return -1; 
        }
        else COM_Printf ( "Service start pending..." );


        //Check the status until the service is no longer start pending. 
        if (!QueryServiceStatus(_Win32_schService,&ssStatus))
        {
            return 0; 
        }


        //Save the tick count and initial checkpoint.
        dwStartTickCount = GetTickCount();
        dwOldCheckPoint = ssStatus.dwCheckPoint;

        while (ssStatus.dwCurrentState == SERVICE_START_PENDING) 
        { 
            //Do not wait longer than the wait hint. A good interval is 
            //one-tenth the wait hint, but no less than 1 second and no 
            //more than 10 seconds. 
            dwWaitTime = ssStatus.dwWaitHint / 10;

            if( dwWaitTime < 1000 )         dwWaitTime = 1000;
            else if ( dwWaitTime > 10000 )  dwWaitTime = 10000;

            Sleep ( dwWaitTime );

            //Check the status again. 
            if (!QueryServiceStatus(_Win32_schService,&ssStatus))
                break; 
     
            if ( ssStatus.dwCheckPoint > dwOldCheckPoint )
            {
                //The service is making progress.
                dwStartTickCount = GetTickCount();
                dwOldCheckPoint = ssStatus.dwCheckPoint;
            }
            else
            {
                if(GetTickCount()-dwStartTickCount > ssStatus.dwWaitHint)
                {
                    // No progress made within the wait hint.
                    break;
                }
            }
        } 

        // Determine whether the service is running.
        if (ssStatus.dwCurrentState == SERVICE_RUNNING) 
        {
            COM_Printf ( "Service started successfully." ); 
        }
        else 
        { 
            COM_Printf ( "Error : Service not started.");
            return -1;
        } 
        return 0;
    }    
    
    long _Win32Service_ServiceStop ( void )
    {
        //SERVICE_STATUS_PROCESS ssp;
        SERVICE_STATUS ssStatus;
        DWORD dwStartTime = GetTickCount();
        //DWORD dwBytesNeeded;
        DWORD dwTimeout = 30000; // 30-second time-out

        //Make sure the service is not already stopped.
        if (!QueryServiceStatus(_Win32_schService,&ssStatus))
        {
            COM_Printf("QueryServiceStatusEx failed (%d)\n", GetLastError()); 
            return -1;
        }

        if ( ssStatus.dwCurrentState == SERVICE_STOPPED )
        {
            COM_Printf("Service is already stopped.\n");
            return 0;
        }

        //If a stop is pending, wait for it.
        while ( ssStatus.dwCurrentState == SERVICE_STOP_PENDING ) 
        {
            COM_Printf("Service stop pending...\n");
            Sleep( ssStatus.dwWaitHint );

            if (!QueryServiceStatus(_Win32_schService,&ssStatus))
            {
                COM_Printf("QueryServiceStatusEx failed (%d)\n", GetLastError()); 
                return -1;
            }

            if ( ssStatus.dwCurrentState == SERVICE_STOPPED )
            {
                COM_Printf("Service stopped successfully.\n");
                return 0;
            }

            if ( GetTickCount() - dwStartTime > dwTimeout )
            {
                COM_Printf("Service stop timed out.\n");
                return -1;
            }
        }

        //If the service is running, dependencies must be stopped first.
        //StopDependentServices();

        //Send a stop code to the service.

        if ( !ControlService( 
                _Win32_schService, 
                SERVICE_CONTROL_STOP, 
                (LPSERVICE_STATUS)&ssStatus ) )
        {
            COM_Printf( "ControlService failed (%d)\n", GetLastError() );
            return -1;
        }

        //Wait for the service to stop.
        while ( ssStatus.dwCurrentState != SERVICE_STOPPED ) 
        {
            Sleep( ssStatus.dwWaitHint );

            if (!QueryServiceStatus(_Win32_schService,&ssStatus))
            {
                COM_Printf( "QueryServiceStatusEx failed (%d)\n", GetLastError() );
                return -1;
            }

            if ( ssStatus.dwCurrentState == SERVICE_STOPPED )
                break;

            if ( GetTickCount() - dwStartTime > dwTimeout )
            {
                COM_Printf( "Wait timed out\n" );
                return -1;
            }
        }
        COM_Printf("Service stopped successfully\n");

        return 0;    
    }    


#endif
